Package: InteractionSet
Version: 1.41.0
Date: 2025-07-15
Title: Base Classes for Storing Genomic Interaction Data
Authors@R: c(person("Aaron", "Lun", role = c("aut", "cre"), email = "infinite.monkeys.with.keyboards@gmail.com"),
        person("Malcolm", "Perry", role = "aut", email = "mgperry32@gmail.com"),
        person("Elizabeth", "Ing-Simmons", role = "aut"))
Depends: GenomicRanges, SummarizedExperiment
Imports: methods, Matrix, Rcpp, BiocGenerics, S4Vectors (>= 0.27.12),
        IRanges, Seqinfo
Suggests: testthat, knitr, rmarkdown, BiocStyle
LinkingTo: Rcpp
biocViews: Infrastructure, DataRepresentation, Software, HiC
Description: Provides the GInteractions, InteractionSet and ContactMatrix
        objects and associated methods for storing and manipulating genomic
        interaction data from Hi-C and ChIA-PET experiments.
License: GPL-3
NeedsCompilation: yes
SystemRequirements: C++11
VignetteBuilder: knitr
Collate: AllGenerics.R AllClasses.R ContactMatrix-methods.R
        GInteractions-methods.R InteractionSet-methods.R
        GRanges-methods.R getset.R swapAnchors.R pairs.R conversion.R
        linearize.R boundingBox.R distances.R overlaps.R linkOverlaps.R
        matching.R updateObject.R
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/InteractionSet
git_branch: devel
git_last_commit: 275c24c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 23:19:23 UTC; biocbuild
Author: Aaron Lun [aut, cre],
  Malcolm Perry [aut],
  Elizabeth Ing-Simmons [aut]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>

useDynLib(InteractionSet, .registration=TRUE, .fixes="cxx_")

# Exporting the InteractionSet class.
exportClasses(
    GInteractions, StrictGInteractions, ReverseStrictGInteractions,
    InteractionSet, 
    ContactMatrix
)

exportMethods(
    GInteractions,
    anchors, anchorIds, regions, reduceRegions, swapAnchors,
    "regions<-", "replaceRegions<-",  "appendRegions<-", "anchorIds<-",
    seqinfo, "seqinfo<-",
    pairs, 
    findOverlaps, countOverlaps, overlapsAny,
    InteractionSet, 
    interactions,
    "interactions<-",
    pairdist, intrachr, linearize, linkOverlaps, boundingBox,
    ContactMatrix, 
    "as.matrix<-",
    inflate, deflate,
    updateObject
)

export(
    makeGInteractionsFromGRangesPairs
)

# Requires several imports.
import(methods, BiocGenerics, S4Vectors, IRanges,
    Seqinfo, GenomicRanges, SummarizedExperiment)

importFrom("Matrix", "Matrix", "sparseMatrix")

importFrom("Rcpp", sourceCpp)

# S4 classes for genomic interaction data

|Environment|Status|
|---|---|
|BioC-release|[![Release OK](https://bioconductor.org/shields/build/release/bioc/InteractionSet.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/InteractionSet/)|
|BioC-devel|[![Devel OK](https://bioconductor.org/shields/build/devel/bioc/InteractionSet.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/InteractionSet/)|

The **InteractionSet** package implements a variety of S4 classes for storing and manipulating genomic interaction data from Hi-C and ChIA-PET experiments.
Check out the user's guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/InteractionSet.html) for more details.

###############################################################
# Setting validity and show methods.

.check_inputs <- function(anchor1, anchor2, regions, same.length=TRUE) {
    if (!all(is.finite(anchor1)) || !all(is.finite(anchor2))) { 
        return("all anchor indices must be finite integers")
    }
    if (!all(anchor1 >= 1L) || !all(anchor2 >= 1L)) {
        return('all anchor indices must be positive integers')
    } 
    nregs <- length(regions)
    if ( !all(anchor1 <= nregs) || !all(anchor2 <= nregs)) {
        return("all anchor indices must refer to entries in 'regions'")
    } 
    if (same.length && length(anchor1)!=length(anchor2)) { 
        return("first and second anchor vectors have different lengths")
    }
    return(TRUE)
}

setValidity2("GInteractions", function(object) {
    if (is.unsorted(regions(object))) { # Don't move into .check_inputs, as resorting comes after checking validity in various methods.
        return("'regions' should be sorted")
    }
    msg <- .check_inputs(anchor1(object), anchor2(object), regions(object))
    if (is.character(msg)) { return(msg) }

    ### Length of anchors versus object is automatically checked by 'parallel_slot_names.'

    if (!is.null(names(object))) {
        if (length(names(object))!=length(object)) {
            stop("'NAMES' must be NULL or have length equal to that of the object")
        }
    }

    return(TRUE)
})

setMethod("parallel_slot_names", "GInteractions", function(x) {
    c("anchor1", "anchor2", "NAMES", callNextMethod())         
})

# For coercion to an environment:
setMethod("parallelVectorNames", "GInteractions", function(x) {
    c("anchor1", "anchor2", "regions", "names")
})

scat <- function(fmt, vals=character(), exdent=2, ...) {
    vals <- ifelse(nzchar(vals), vals, "''")
    lbls <- paste(S4Vectors:::selectSome(vals), collapse=" ")
    txt <- sprintf(fmt, length(vals), lbls)
    cat(strwrap(txt, exdent=exdent, ...), sep="\n")
}

setMethod("show", "GInteractions", function(object){
    showGInteractions(object, margin="  ", print.seqinfo=TRUE, print.classinfo=TRUE)
})

showGInteractions <- function(x, margin="", print.seqinfo=FALSE, print.classinfo=FALSE) {
    lx <- length(x)
    nr <- length(regions(x))
    nc <- .safeNMcols(x)
    cat(class(x), " object with ",
        lx, " ", ifelse(lx == 1L, "interaction", "interactions"), " and ",
        nc, " metadata ", ifelse(nc == 1L, "column", "columns"),
        ":\n", sep="")
    out <- makePrettyMatrixForCompactPrinting(x, .makeNakedMatFromGInteractions)
   
    # Ripped from GenomicRanges:::showGenomicRanges (with some mods).
    if (print.classinfo) { 
        .COL2CLASS <- c(seqnames1 = "Rle", ranges1 = "IRanges", "   "="", seqnames2="Rle", ranges2="IRanges")
        extraColumnNames <- GenomicRanges:::extraColumnSlotNames(x)
        .COL2CLASS <- c(.COL2CLASS, getSlots(class(x))[extraColumnNames])
        classinfo <- makeClassinfoRowForCompactPrinting(x, .COL2CLASS)
        classinfo[,"   "] <- ""
        stopifnot(identical(colnames(classinfo), colnames(out)))
        out <- rbind(classinfo, out)
    }

    if (nrow(out) != 0L) {
        rownames(out) <- paste0(margin, rownames(out))
    }
    print(out, quote=FALSE, right=TRUE, max=length(out))
    if (print.seqinfo) {
        cat(margin, "-------\n", sep="")
        ncr <- .safeNMcols(regions(x))
        cat(margin, "regions: ", nr, " ranges and ", ncr, " metadata ", ifelse(ncr==1L, "column", "columns"), "\n", sep="")
        cat(margin, "seqinfo: ", summary(seqinfo(x)), "\n", sep="")
    }
}

.makeNakedMatFromGInteractions <- function(x) {
    lx <- length(x)
    nc <- .safeNMcols(x)
    ans <- cbind(.pasteAnchor(anchors(x, type="first"), append="1"),
                 "   "=rep.int("---", lx),
                 .pasteAnchor(anchors(x, type="second"), append="2"))
    if (nc > 0L) {
        tmp <- do.call(data.frame, c(lapply(mcols(x), showAsCell), list(check.names=FALSE)))
        ans <- cbind(ans, `|`=rep.int("|", lx), as.matrix(tmp))
    }
    ans
}

.safeNMcols <- function(x) {
    nc <- ncol(mcols(x))
    if (is.null(nc)) { nc <- 0L }
    return(nc)
}

.pasteAnchor <- function(x, append) {
    out <- cbind(as.character(seqnames(x)), showAsCell(ranges(x)))
    colnames(out) <- paste0(c("seqnames", "ranges"), append)
    out
}

###############################################################
# Ordered equivalents, where swap state is enforced.

setValidity2("StrictGInteractions", function(object) {
    if (any(anchor1(object) > anchor2(object))) { 
        stop("'anchor1' cannot be greater than 'anchor2'")
    }
    return(TRUE)
})

setValidity2("ReverseStrictGInteractions", function(object) {
    if (any(anchor1(object) < anchor2(object))) { 
        stop("'anchor1' cannot be less than 'anchor2'")
    }
    return(TRUE)
})

###############################################################
# Constructors

.enforce_order <- function(anchor1, anchor2) {
    swap <- anchor2 < anchor1
    if (any(swap)) { 
        temp <- anchor1[swap]
        anchor1[swap] <- anchor2[swap]
        anchor2[swap] <- temp
    }
    return(list(anchor1=anchor1, anchor2=anchor2))
}

.resort_regions <- function(anchor1, anchor2, regions) {
    if (is.unsorted(regions)) { 
        o <- order(regions)
        new.pos <- seq_along(o)
        new.pos[o] <- new.pos
        anchor1 <- new.pos[anchor1]
        anchor2 <- new.pos[anchor2]
        regions <- regions[o]
    }
    return(list(anchor1=anchor1, anchor2=anchor2, regions=regions)) 
}

.new_GInteractions <- function(anchor1, anchor2, regions, metadata, mode=c("normal", "strict", "reverse")) {
    mode <- match.arg(mode)
    elementMetadata <- make_zero_col_DFrame(length(anchor1))

    # Checking odds and ends.
    anchor1 <- as.integer(anchor1)
    anchor2 <- as.integer(anchor2)
    msg <- .check_inputs(anchor1, anchor2, regions)
    if (is.character(msg)) { stop(msg) }

    out <- .resort_regions(anchor1, anchor2, regions)
    anchor1 <- out$anchor1
    anchor2 <- out$anchor2
    regions <- out$regions

    # Checking what the mode should be.
    if (mode=="normal") {
        cls <- "GInteractions"
    } else {
        if (mode=="strict") {
            cls <- "StrictGInteractions"
            out <- .enforce_order(anchor1, anchor2)
        } else {
            cls <- "ReverseStrictGInteractions"
            out <- rev(.enforce_order(anchor1, anchor2))
        }
        anchor1 <- out[[1]]
        anchor2 <- out[[2]]
    }

    new(cls, 
        anchor1=anchor1,
        anchor2=anchor2,
        regions=regions,
        elementMetadata=elementMetadata,
        metadata=as.list(metadata))
}

setMethod("GInteractions", c("numeric", "numeric", "GRanges"), 
    function(anchor1, anchor2, regions, metadata=list(), mode="normal", ...) {
        out <- .new_GInteractions(anchor1, anchor2, regions=regions, metadata=metadata, mode=mode)
        extraCols <- DataFrame(...)
        if (ncol(extraCols) == 0L) {
            extraCols <- make_zero_col_DFrame(length(anchor1))
        }
        mcols(out) <- extraCols
        out
})

.collate_GRanges <- function(...) {
    incoming <- list(...)
    obj.dex <- rep(factor(seq_along(incoming)), lengths(incoming))
    combined <- do.call(c, incoming)
    
    # Sorting and re-indexing.
    o <- order(combined)
    refdex <- integer(length(o))
    refdex[o] <- seq_along(combined)
    combined <- combined[o]

    # Removing duplicates and re-indexing.
    is.first <- !duplicated(combined)
    new.pos <- cumsum(is.first)
    combined <- combined[is.first]
    refdex <- new.pos[refdex]    
    return(list(indices=split(refdex, obj.dex), ranges=combined))
}

setMethod("GInteractions", c("GRanges", "GRanges", "GenomicRanges_OR_missing"),
    function(anchor1, anchor2, regions, metadata=list(), mode="normal", ...) {
        # Stripping metadata and putting it somewhere else.
        mcol1 <- mcols(anchor1)
        mcols(anchor1) <- NULL
        colnames(mcol1) <- sprintf("anchor1.%s", colnames(mcol1))
        mcol2 <- mcols(anchor2)
        mcols(anchor2) <- NULL
        colnames(mcol2) <- sprintf("anchor2.%s", colnames(mcol2))

        # Additional Interaction-specific metadata
        extraCols <- DataFrame(...)
        if (ncol(extraCols) == 0L) {
            extraCols <- make_zero_col_DFrame(length(anchor1))
        } 

        if (missing(regions)) {
            # Making unique regions to save space (metadata is ignored)
            collated <- .collate_GRanges(anchor1, anchor2)
            regions <- collated$ranges
            anchor1 <- collated$indices[[1]]
            anchor2 <- collated$indices[[2]]
        } else {
            anchor1 <- match(anchor1, regions)
            anchor2 <- match(anchor2, regions)
            if (any(is.na(anchor1)) || any(is.na(anchor2))) {
                stop("anchor regions missing in specified 'regions'")
            }
        }

        out <- .new_GInteractions(anchor1=anchor1, anchor2=anchor2, 
                                  regions=regions, metadata=metadata, mode=mode)
        mcols(out) <- cbind(extraCols, mcol1, mcol2)
        out
   }
)

setMethod("GInteractions", c("missing", "missing", "GenomicRanges_OR_missing"),
    function(anchor1, anchor2, regions, metadata=list(), mode="normal", ...) {
        if (missing(regions)) { regions <- GRanges() }
        out <- .new_GInteractions(integer(0), integer(0), regions, metadata, mode=mode)
        mcols(out) <- DataFrame(...)[integer(0),,drop=FALSE]
        out
})

###############################################################
# Concatenation

setMethod("bindROWS", "GInteractions",
  function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE) {
    objects <- S4Vectors:::prepare_objects_to_bind(x, objects)
    all.objects <- c(list(x), objects)

    # Taken from XVector:::concatenate_XVectorList_objects. Note that the 
    # object may not be valid at this point due to anchor1/anchor2
    # reordering for the Strict subclasses, hence check=FALSE.
    ans <- callNextMethod(x, objects, use.names=use.names,
                          ignore.mcols=ignore.mcols, check=FALSE)

    # Coercing everyone to the same region set.
    all.regions <- lapply(all.objects, FUN=regions)
    all.anchor1 <- lapply(all.objects, FUN=anchor1)
    all.anchor2 <- lapply(all.objects, FUN=anchor2)
    unified <- .coerce_to_union(all.regions, all.anchor1, all.anchor2)
    unchecked_regions(ans) <- unified$region
    unchecked_anchor1(ans) <- unlist(unified$anchor1)
    unchecked_anchor2(ans) <- unlist(unified$anchor2)

    # Coerce to the same strictness, if different inputs were supplied.
    # We need to decide whether or not to trigger the validObject() in the new() inside as().
    old_val <- S4Vectors:::disableValidity() 
    if (old_val!=check) {
        on.exit(S4Vectors:::disableValidity(old_val))
        S4Vectors:::disableValidity(check)
    }
    as(ans, class(x))
})

.coerce_to_union <- function(all.regions, all.anchor1, all.anchor2) {
    issame <- TRUE
    if (length(all.regions) > 1L) { 
        if (length(unique(lengths(all.regions)))>1L) {
            issame <- FALSE
        } else {
            ref <- all.regions[[1]]
            for (alt in 2:length(all.regions)) { 
                if (any(all.regions[[alt]]!=ref)) { 
                    issame <- FALSE
                    break 
                }
            }
        }
    } 

    if (!issame) { 
        collated <- do.call(.collate_GRanges, all.regions)
        for (i in seq_along(all.regions)) {
            all.anchor1[[i]] <- collated$indices[[i]][all.anchor1[[i]]]
            all.anchor2[[i]] <- collated$indices[[i]][all.anchor2[[i]]]
        }
        out.range <- collated$ranges
    } else {
        if (length(all.regions)) { 
            out.range <- all.regions[[1]]
        } else {
            out.range <- GRanges()
        }
    }
    return(list(region=out.range, anchor1=all.anchor1, anchor2=all.anchor2))
}

###############################################################
# Other methods

setMethod("order", "GInteractions", function(..., na.last=TRUE, decreasing=FALSE) {
    all.ids <- unlist(lapply(list(...), anchors, id=TRUE), recursive=FALSE)
    do.call(order, c(all.ids, list(na.last=na.last, decreasing=decreasing)))
})

setMethod("duplicated", "GInteractions", function(x, incomparables=FALSE, fromLast=FALSE, ...) 
# Stable sort required here: first entry in 'x' is always non-duplicate if fromLast=FALSE,
# and last entry is non-duplicate if fromLast=TRUE.
{
    if (!length(x)) { return(logical(0)) }
    a1 <- anchors(x, id=TRUE, type="first")
    a2 <- anchors(x, id=TRUE, type="second")
    o <- order(a1, a2) 
    if (fromLast) { o <- rev(o) }
    is.dup <- c(FALSE, diff(a1[o])==0L & diff(a2[o])==0L)
    is.dup[o] <- is.dup
    return(is.dup)
})

setMethod("as.data.frame", "GInteractions", function (x, row.names=NULL, optional=FALSE, ...) {
    all1 <- anchors(x, type="first", id=TRUE)
    all2 <- anchors(x, type="second", id=TRUE)
    used <- logical(length(regions(x)))
    used[all1] <- TRUE
    used[all2] <- TRUE
    new.index <- cumsum(used) # Accelerate by only converting what we need.

    regs.dframe <- as.data.frame(unname(regions(x)[used]), optional=optional, ...)
    a1.dframe <- regs.dframe[new.index[all1],]
    colnames(a1.dframe) <- paste0(colnames(a1.dframe), "1")
    a2.dframe <- regs.dframe[new.index[all2],]
    colnames(a2.dframe) <- paste0(colnames(a2.dframe), "2")
    if (missing(row.names)) { row.names <- names(x) }
    data.frame(a1.dframe, a2.dframe, mcols(x), row.names=row.names)
})

###############################################################
# End

# Defines the match function.

#' @importFrom BiocGenerics match
#' @importFrom S4Vectors Hits
#' @importMethodsFrom S4Vectors match
setMethod("match", c("GInteractions", "GInteractions"), 
    function(x, table, nomatch = NA_integer_, incomparables = NULL, ...) {
        .strict_check(x, table)

        rx <- regions(x)
        rtab <- regions(table)
        if (length(rx)!=length(rtab) || any(rx!=rtab)) { 
            # Diverting to findOverlaps if the regions are not equal.
            return(findOverlaps(x, table, type="equal", select="first", use.region="same"))
        }

        # Using the Hits method, for convenience.
        nR <- length(rx)
        a1 <- Hits(anchor1(x), anchor2(x), nR, nR, sort.by.query=FALSE)
        a2 <- Hits(anchor1(table), anchor2(table), nR, nR, sort.by.query=FALSE)
        match(a1, a2, nomatch=nomatch, incomparables=incomparables, ...)
    }
)

setMethod("match", c("GInteractions", "InteractionSet"), 
    function(x, table, nomatch = NA_integer_, incomparables = NULL, ...) {
        match(x, interactions(table), nomatch=nomatch, incomparables=incomparables, ...)
    }
)

setMethod("match", c("InteractionSet", "GInteractions"), 
    function(x, table, nomatch = NA_integer_, incomparables = NULL, ...) {
        match(interactions(x), table, nomatch=nomatch, incomparables=incomparables, ...)
    }
)

setMethod("match", c("InteractionSet", "InteractionSet"), 
    function(x, table, nomatch = NA_integer_, incomparables = NULL, ...) {
        match(interactions(x), table, nomatch=nomatch, incomparables=incomparables, ...)
    }
)

# Defining the compare function (only for two GInteractions, it wouldn't make sense otherwise).

setMethod("pcompare",  c("GInteractions", "GInteractions"), function(x, y) { 
    .strict_check(x, y)
    a1.x <- anchor1(x)
    a2.x <- anchor2(x)
    a1.y <- anchor1(y)
    a2.y <- anchor2(y)

    rx <- regions(x)
    ry <- regions(y)
    if (length(rx)!=length(ry) || any(rx!=ry)) { # Coercing them to the same system, if they're not equal.
        collated <- .collate_GRanges(rx, ry)
        a1.x <- collated$indices[[1]][a1.x]
        a2.x <- collated$indices[[1]][a2.x]
        a1.y <- collated$indices[[2]][a1.y]
        a2.y <- collated$indices[[2]][a2.y]        
    }

    output1 <- a1.x - a1.y
    output2 <- a2.x - a2.y # explicitly formed for proper recycling.
    tied1 <- output1==0L
    output1[tied1] <- output2[tied1]
    return(output1)
})

.strict_check <- function(x, y) {
    if (is(x, "StrictGInteractions")!=is(y, "StrictGInteractions") 
        || is(x, "ReverseStrictGInteractions")!=is(y, "ReverseStrictGInteractions")) { 
        warning("comparison between GInteractions objects of different strictness")
    }
}

# This defines the linkOverlaps method. The idea is to use interactions
# to link any two sets of regions together; to figure out which ones are
# linked, and to identify the interactions linking them.

.linkOverlap <- function(query, subject1, subject2, ..., ignore.strand=TRUE, use.region="both") {
    a1 <- anchors(query, id=TRUE, type="first")
    a2 <- anchors(query, id=TRUE, type="second")
    nregs <- length(regions(query))

    olap1 <- .fast_overlap(query, subject1, ..., ignore.strand=ignore.strand, gi.is.query=TRUE)
    bounds1 <- .get_olap_bounds(olap1$gi.dex, nregs)
    nregs1 <- length(subject1)
    if (missing(subject2)) { 
        olap2 <- olap1
        bounds2 <- bounds1
        is.same <- TRUE
        nregs2 <- nregs1
    } else {
        olap2 <- .fast_overlap(query, subject2, ..., ignore.strand=ignore.strand, gi.is.query=TRUE)
        bounds2 <- .get_olap_bounds(olap2$gi.dex, nregs)
        is.same <- FALSE
        nregs2 <- length(subject2)
    }

    out <- .Call(cxx_expand_pair_links, a1 - 1L, a2 - 1L, 
                 bounds1$first - 1L, bounds1$last, olap1$ranges.dex - 1L, nregs1,
                 bounds2$first - 1L, bounds2$last, olap2$ranges.dex - 1L, nregs2,
                 is.same, .decode_region_mode(use.region, c("both", "same", "reverse")))
    return(DataFrame(query=out[[1]]+1L, subject1=out[[2]]+1L, subject2=out[[3]]+1L))    
}

setMethod("linkOverlaps", c("GInteractions", "Vector", "Vector"), .linkOverlap)
setMethod("linkOverlaps", c("GInteractions", "Vector", "missing"), .linkOverlap)
setMethod("linkOverlaps", c("InteractionSet", "Vector", "Vector"), .linkOverlap)
setMethod("linkOverlaps", c("InteractionSet", "Vector", "missing"), .linkOverlap)

# Defining another function for Hits inputs. This is more efficient in cases
# where overlaps between 'regions(query)' and the subjects have already been 
# computed, and you want to identify linkages based on pairing of regions,
# e.g., if the intervals being overlapped are constant across calls.

.linkOverlap_Hits <- function(query, subject1, subject2, use.region="both") {
    a1 <- anchors(query, id=TRUE, type="first")
    a2 <- anchors(query, id=TRUE, type="second")
    nregs <- length(regions(query))
    if (nLnode(subject1)!=nregs) {
        stop("'nLnode' of 'subject1' should be equal to 'length(region(query))'")
    }
 
    bounds1 <- .get_olap_bounds(queryHits(subject1), nregs)
    nregs1 <- nRnode(subject1)
    if (missing(subject2)) { 
        subject2 <- subject1
        bounds2 <- bounds1
        is.same <- TRUE
        nregs2 <- nregs1
    } else {
        if (nLnode(subject2)!=nregs) {
            stop("'nLnode' of 'subject2' should be equal to 'length(region(query))'")
        }
        bounds2 <- .get_olap_bounds(queryHits(subject2), nregs)
        is.same <- FALSE
        nregs2 <- nRnode(subject2)
    }

    out <- .Call(cxx_expand_pair_links, a1 - 1L, a2 - 1L, 
                 bounds1$first - 1L, bounds1$last, subjectHits(subject1) - 1L, nregs1,
                 bounds2$first - 1L, bounds2$last, subjectHits(subject2) - 1L, nregs2,
                 is.same, .decode_region_mode(use.region, c("both", "same", "reverse")))
    return(DataFrame(query=out[[1]]+1L, subject1=out[[2]]+1L, subject2=out[[3]]+1L))    
}

setMethod("linkOverlaps", c("GInteractions", "Hits", "Hits"), .linkOverlap_Hits)
setMethod("linkOverlaps", c("GInteractions", "Hits", "missing"), .linkOverlap_Hits)
setMethod("linkOverlaps", c("InteractionSet", "Hits", "Hits"), .linkOverlap_Hits)
setMethod("linkOverlaps", c("InteractionSet", "Hits", "missing"), .linkOverlap_Hits)

#######################################
# Internal functions for use in GRanges methods for paired regions.

.generate_regions <- function(x, FUN, args, other.args) {
    a1 <- anchor1(x)
    a2 <- anchor2(x)
    N1 <- length(a1)
    N2 <- length(a2)

    # Expanding all arguments to the requested length.
    arg1 <- arg2 <- args
    for (arg in names(args)) { 
        current <- args[[arg]]

        if (is.list(current)) { 
            c1 <- current[[1]]
            c2 <- current[[2]]            
        } else {
            c1 <- c2 <- current
        }

        c1 <- .expand_to_length(c1, N1)
        c2 <- .expand_to_length(c2, N2)
        arg1[[arg]] <- c1
        arg2[[arg]] <- c2
    }

    # Minimizing expansion by identifying unique modifications for each anchor region.
    reg <- regions(x)
    reg1 <- .apply_unique_mods(reg, a1, FUN, arg1, other.args)
    reg2 <- .apply_unique_mods(reg, a2, FUN, arg2, other.args)
    mod.reg <- .collate_GRanges(reg1, reg2)

    # Fiddling with the input/output ranges.
    unchecked_regions(x) <- mod.reg$ranges
    unchecked_anchor1(x) <- mod.reg$indices[[1]][a1]
    unchecked_anchor2(x) <- mod.reg$indices[[2]][a2]
    return(x)
}

.expand_to_length <- function(v, N) {
    tmp <- rep(v, length.out=N)
    tmp[] <- v # to trigger warning upon recycling a non-multiple vector.
    return(tmp)
}

.apply_unique_mods <- function(regions, index, FUN, args, other.args) {
    to.order <- c(list(index), args)
    o <- do.call(order, to.order)

    # Identifying unique modifications.
    mod <- integer(0) 
    if (length(o)) { 
        is.uniq <- FALSE
        for (element in to.order) {
            element <- element[o]
            is.uniq <- is.uniq | (element[-1]!=element[-length(o)])
        }
        mod <- o[c(TRUE, is.uniq)]
    }

    # Applying the modifications.
    mod.args <- lapply(args, "[", mod)
    to.mod <- index[mod]
    regions[to.mod] <- do.call(FUN, c(list(x=regions[to.mod]), mod.args, other.args))
    return(regions)
}

#######################################

for (siglist in c("GInteractions", "ContactMatrix")) {
    setMethod("trim", siglist, function(x, use.names=TRUE) {
        regions(x) <- trim(regions(x), use.names=use.names)
        return(x)
    })

    setMethod("resize", siglist, function(x, width, fix="start", use.names=TRUE, ...) {
        .generate_regions(x, FUN=resize, args=list(width=width, fix=fix), other.args=list(use.names=use.names, ...))
    })

    setMethod("narrow", siglist, function(x, start=NA, end=NA, width=NA, use.names=TRUE) {
        .generate_regions(x, FUN=narrow, args=list(start=start, end=end, width=width), other.args=list(use.names=use.names))
    })  

    setMethod("shift", siglist, function(x, shift=0L, use.names=TRUE) {
        .generate_regions(x, FUN=IRanges::shift, args=list(shift=shift), other.args=list(use.names=use.names))
    })

    setMethod("flank", siglist, function(x, width, start=TRUE, both=FALSE, use.names=TRUE, ignore.strand=FALSE) {
        .generate_regions(x, FUN=flank, args=list(width=width, start=start), 
                          other.args=list(use.names=use.names, ignore.strand=ignore.strand))
    })
}

setMethod("width", "GInteractions", function(x) {
    w <- width(regions(x))          
    list(first=w[anchor1(x)], second=w[anchor2(x)])
})

setMethod("width", "ContactMatrix", function(x) {
    w <- width(regions(x))          
    list(row=w[anchor1(x)], column=w[anchor2(x)])
})

# Same methods for the interaction set.

setMethod("trim", "InteractionSet", function(x, use.names=TRUE) { 
    interactions(x) <- trim(interactions(x), use.names=use.names) 
    return(x)
})

setMethod("resize", "InteractionSet", function(x, width, fix="start", use.names=TRUE, ...) {
    interactions(x) <- resize(interactions(x), width=width, fix=fix, use.names=use.names, ...)
    return(x)
})

setMethod("narrow", "InteractionSet", function(x, start=NA, end=NA, width=NA, use.names=TRUE) {
    interactions(x) <- narrow(interactions(x), start=start, end=end, width=width, use.names=use.names)
    return(x)        
})  

setMethod("shift", "InteractionSet", function(x, shift=0L, use.names=TRUE) {
    interactions(x) <- shift(interactions(x), shift=shift, use.names=use.names)
    return(x)
})

setMethod("flank", "InteractionSet", function(x, width, start=TRUE, both=FALSE, use.names=TRUE, ignore.strand=FALSE) {
    interactions(x) <- flank(interactions(x), width=width, start=start, both=both, 
                             use.names=use.names, ignore.strand=ignore.strand)
    return(x)
})

setMethod("width", "InteractionSet", function(x) { width(interactions(x)) })

.get_dist_output <- function(regs, ai1, ai2, type) {
    type <- match.arg(type, c("mid", "gap", "span", "diag", "intra"))
    chr <- as.character(seqnames(regs))

    # To get sensible distances
    swapped <- .enforce_order(ai1, ai2) 
    larger <- swapped[[2]]
    smaller <- swapped[[1]]

    # Protection when all inter's.
    is.same <- chr[larger]==chr[smaller]
    if (type=="intra") { return(is.same) }
    output <- rep(as.integer(NA), length(larger))
    if (!any(is.same)) { return(output) }

    st <- start(regs)
    en <- end(regs)
    larger <- larger[is.same]
    smaller <- smaller[is.same]
    all.as <- st[larger]
    all.ae <- en[larger]
    all.ts <- st[smaller]
    all.te <- en[smaller]

    if (type=="gap") {
        output[is.same] <- pmax(all.as, all.ts) - pmin(all.ae, all.te) - 1L
    } else if (type=="span") {
        output[is.same] <- pmax(all.ae, all.te) - pmin(all.as, all.ts) + 1L
    } else if (type=="mid") {
        output[is.same] <- as.integer(abs(all.as + all.ae - all.ts - all.te)/2L) # Need 'abs', in case later range has earlier midpoint (e.g., if nested).
    } else if (type=="diag") {
        output[is.same] <- larger - smaller
    }
    return(output)
}

setMethod("pairdist", "GInteractions", function(x, type="mid") 
# Outputs an integer vector specifying the distance between the interacting bins,
# depending on the type of distance specified.
{
    ai1 <- anchors(x, type="first", id=TRUE)
    ai2 <- anchors(x, type="second", id=TRUE)
    .get_dist_output(regions(x), ai1, ai2, type)
})

setMethod("pairdist", "InteractionSet", function(x, type="mid") { 
    pairdist(interactions(x), type=type) 
})

setMethod("pairdist", "ContactMatrix", function(x, type="mid") 
# Outputs an integer vector specifying the distance between the interacting bins,
# depending on the type of distance specified.
{
    ai1 <- rep(anchors(x, type="row", id=TRUE), ncol(x))
    ai2 <- rep(anchors(x, type="column", id=TRUE), each=nrow(x))
    out <- .get_dist_output(regions(x), ai1, ai2, type)
    dim(out) <- dim(as.matrix(x))
    return(out)
})

setMethod("intrachr", "GInteractions", function(x) { pairdist(x, type="intra") })
setMethod("intrachr", "InteractionSet", function(x) { pairdist(x, type="intra") })
setMethod("intrachr", "ContactMatrix", function(x) { pairdist(x, type="intra") })

###############################################################
# Simple getters and setters (some of these are exported):

for (siglist in list("GInteractions", "ContactMatrix")) {
    setMethod("regions", siglist, function(x) { x@regions })
}

for (siglist in list("GInteractions", "ContactMatrix")) { 
    # internal, for use with environments and with().
    setMethod("anchor1", siglist, function(x) { x@anchor1 })
    setMethod("anchor2", siglist, function(x) { x@anchor2 })

    setReplaceMethod("unchecked_regions", siglist, function(x, value) {
        x@regions <- value
        return(x)        
    })

    setReplaceMethod("unchecked_anchor1", siglist, function(x, value) {
        x@anchor1 <- value 
        return(x)        
    })

    setReplaceMethod("unchecked_anchor2", siglist, function(x, value) {
        x@anchor2 <- value 
        return(x)        
    })
}

setMethod("interactions", "InteractionSet", function(x) { return(x@interactions) })

setReplaceMethod("unchecked_interactions", "InteractionSet", function(x, value) { 
    x@interactions <- value
    return(x)
}) 

setMethod("as.matrix", "ContactMatrix", function(x) {
    return(x@matrix)
}) 

setReplaceMethod("unchecked_matrix", "ContactMatrix", function(x, value) {
    x@matrix <- value
    return(x)
}) 

###############################################################
# Exported getters for anchors. 

setMethod("anchorIds", "GInteractions", function(x, type="both") {
    type <- match.arg(type, c("both", "first", "second"))
    if (type=="both") {
        out <- list(first=anchor1(x), second=anchor2(x))
        names(out$first) <- names(out$second) <- names(x)
    } else if (type=="first") {
        out <- anchor1(x)
        names(out) <- names(x)
    } else {
        out <- anchor2(x)
        names(out) <- names(x)
    }
    return(out)
}) 

setMethod("anchors", "GInteractions", function(x, type="both", id=FALSE) {
    if (id) { 
        return(anchorIds(x, type=type)) 
    }

    type <- match.arg(type, c("both", "first", "second"))
    if (type=="both") {
        out <- list(first=regions(x)[anchor1(x)], second=regions(x)[anchor2(x)])
        names(out$first) <- names(out$second) <- names(x)
    } else if (type=="first") {
        out <- regions(x)[anchor1(x)]
        names(out) <- names(x)
    } else {
        out <- regions(x)[anchor2(x)]
        names(out) <- names(x)
    }
    return(out)
})

# Defining some convenience methods.
setMethod("first", "GInteractions", function(x) { anchors(x, type="first") })
setMethod("second", "GInteractions", function(x) { anchors(x, type="second") })

# Same again for ContactMatrix objects.
setMethod("anchorIds", "ContactMatrix", function(x, type="both") {
    type <- match.arg(type, c("both", "row", "column"))
    if (type=="both") {
        out <- list(row=anchor1(x), column=anchor2(x))
        names(out$row) <- rownames(x)
        names(out$column) <- colnames(x)
    } else if (type=="row") {
        out <- anchor1(x)
        names(out) <- rownames(x)
    } else {
        out <- anchor2(x)
        names(out) <- colnames(x)
    }
    return(out)
}) 

setMethod("anchors", "ContactMatrix", function(x, type="both", id=FALSE) {
    if (id) { 
        return(anchorIds(x, type=type)) 
    }

    type <- match.arg(type, c("both", "row", "column"))
    if (type=="both") {
        out <- list(row=regions(x)[anchor1(x)], column=regions(x)[anchor2(x)])
        names(out$row) <- rownames(x)
        names(out$column) <- colnames(x)
    } else if (type=="row") {
        out <- regions(x)[anchor1(x)]
        names(out) <- rownames(x)
    } else {
        out <- regions(x)[anchor2(x)]
        names(out) <- colnames(x)
    }
    return(out)
})

###############################################################
# Setters for regions:

for (siglist in c("GInteractions", "ContactMatrix")) { 
    setReplaceMethod("regions", siglist, function(x, value) {
        if (length(value)!=length(regions(x))) { 
            stop("assigned value must be of the same length as 'regions(x)'")
        }
        out <- .resort_regions(anchor1(x), anchor2(x), value)
        unchecked_anchor1(x) <- out$anchor1
        unchecked_anchor2(x) <- out$anchor2
        unchecked_regions(x) <- out$regions
        validObject(x)
        return(x)
    })
}

# Also allow setting of regions of different length.

for (siglist in c("GInteractions", "ContactMatrix")) { 
    setReplaceMethod("replaceRegions", siglist, function(x, value) {
        converter <- match(regions(x), value)
        new.a1 <- converter[anchor1(x)]
        new.a2 <- converter[anchor2(x)]
        if (any(is.na(new.a1)) || any(is.na(new.a2))) { 
            stop("some existing ranges do not exist in replacement GRanges") 
        }

        out <- .resort_regions(new.a1, new.a2, value)
        unchecked_anchor1(x) <- out$anchor1
        unchecked_anchor2(x) <- out$anchor2
        unchecked_regions(x) <- out$regions
        return(x)
    })
}

# Append regions.

for (siglist in c("GInteractions", "ContactMatrix")) {
    setReplaceMethod("appendRegions", siglist, function(x, value) {
        out <- .resort_regions(anchor1(x), anchor2(x), c(regions(x), value)) 
        unchecked_anchor1(x) <- out$anchor1
        unchecked_anchor2(x) <- out$anchor2
        unchecked_regions(x) <- out$regions
        return(x)
    })
}

# Reduce regions.

for (siglist in c("GInteractions", "ContactMatrix")) { 
    setMethod("reduceRegions", siglist, function(x) {
        used <- logical(length(regions(x)))
        used[anchor1(x)] <- TRUE
        used[anchor2(x)] <- TRUE
        new.dex <- integer(length(used))
        new.dex[used] <- seq_len(sum(used))
        unchecked_anchor1(x) <- new.dex[anchor1(x)]
        unchecked_anchor2(x) <- new.dex[anchor2(x)]
        unchecked_regions(x) <- regions(x)[used]
        return(x)
    })
}

###############################################################
# Setters for anchors.

setReplaceMethod("anchorIds", "GInteractions", function(x, type="both", ..., value) {
    type <- match.arg(type, c("both", "first", "second"))
    if (type=="both") { 
        if (length(value)!=2L) { 
            stop("'value' must be a list of 2 numeric vectors")
        }
        unchecked_anchor1(x) <- as.integer(value[[1]])
        unchecked_anchor2(x) <- as.integer(value[[2]])
    } else if (type=="first") {
        unchecked_anchor1(x) <- as.integer(value)
    } else {
        unchecked_anchor2(x) <- as.integer(value)
    }

    validObject(x)
    return(x)
})

setReplaceMethod("anchorIds", "ContactMatrix", function(x, type="both", ..., value) {
    type <- match.arg(type, c("both", "row", "column"))
    if (type=="both") { 
        if (length(value)!=2L) { 
            stop("'value' must be a list of 2 numeric vectors")
        }
        unchecked_anchor1(x) <- as.integer(value[[1]])
        unchecked_anchor2(x) <- as.integer(value[[2]])
    } else if (type=="row") {
        unchecked_anchor1(x) <- as.integer(value)
    } else {
        unchecked_anchor2(x) <- as.integer(value)
    }

    validObject(x)
    return(x)
})

# Specialist methods for enforced classes.
setReplaceMethod("anchorIds", "StrictGInteractions", function(x, type="both", ..., value) {
    x <- as(x, "GInteractions")
    anchorIds(x, type=type, ...) <- value
    x <- swapAnchors(x)
    as(x, "StrictGInteractions")
})

setReplaceMethod("anchorIds", "ReverseStrictGInteractions", function(x, type="both", ..., value) {
    x <- as(x, "GInteractions")
    anchorIds(x, type=type, ...) <- value
    x <- swapAnchors(x, mode="reverse")
    as(x, "ReverseStrictGInteractions")
})

###############################################################
# Methods on InteractionSet that operate on GInteractions.

setReplaceMethod("interactions", "InteractionSet", function(x, value) { 
    unchecked_interactions(x) <- value
    validObject(x)
    return(x)
}) 

setMethod("anchors", "InteractionSet", function(x, type="both", id=FALSE) { 
    anchors(interactions(x), type=type, id=id) 
})

setMethod("anchorIds", "InteractionSet", function(x, type="both") {
    anchorIds(interactions(x), type=type)
})

setMethod("first", "InteractionSet", function(x) { first(interactions(x)) })

setMethod("second", "InteractionSet", function(x) { second(interactions(x)) })

setMethod("regions", "InteractionSet", function(x) { regions(interactions(x)) })

# Modification of regions in GInteractions doesn't affect validity of InteractionSet.

setReplaceMethod("anchorIds", "InteractionSet", function(x, type="both", ..., value) { 
    i <- interactions(x)
    anchorIds(i, type=type, ...) <- value 
    unchecked_interactions(x) <- i
    return(x)
})

setReplaceMethod("regions", "InteractionSet", function(x, value) { 
    i <- interactions(x)
    regions(i) <- value
    unchecked_interactions(x) <- i
    return(x)
})

setReplaceMethod("replaceRegions", "InteractionSet", function(x, value) { 
    i <- interactions(x)
    replaceRegions(i) <- value
    unchecked_interactions(x) <- i
    return(x)
})

setReplaceMethod("appendRegions", "InteractionSet", function(x, value) { 
    i <- interactions(x)
    appendRegions(i) <- value
    unchecked_interactions(x) <- i
    return(x)
})

setMethod("reduceRegions", "InteractionSet", function(x) {
    unchecked_interactions(x) <- reduceRegions(interactions(x))
    return(x)
})

###############################################################
# Defining some other getters and setters.

setMethod("$", "GInteractions", function(x, name) {
    return(mcols(x)[[name]])
})

setReplaceMethod("$", "GInteractions", function(x, name, value) {
    mcols(x)[[name]] <- value
    return(x)
})

setMethod("mcols", "InteractionSet", function(x, use.names=FALSE) {
    mcols(interactions(x), use.names=use.names)
})

setReplaceMethod("mcols", "InteractionSet", function(x, ..., value) {
    i <- interactions(x)                 
    mcols(i, ...) <- value
    unchecked_interactions(x) <- i
    return(x)
})

###############################################################
# Name getting and setting.

setMethod("names", "GInteractions", function(x) { 
    x@NAMES 
})

setReplaceMethod("names", "GInteractions", function(x, value) {
    if (!is.null(value) && !is.character(value)) { value <- as.character(value) }                
    x@NAMES <- value
    validObject(x)
    return(x)
})

setMethod("names", "InteractionSet", function(x) {
    names(interactions(x))
})

setReplaceMethod("names", "InteractionSet", function(x, value) {
    i <- interactions(x)                 
    names(i) <- value
    unchecked_interactions(x) <- i
    return(x)
})

setMethod("dimnames", "ContactMatrix", function(x) {
    dimnames(as.matrix(x))
})

setReplaceMethod("dimnames", "ContactMatrix", function(x, value) {
    m <- as.matrix(x)
    dimnames(m) <- value
    unchecked_matrix(x) <- m
    return(x)
})

###############################################################
# Seqinfo getting and setting.

for (siglist in c("GInteractions", "ContactMatrix")) { 
    setMethod("seqinfo", siglist, function(x) {
        seqinfo(regions(x))
    })
    
    setReplaceMethod("seqinfo", siglist, function(x, new2old = NULL, pruning.mode = c("error", "coarse", "fine", "tidy"), value) {
        r <- regions(x)
        seqinfo(r, new2old=new2old, pruning.mode=pruning.mode) <- value
        unchecked_regions(x) <- r
        return(x)
    })
}

setMethod("seqinfo", "InteractionSet", function(x) {
     seqinfo(interactions(x))
})

setReplaceMethod("seqinfo", "InteractionSet", function(x, new2old = NULL, pruning.mode = c("error", "coarse", "fine", "tidy"), value) {
    i <- interactions(x)                 
    seqinfo(i) <- value
    unchecked_interactions(x) <- i
    return(x)
})

##############################################
# Matrix dimensions

setMethod("dim", "ContactMatrix", function(x) { 
    dim(as.matrix(x))
})

setMethod("length", "ContactMatrix", function(x) { 
    length(as.matrix(x))
})

setReplaceMethod("as.matrix", "ContactMatrix", function(x, value) {
    if (is(value, "Matrix")) {
        if (!identical(dim(x), dim(value))) { 
            stop("replacement Matrix must have same dimensions as 'x'")
        }
        unchecked_matrix(x) <- value
    } else {
        x@matrix[] <- value
    }
    return(x)
}) 

###############################################################
# End

.boundingBox <- function(x, f) 
# As the name suggests, it computes a bounding box for groups of interactions in 'x'.
# Groups are specified by a factor 'f'.
#
# written by Aaron Lun
# created 2 December 2015
{
    if (missing(f)) { 
        f <- rep(1L, length(x))
    }
    if (length(f)!=length(x)) { 
        stop("length of 'f' must be equal to number of interactions")
    } 

    a1 <- anchors(x, type="first", id=TRUE)
    a2 <- anchors(x, type="second", id=TRUE)

    # Get the run lengths and values in a slightly convoluted way to handle factors (as rle() does not).
    by.f <- split(seq_along(f), f)
    o <- unlist(by.f)
    f.runs <- lengths(by.f)
    f.values <- names(by.f)

    # Extract the chromosomal information.
    x.reg <- regions(x)
    starts <- start(x.reg)
    ends <- end(x.reg)
    
    chrs <- seqnames(x.reg)
    ref.chr <- levels(chrs)
    chrs <- as.integer(chrs)

    # Using indices for easy comparison inside C++.
    bound1 <- .Call(cxx_get_box_bounds, f.runs, f.values, a1[o]-1L, chrs, starts, ends) 
    bound2 <- .Call(cxx_get_box_bounds, f.runs, f.values, a2[o]-1L, chrs, starts, ends) 

    gr1 <- GRanges(ref.chr[bound1[[1]]], IRanges(bound1[[2]], bound1[[3]]), seqinfo=seqinfo(x)) 
    gr2 <- GRanges(ref.chr[bound2[[1]]], IRanges(bound2[[2]], bound2[[3]]), seqinfo=seqinfo(x))
    out <- GInteractions(gr1, gr2)
    names(out) <- f.values
    return(out)
}

setMethod("boundingBox", "GInteractions", .boundingBox)
setMethod("boundingBox", "InteractionSet", .boundingBox)

# Defines the ContactMatrix class.

setClass("ContactMatrix",
    contains="Annotated", 
    slots=list(
        matrix="ANY", 
        anchor1="integer",
        anchor2="integer",
        regions="GRanges"
    )       
)

# Defines the GInteractions class, to hold interacting coordinates. 
# (Could inherit from Hits with an extra 'regions'; but I don't want to
# deal with 'queryHits' and 'subjectHits', which would get very confusing
# when you're dealing with queries and subjects in the overlap section.)


setClass("GInteractions", 
    contains="Vector",
    representation(
        anchor1="integer",
        anchor2="integer",
        regions="GRanges",
        NAMES="character_OR_NULL",
        elementMetadata="DataFrame"
    )
)

setClass("StrictGInteractions", contains="GInteractions")
setClass("ReverseStrictGInteractions", contains="GInteractions")

# Defines the InteractionSet class, based on the SummarizedExperiment base class.
# This allows us to avoid re-defining various standard functions.

setClass("InteractionSet", 
    contains="SummarizedExperiment",
    representation(
        interactions="GInteractions"
    )
)

# Inflate from InteractionSet to ContactMatrix.

.make_to_indices <- function(regs, i, ...) {
    nregs <- length(regs)
    if (is.numeric(i)) { 
        i <- as.integer(i)
        if (any(!is.finite(i)) || any(i<=0L) || any(i > nregs)) { 
            stop("indices must be positive integers no greater than 'length(regions(x))'") 
        }
        return(i)
    } else if (is.logical(i)) {
        if (length(i)!=nregs) {
            stop("logical index vector must be of length equal to 'length(regions(x))'")
        }
        return(which(i))
    } else if (is.character(i)) { 
        return(which(seqnames(regs) %in% i))
    } else if (is(i, "GRanges")) {
        return(which(overlapsAny(regs, i, ...)))
    } else if (is.null(i)) {
        return(seq_len(nregs))    
    } else {
        stop("invalid value for row/column selection")
    }
}

setMethod("inflate", "GInteractions", function(x, rows, columns, fill=TRUE, swap=TRUE, sparse=FALSE, ...) {
    row.chosen <- .make_to_indices(regions(x), rows, ...)
    col.chosen <- .make_to_indices(regions(x), columns, ...)
    fill <- rep(fill, length.out=length(x))
     
    # Removing duplicated rows and resorting (we'll put them back in later)
    ro <- order(row.chosen)
    co <- order(col.chosen)
    row.chosen <- row.chosen[ro]
    col.chosen <- col.chosen[co]
    rnd <- !duplicated(row.chosen)
    cnd <- !duplicated(col.chosen)
    row.chosen <- row.chosen[rnd]   
    col.chosen <- col.chosen[cnd]

    # Duplicated interactions can't be handled.
    dx <- duplicated(x)
    if (any(dx)) { 
        warning("duplicated interactions in 'x' are removed")
        x <- x[!dx,]
        fill <- fill[!dx]
    }

    # Matching.
    a1 <- anchors(x, type="first", id=TRUE)
    a2 <- anchors(x, type="second", id=TRUE)
    ar1 <- match(a1, row.chosen)
    ac1 <- match(a1, col.chosen)
    ar2 <- match(a2, row.chosen)
    ac2 <- match(a2, col.chosen)

    # Filling.
    nR <- length(row.chosen)
    nC <- length(col.chosen)
    relevantA <- !is.na(ar1) & !is.na(ac2)
    relevantB <- !is.na(ar2) & !is.na(ac1)

    if (!sparse) { 
        out.mat <- matrix(as(NA, typeof(fill)), nR, nC)
        idx <- cbind(ar1[relevantA], ac2[relevantA])
        out.mat[idx] <- fill[relevantA] 
    } else {
        out.mat <- sparseMatrix(i=ar1[relevantA], j=ac2[relevantA],
                                x=fill[relevantA], dims=c(nR, nC))
    }
    if (swap) { # Swapping anchors.
        idx <- cbind(ar2[relevantB], ac1[relevantB])
        out.mat[idx] <- fill[relevantB] 
    }

    # Restoring the original order.
    original.rows <- cumsum(rnd)
    original.rows[ro] <- original.rows
    original.cols <- cumsum(cnd)
    original.cols[co] <- original.cols

    return(ContactMatrix(out.mat[original.rows,original.cols,drop=FALSE], 
                row.chosen[original.rows], col.chosen[original.cols], regions(x)))
})
 
setMethod("inflate", "InteractionSet", function(x, rows, columns, assay=1, sample=1, fill, swap=TRUE, sparse=FALSE, ...) {
    if (missing(fill)) { fill <- assay(x, assay)[,sample] }
    inflate(interactions(x), rows, columns, fill=fill, swap=swap, sparse=sparse, ...)
})

setMethod("deflate", "ContactMatrix", function(x, collapse=TRUE, extract, use.zero, use.na, ...) {
    # Choosing the expansion strategy.
    if (missing(extract)) { 
        is.sparse <- is(as.matrix(x), "sparseMatrix")
        if (missing(use.zero)) {
            use.zero <- !is.sparse
        }
        if (missing(use.na)) { 
            use.na <- is.sparse
        }
        
        if (use.na && use.zero) { 
            is.valid <- seq_along(as.matrix(x))
        } else {
            is.valid <- TRUE
            if (!use.zero) { 
                is.valid <- is.valid & as.matrix(x)!=0
            } 
            if (!use.na) { 
                is.valid <- is.valid & !is.na(as.matrix(x))
            }
            is.valid <- Matrix::which(is.valid)
        }
    } else {
        if (!identical(length(extract), length(x))) { 
            stop("extraction matrix must be of the same length as 'x'")
        }
        is.valid <- Matrix::which(extract)
    }

    valid.coords <- arrayInd(is.valid, dim(x))
    row.index <- anchors(x, type="row", id=TRUE)[valid.coords[,1]]
    col.index <- anchors(x, type="column", id=TRUE)[valid.coords[,2]]

    all.values <- as.matrix(x)[is.valid]
    dim(all.values) <- c(length(all.values), 1L)
    colnames(all.values) <- "1"
        
    final <- InteractionSet(all.values, GInteractions(row.index, col.index, regions(x), 
                mode=ifelse(collapse, "strict", "normal")), ...)
    if (collapse) {
        final <- unique(final)
    }
    return(final)
})

# Convert between GInteractions objects of different strictness.

setAs("GInteractions", "StrictGInteractions", function(from) {
    new("StrictGInteractions", .relaxed_strictness_converter(from, mode="order"))
})

setAs("GInteractions", "ReverseStrictGInteractions", function(from) {
    new("ReverseStrictGInteractions", .relaxed_strictness_converter(from, mode="reverse"))
})

.relaxed_strictness_converter <- function(from, mode) {
    # Disabling the validity, just in case we convert from reverse to strict,
    # which would trigger failure of the validObject() in swapAnchors().
    old_val <- S4Vectors:::disableValidity() 
    if (!old_val) { 
        on.exit(S4Vectors:::disableValidity(old_val))
        S4Vectors:::disableValidity(TRUE)
    }
    swapAnchors(from, mode=mode)
}

setMethod("swapAnchors", "GInteractions", function(x, mode=c("order", "reverse", "all")) {
    mode <- match.arg(mode)
    if (mode=="order") {
        out <- .enforce_order(anchor1(x), anchor2(x))
    } else if (mode=="reverse") {
        out <- rev(.enforce_order(anchor1(x), anchor2(x)))
    } else {
        out <- list(anchor2(x), anchor1(x))
    }
    unchecked_anchor1(x) <- out[[1]]        
    unchecked_anchor2(x) <- out[[2]]
    validObject(x)
    return(x)
})

setMethod("swapAnchors", "InteractionSet", function(x, mode=c("order", "reverse", "all")) {
    interactions(x) <- swapAnchors(interactions(x), mode)
    return(x)
})

setAs("GInteractions", "SelfHits", function(from) {
      SelfHits(from=anchor1(from), to=anchor2(from), nnode=length(regions(from)), sort.by.query=FALSE)
})

.flipGI <- function(x) {
    in.order <- as.vector(do.call(rbind, anchors(x, id=TRUE)))
    all.regions <- regions(x)[in.order]
    names(all.regions) <- rep(c('first', 'second'), length(x))

    out_breakpoints <- seq.int(2L, by=2L, length.out=length(x))
    out_partitioning <- PartitioningByEnd(out_breakpoints, names=names(x))
    out <- relist(all.regions, out_partitioning)
    
    mcols(out) <- mcols(x)
    metadata(out) <- metadata(x)
    return(out)
}

setAs("GInteractions", "GRangesList", function(from) .flipGI(from))

setAs("GInteractions", "Pairs", function(from) {
      Pairs(first(from), second(from), names=names(from), mcols(from))
})

setMethod("pairs", "GInteractions", function(x, id=FALSE, as.grlist=FALSE) {
    if (id) {
        return(as(x, "SelfHits"))
    } else if (as.grlist) {
        return(as(x, "GRangesList"))
    } else {
        return(as(x, "Pairs"))
    }
})

setMethod("pairs", "InteractionSet", function(x, id=FALSE, as.grlist=FALSE) {
    pairs(interactions(x), id=id, as.grlist=as.grlist)
})

# Probably not to be used, as GRangesList may not always have two entries.
.unflipGI <- function(x, ...) {
    if (!all(lengths(x)==2L)) { 
        stop("'x' can only contain GRanges of length 2") 
    }
    everything <- unname(unlist(x))
    all.first <- seq(1, length(x)*2L, by=2)
    all.second <- seq(2, length(x)*2L, by=2)

    out <- GInteractions(everything[all.first], everything[all.second], ...)
    mcols(out) <- mcols(x)
    metadata(out) <- metadata(x)
    names(out) < names(x)
    return(out)
}

makeGInteractionsFromGRangesPairs <- function(x) {
    if (!is(x, "Pairs")) { 
        stop("'x' must be a Pairs object")
    }
    if (!is(first(x), "GRanges") || !is(second(x), "GRanges")) {
        stop("both paired elements must be GRanges")
    }
    out <- GInteractions(anchor1=first(x), anchor2=second(x))
    mcols(out) <- mcols(x)
    names(out) <- names(x)
    return(out)
}

# Tests the construction and manipulation of GInteractions objects.
# library(InteractionSet); library(testthat); source("test-GI.R")

set.seed(7000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
x <- GInteractions(all.anchor1, all.anchor2, all.regions)

test_that("show methods work for GI objects", {
    expect_output(sub("[0-9]", " ", show(x)), "GInteractions object with 20 interactions and 0 metadata columns:
       seqnames1   ranges1     seqnames2   ranges2
           <Rle> <IRanges>         <Rle> <IRanges>
   [1]      chrA    94-105 ---      chrA     41-48
   [2]      chrB     42-54 ---      chrA    94-105
   [3]      chrA     41-48 ---      chrA     59-72
   [4]      chrB     64-78 ---      chrA     55-68
   [5]      chrA     41-59 ---      chrA      3-23
   ...       ...       ... ...       ...       ...
  [16]      chrA     76-95 ---      chrB     91-98
  [17]      chrA     46-66 ---      chrB     67-84
  [18]      chrA     68-78 ---      chrA     20-33
  [19]      chrA     61-67 ---      chrA    87-104
  [20]      chrA     18-35 ---      chrB     63-76
  -------
  regions: 30 ranges and 0 metadata columns
  seqinfo: 2 sequences from an unspecified genome; no seqlengths", fixed=TRUE)
})

######################################

# Needed in many test chunks, so I'll let it out here.
o <- order(all.regions)
ref.regions <- all.regions[o]
new.pos <- integer(length(o))
new.pos[o] <- seq_along(new.pos)
ref.anchor1 <- new.pos[all.anchor1]
ref.anchor2 <- new.pos[all.anchor2]

test_that("slot access works in GI objects", {
    expect_that(x, is_a("GInteractions"))
    expect_true(!is.unsorted(regions(x)))   
    expect_identical(regions(x), ref.regions)

    expect_identical(anchors(x, id=TRUE, type="first"), ref.anchor1)
    expect_identical(anchors(x, id=TRUE, type="second"), ref.anchor2)
    expect_identical(anchors(x, id=TRUE), list(first=ref.anchor1, second=ref.anchor2))
    
    expect_identical(anchors(x, type="first"), ref.regions[ref.anchor1])
    expect_identical(anchors(x, type="second"), ref.regions[ref.anchor2])
    expect_identical(anchors(x), list(first=ref.regions[ref.anchor1], second=ref.regions[ref.anchor2]))
    expect_identical(anchors(x, type="first"), first(x))
    expect_identical(anchors(x, type="second"), second(x))
})

######################################

test_that("alternative construction methods work for GI objects", {
    x2 <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2])
    was.used <- sort(unique(all.regions[union(all.anchor1, all.anchor2)])) # Only includes the regions actually used.
    expect_identical(regions(x2), was.used)
    expect_identical(anchors(x2), anchors(x))
    
    x3 <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], was.used)
    expect_identical(anchors(x3, id=TRUE), anchors(x2, id=TRUE))
    expect_identical(regions(x3), regions(x2))
    
    anno.regions <- all.regions
    anno.regions$score <- seq_along(anno.regions) # Seeing what happens with annotation.
    anno.regions$revscore <- rev(seq_along(anno.regions))
    x4 <- GInteractions(anno.regions[all.anchor1], anno.regions[all.anchor2])
    expect_identical(regions(x2), regions(x4))
    expect_identical(mcols(x4), DataFrame(anchor1.score=anno.regions$score[all.anchor1], anchor1.revscore=anno.regions$revscore[all.anchor1],
                                          anchor2.score=anno.regions$score[all.anchor2], anchor2.revscore=anno.regions$revscore[all.anchor2]))
})

######################################

test_that("GI constructors behave properly on crappy inputs", {
    empty <- GInteractions(integer(0), numeric(0), GRanges())
    expect_identical(length(empty), 0L)
    expect_identical(length(anchors(empty, type="first")), 0L)
    expect_identical(empty, GInteractions())
    
    empty <- GInteractions(GRanges(), GRanges())
    expect_identical(length(empty), 0L)
    expect_identical(length(anchors(empty, type="first")), 0L)
    expect_identical(empty, GInteractions())
    
    empty <- GInteractions(GRanges(), GRanges(), all.regions)
    expect_identical(length(empty), 0L)
    expect_identical(length(anchors(empty, type="first")), 0L)
    expect_identical(length(regions(empty)), length(all.regions))
    
    expect_error(GInteractions(1:4, 1, all.regions), "first and second anchor vectors have different lengths")
    expect_error(GInteractions(0:3, 1:4, all.regions), "all anchor indices must be positive integers")
    expect_error(GInteractions(c(1,2,3,NA), 1:4, all.regions), "all anchor indices must be finite integers")
    expect_error(GInteractions(c(1,2,3,length(all.regions)+1L), 1:4, all.regions), "all anchor indices must refer to entries in 'regions'")
    missing.value <- GRanges("chrB", IRanges(1000,1000))
    expect_error(GInteractions(missing.value, missing.value, all.regions), "anchor regions missing in specified 'regions'")
})

######################################

test_that("GI constructors preserve metadata", {
    m.x <- GInteractions(all.anchor1, all.anchor2, all.regions, score=1L)
    expect_identical(m.x$score, rep(1L, length(m.x)))
    m.x <- GInteractions(all.anchor1, all.anchor2, all.regions, score=seq_along(all.anchor1))
    expect_identical(m.x$score, seq_along(m.x))
    
    m.x <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], all.regions, score=1L)
    expect_identical(m.x$score, rep(1L, length(m.x)))
    m.x <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], all.regions, score=seq_along(all.anchor1))
    expect_identical(m.x$score, seq_along(m.x))
    
    m.x <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], score=1L)
    expect_identical(m.x$score, rep(1L, length(m.x)))
    m.x <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], score=seq_along(all.anchor1))
    expect_identical(m.x$score, seq_along(m.x))
    
    m.regions <- all.regions
    m.regions$GC <- rev(seq_along(m.regions))
    m.x <- GInteractions(m.regions[all.anchor1], m.regions[all.anchor2], score=seq_along(all.anchor1))
    expect_identical(m.x$score, seq_along(m.x))
    expect_identical(m.x$anchor1.GC, m.regions$GC[all.anchor1])
    expect_identical(m.x$anchor2.GC, m.regions$GC[all.anchor2])
    
    m.x <- GInteractions(integer(0), integer(0), all.regions, score=1)
    expect_identical(m.x$score, numeric(0))
    m.x <- GInteractions(regions=all.regions, score=numeric(0))
    expect_identical(m.x$score, numeric(0))
    m.x <- GInteractions(regions=all.regions, score=1)
    expect_identical(m.x$score, numeric(0))
    m.x <- GInteractions(regions=all.regions, score=1:2)
    expect_identical(m.x$score, integer(0))
})

######################################

set.seed(7001)

test_that("anchor setters work properly for GI objects", { 
    fresh.anchor1 <- sample(N, Np)
    fresh.anchor2 <- sample(N, Np)
    anchorIds(x) <- list(fresh.anchor1, fresh.anchor2)
    expect_identical(anchors(x, id=TRUE, type="first"), fresh.anchor1)
    expect_identical(anchors(x, id=TRUE, type="second"), fresh.anchor2)
    expect_error(anchorIds(x) <- list(ref.anchor1, ref.anchor2, ref.anchor1), "must be a list of 2 numeric vectors")
    expect_error(anchorIds(x) <- list(ref.anchor1[1:(Np/2)], ref.anchor2), "x@anchor2' is not parallel to 'x'")
    
    mod.x <- x
    anchorIds(mod.x, type="first") <- ref.anchor1 # Checking that this also works
    expect_identical(anchors(mod.x, id=TRUE, type="first"), ref.anchor1)
    mod.x <- x
    anchorIds(mod.x, type="second") <- ref.anchor2
    expect_identical(anchors(mod.x, id=TRUE, type="second"), ref.anchor2)
    anchorIds(x, type="both") <- list(ref.anchor1, ref.anchor2) # Restoring.
    expect_identical(anchors(x, id=TRUE, type="first"), ref.anchor1)
    expect_identical(anchors(x, id=TRUE, type="second"), ref.anchor2)
})

test_that("region setters work properly for GI objects", { 
    shuffled <- sample(100, N, replace=TRUE)
    regions(x)$score <- shuffled
    expect_identical(regions(x)$score, shuffled)
    expect_false(identical(regions(x), ref.regions))
    regions(x) <- ref.regions # Restoring.
    expect_true(identical(regions(x), ref.regions))

    x.dump <- x
    mod.ranges <- resize(regions(x), fix="center", width=50)
    new.ranges <- c(regions(x), mod.ranges) 
    expect_error(regions(x.dump) <- new.ranges, "assigned value must be of the same length")
    
    replaceRegions(x.dump) <- new.ranges
    expect_identical(anchors(x.dump), anchors(x))
    expect_identical(sort(new.ranges), regions(x.dump))
    expect_error(replaceRegions(x.dump) <- mod.ranges, "some existing ranges do not exist in replacement GRanges")
    
    x.dump2 <- x
    appendRegions(x.dump2) <- mod.ranges
    expect_identical(anchors(x.dump2), anchors(x))
    expect_identical(regions(x.dump), regions(x.dump2))
    
    x.dump <- reduceRegions(x)
    expect_identical(anchors(x), anchors(x.dump))
    expect_identical(regions(x)[sort(unique(unlist(anchors(x, id=TRUE))))], regions(x.dump))
})

ref.score <- runif(Np)
test_that("other setters work properly for GI objects", { 
    x$stuff <- ref.score
    expect_identical(x$stuff, mcols(x)$stuff)
    expect_identical(colnames(mcols(x)), "stuff")
    expect_output(show(x), "stuff")
    x$stuff <- NULL
    
    new.si <- Seqinfo(seqnames=c("chrA", "chrB"), seqlengths=c(1000, 2000))
    new.x <- x
    seqinfo(new.x) <- new.si
    expect_identical(seqinfo(new.x), new.si)
})

######################################

test_that("subsetting works for GI objects", {
    rchosen <- 1:10
    xsub <- x[rchosen,]
    expect_output(show(xsub), "GInteractions object with 10 interactions and 0 metadata columns:
       seqnames1   ranges1     seqnames2   ranges2
           <Rle> <IRanges>         <Rle> <IRanges>
   [1]      chrA    94-105 ---      chrA     41-48
   [2]      chrB     42-54 ---      chrA    94-105
   [3]      chrA     41-48 ---      chrA     59-72
   [4]      chrB     64-78 ---      chrA     55-68
   [5]      chrA     41-59 ---      chrA      3-23
   [6]      chrA     47-54 ---      chrA     14-19
   [7]      chrA    82-100 ---      chrA    84-103
   [8]      chrB     89-97 ---      chrA     46-66
   [9]      chrA    86-105 ---      chrA    82-100
  [10]      chrA     55-68 ---      chrB     81-98
  -------
  regions: 30 ranges and 0 metadata columns
  seqinfo: 2 sequences from an unspecified genome; no seqlengths", fixed=TRUE)
    expect_identical(xsub, x[rchosen])

    log.chosen <- logical(length(x))
    log.chosen[rchosen] <- TRUE
    expect_identical(xsub, subset(x, log.chosen))
    expect_identical(x[log.chosen], subset(x, log.chosen))

    expect_identical(length(xsub), length(rchosen))
    expect_identical(regions(xsub), regions(x))
    expect_identical(anchors(xsub, type="first"), ref.regions[ref.anchor1][rchosen])
    expect_identical(anchors(xsub, type="second"), ref.regions[ref.anchor2][rchosen])

    temp.x <- x
    temp.x$score <- ref.score
    expect_identical(temp.x[rchosen]$score, ref.score[rchosen])
    expect_identical(nrow(mcols(temp.x[rchosen])), length(rchosen))
    
    expect_identical(length(x[0,]), 0L)
    expect_identical(length(x[0]), 0L)
    expect_error(x[,1], "subscript contains out-of-bounds indices")

    # Subsetting by mcols works as expected.
    mcols(x)$stuff <- runif(length(x))
    expect_identical(x[,1], x)

    alt.x <- x
    mcols(alt.x)$whee <- runif(length(x))
    mcols(x)$stuff <- NULL
    mcols(x)$whee <- mcols(alt.x)$whee
    expect_identical(alt.x[,2], x)
    expect_error(x[,,1], "invalid subsetting")
})

test_that("subset assignment works for GI objects", {
    temp.x <- x
    temp.x[1:5+10,] <- x[1:5,]
    new.index <- seq_along(x)
    new.index[1:5+10] <- 1:5
    expect_identical(anchors(temp.x, type="first"), anchors(x, type="first")[new.index,])
    expect_identical(anchors(temp.x, type="second"), anchors(x, type="second")[new.index,])
    
    temp.x <- x
    temp.x[0,] <- x[0,]
    expect_identical(temp.x, x)
    temp.x[] <- x
    expect_identical(temp.x, x)
    
    temp.x <- x
    temp.x$score <- ref.score
    temp.x[1:5]$score <- 5:1
    mod.score <- ref.score
    mod.score[1:5] <- 5:1
    expect_identical(temp.x$score, mod.score)

    rchosen <- 1:10
    expect_identical(temp.x[rchosen]$score, mod.score[rchosen])
    expect_identical(nrow(mcols(temp.x[rchosen])), length(rchosen))
    
    temp.x <- x
    regions(temp.x) <- resize(regions(temp.x), 10) # what happens with different regions?
    ref <- c(x[1], temp.x[-1])
    temp.x[1] <- x[1]
    expect_identical(temp.x, ref)
    
    temp.x <- x
    regions(temp.x) <- resize(regions(temp.x), 20) # Checking again, just in case.
    ref <- c(temp.x[1:5], x[1:5], temp.x[11:length(temp.x)])
    temp.x[6:10] <- x[1:5]
    expect_identical(temp.x, ref)
})

######################################

test_that("combining works for GI obejcts", {
    # When regions are the same.
    xsub <- x[1:5,]
    xsub2 <- x[6:20,]
    expect_identical(c(xsub, xsub2), x)
    xsub <- x[1:15]
    xsub2 <- x[16:20]
    expect_identical(c(xsub, xsub2), x)
    
    expect_identical(c(x[0,], x[0,]), x[0,])
    expect_identical(c(x, x[0,]), x)
    
    temp.x <- x
    temp.x$score <- ref.score
    double.up <- c(temp.x, temp.x)
    expect_identical(regions(double.up), regions(x))
    expect_identical(anchors(double.up, type="first"), rep(anchors(x, type="first"), 2))
    expect_identical(anchors(double.up, type="second"), rep(anchors(x, type="second"), 2))
    expect_identical(double.up$score, rep(temp.x$score, 2))

    # When regions are different.
    set.seed(7002)
    next.starts <- round(runif(N, 1, 100))
    next.ends <- next.starts + round(runif(N, 5, 20))
    next.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(next.starts, next.ends))
    
    next.anchor1 <- sample(N, Np)
    next.anchor2 <- sample(N, Np)
    next.x <- GInteractions(next.anchor1, next.anchor2, next.regions)
    
    c.x <- c(x, next.x)
    expect_identical(c(anchors(x, type="first"), anchors(next.x, type="first")), anchors(c.x, type="first"))
    expect_identical(c(anchors(x, type="second"), anchors(next.x, type="second")), anchors(c.x, type="second"))
    expect_identical(unique(sort(c(regions(x), regions(next.x)))), regions(c.x))
    
    expect_identical(anchors(c(x[0,], next.x[0,])), anchors(x[0,])) # Behaviour with empties.
    expect_identical(anchors(c(x, next.x[0,])), anchors(x)) # Not fully equal, as regions have changed.
    
    next.x2 <- GInteractions(1:10, 1:10, next.regions[1:10]) # What happens with non-equal lengths of the regions?
    c.x <- c(x, next.x2)
    expect_identical(c(anchors(x, type="first"), anchors(next.x2, type="first")), anchors(c.x, type="first"))
    expect_identical(c(anchors(x, type="second"), anchors(next.x2, type="second")), anchors(c.x, type="second"))
    expect_identical(unique(sort(c(regions(x), regions(next.x2)))), regions(c.x))

    # Plus metadata
    temp.x <- x
    temp.x$score <- ref.score
    double.up <- c(temp.x, temp.x)
    expect_identical(regions(double.up), regions(x))
    expect_identical(anchors(double.up, type="first"), rep(anchors(x, type="first"), 2))
    expect_identical(anchors(double.up, type="second"), rep(anchors(x, type="second"), 2))
    expect_identical(double.up$score, rep(temp.x$score, 2))

    # Plus names.
    temp.x <- temp.x2 <- x
    names(temp.x) <- paste0("Inter", seq_along(temp.x))
    names(temp.x2) <- paste0("Whee", seq_along(temp.x2))
    combined <- c(temp.x, temp.x2)
    expect_identical(names(combined), c(names(temp.x), names(temp.x2)))
})

######################################

test_that("sorting and deduplication of GI objects works", {
    o.x <- order(anchors(x, type="first"), anchors(x, type="second"))
    expect_identical(o.x, order(x))
    expect_identical(sort(x), x[o.x,])
   
    set.seed(70003)
    x.1 <- x[sample(length(x), 100, replace=TRUE)]
    x.2 <- x[sample(length(x), 100, replace=TRUE)]
    o.x2 <- order(anchors(x.1, type="first"), anchors(x.1, type="second"), anchors(x.2, type="first"), anchors(x.2, type="second"))
    expect_identical(o.x2, order(x.1, x.2))
    
    is.dup <- duplicated(paste0(anchors(x, type="first"), ".", anchors(x, type="second")))
    expect_identical(is.dup, duplicated(x))
    temp.x <- c(x, x)    
    is.dup <- duplicated(paste0(anchors(temp.x, type="first"), ".", anchors(temp.x, type="second")))
    expect_identical(is.dup, duplicated(temp.x))
    expect_true(all(tail(is.dup, length(x)))) # if ordering is stable; only the first occurrence should be true.
    expect_identical(x, unique(temp.x))
    
    is.dup <- duplicated(paste0(anchors(temp.x, type="first"), ".", anchors(temp.x, type="second")), fromLast=TRUE)
    expect_identical(is.dup, duplicated(temp.x, fromLast=TRUE))
    expect_true(all(head(is.dup, length(x)))) # if ordering is stable; only the first occurrence should be true.
    expect_equal(x, unique(temp.x, fromLast=TRUE))
    expect_false(any(duplicated(unique(temp.x))))
    
    expect_identical(order(x[0,]), integer(0))
    expect_identical(duplicated(x[0,]), logical(0))
})

test_that("anchor swapping for GI objects works", {
    new.x <- swapAnchors(x)
    expect_identical(anchors(new.x, type="first", id=TRUE), pmin(anchors(x, type="first", id=TRUE), anchors(x, type="second", id=TRUE)))
    expect_identical(anchors(new.x, type="second", id=TRUE), pmax(anchors(x, type="first", id=TRUE), anchors(x, type="second", id=TRUE)))
    expect_identical(regions(x), regions(new.x))
    
    new.x2 <- swapAnchors(x, mode="reverse")
    expect_identical(anchors(new.x2, type="first"), anchors(new.x, type="second"))
    expect_identical(anchors(new.x, type="first"), anchors(new.x2, type="second"))
    expect_identical(regions(x), regions(new.x2))
    
    new.x3 <- swapAnchors(x, mode="all")
    expect_identical(anchors(new.x3, type="first"), anchors(x, type="second"))
    expect_identical(anchors(x, type="first"), anchors(new.x3, type="second"))
    expect_identical(regions(x), regions(new.x3))
})

test_that("splitting of GI objects works", {
    flen <- c(5L, 10L, 5L)
    f <- rep(1:3, flen)
    out <- split(x, f)
    expect_equivalent(lengths(out), flen)
    for (i in seq_along(flen)) {
        expect_identical(out[[i]], x[f==i])
    }
    
    temp.x <- x
    temp.x$score <- ref.score
    out <- split(temp.x, f)
    for (i in seq_along(flen)) {
        expect_identical(out[[i]]$score, temp.x[f==i]$score)
    }
})

test_that("data.frame coercions of GI objects works", {
    out <- as.data.frame(x)
    ref1 <- as.data.frame(anchors(x, type="first"))
    colnames(ref1) <- paste0(colnames(ref1), "1")
    ref2 <- as.data.frame(anchors(x, type="second"))
    colnames(ref2) <- paste0(colnames(ref2), "2")
    expect_identical(out, data.frame(ref1, ref2))
    
    temp.x <- x
    temp.x$stuff <- ref.score
    out <- as.data.frame(temp.x)
    expect_identical(out, data.frame(ref1, ref2, stuff=ref.score))
    
    names(temp.x) <- paste0("X", seq_along(temp.x))
    out <- as.data.frame(temp.x)
    expect_identical(out, data.frame(ref1, ref2, stuff=ref.score, row.names=names(temp.x)))
    
    empty <- as.data.frame(x[0,])
    expect_identical(colnames(empty), colnames(as.data.frame(x)))
    expect_identical(nrow(empty), 0L)
})

test_that("environment generation works for GI objects", {
    expect_identical(anchors(x, id=TRUE, type="first"), with(x, anchor1))
    expect_identical(anchors(x, id=TRUE, type="second"), with(x, anchor2))
    expect_identical(regions(x), with(x, regions))
    expect_identical(anchors(x, type="first"), with(x, regions[anchor1]))
    expect_identical(anchors(x, type="second"), with(x, regions[anchor2]))
    expect_identical(names(x), with(x, names))
})

test_that("conversion to Pairs/GRangesList works for GI objects", {
    temp.x <- x
    temp.x$score <- ref.score
    prs <- pairs(temp.x)
    expect_identical(anchors(x, type="first"), first(prs))
    expect_identical(anchors(x, type="second"), second(prs))
    expect_identical(mcols(temp.x), mcols(prs))
    expect_identical(names(temp.x), names(prs))
    expect_identical(makeGInteractionsFromGRangesPairs(prs), reduceRegions(temp.x))                 
    
    grl <- pairs(temp.x, as.grlist=TRUE)
    first <- do.call(c, sapply(grl, function(x) { unname(x[1]) }))
    second <- do.call(c, sapply(grl, function(x) { unname(x[2]) }))
    expect_identical(anchors(x, type="first"), first)
    expect_identical(anchors(x, type="second"), second)
    expect_identical(mcols(temp.x), mcols(grl))
    
    out <- pairs(temp.x, id=TRUE)
    expect_identical(from(out), anchors(temp.x, id=TRUE, type="first"))
    expect_identical(to(out), anchors(temp.x, id=TRUE, type="second"))
    expect_identical(nnode(out), length(regions(temp.x)))
})

######################################

test_that("name handling is correct with GI objects", {
    temp.x <- x
    ref.names <- paste0("X", seq_along(temp.x))
    names(temp.x) <- ref.names
    expect_output(show(temp.x),"GInteractions object with 20 interactions and 0 metadata columns:
      seqnames1   ranges1     seqnames2   ranges2
          <Rle> <IRanges>         <Rle> <IRanges>
   X1      chrA    94-105 ---      chrA     41-48
   X2      chrB     42-54 ---      chrA    94-105
   X3      chrA     41-48 ---      chrA     59-72
   X4      chrB     64-78 ---      chrA     55-68
   X5      chrA     41-59 ---      chrA      3-23
  ...       ...       ... ...       ...       ...
  X16      chrA     76-95 ---      chrB     91-98
  X17      chrA     46-66 ---      chrB     67-84
  X18      chrA     68-78 ---      chrA     20-33
  X19      chrA     61-67 ---      chrA    87-104
  X20      chrA     18-35 ---      chrB     63-76
  -------
  regions: 30 ranges and 0 metadata columns
  seqinfo: 2 sequences from an unspecified genome; no seqlengths", fixed=TRUE)

    expect_identical(names(temp.x), ref.names)
    expect_identical(names(temp.x[2:5]), ref.names[2:5])
    expect_identical(names(c(temp.x, temp.x)), c(ref.names, ref.names))
    expect_identical(names(c(temp.x, x)), c(ref.names, character(length(x))))
    
    for (id in c(TRUE, FALSE)) {
        expect_identical(names(anchors(temp.x, id=id)[[1]]), ref.names)
        expect_identical(names(anchors(temp.x, id=id)[[2]]), ref.names)
        expect_identical(names(anchors(temp.x, id=id, type="first")), ref.names)
        expect_identical(names(anchors(temp.x, id=id, type="second")), ref.names)
    }
})

######################################

test_that("StrictGI objects are correctly formed", {
    sx <- GInteractions(all.anchor1, all.anchor2, all.regions, mode="strict")
    expect_that(sx, is_a("StrictGInteractions"))
    expect_identical(sx, as(x, "StrictGInteractions"))

    expect_identical(anchors(sx, id=TRUE, type="first"), do.call(pmin, anchors(x, id=TRUE)))
    expect_identical(anchors(sx, id=TRUE, type="second"), do.call(pmax, anchors(x, id=TRUE)))
    expect_identical(regions(sx), regions(x))
    
    temp.sx <- sx
    set.seed(70005)
    fresh.anchor1 <- sample(N, Np)
    fresh.anchor2 <- sample(N, Np)
    anchorIds(temp.sx, type="first") <- fresh.anchor1
    expect_identical(anchors(temp.sx, id=TRUE, type="first"), pmin(fresh.anchor1, anchors(sx, id=TRUE, type="second")))
    expect_identical(anchors(temp.sx, id=TRUE, type="second"), pmax(fresh.anchor1, anchors(sx, id=TRUE, type="second")))
    
    temp.sx <- sx
    anchorIds(temp.sx, type="second") <- fresh.anchor2
    expect_identical(anchors(temp.sx, id=TRUE, type="first"), pmin(fresh.anchor2, anchors(sx, id=TRUE, type="first")))
    expect_identical(anchors(temp.sx, id=TRUE, type="second"), pmax(fresh.anchor2, anchors(sx, id=TRUE, type="first")))
    
    temp.sx <- sx
    anchorIds(temp.sx) <- list(fresh.anchor1, fresh.anchor2)
    expect_identical(anchors(temp.sx, id=TRUE, type="first"), pmin(fresh.anchor1, fresh.anchor2))
    expect_identical(anchors(temp.sx, id=TRUE, type="second"), pmax(fresh.anchor1, fresh.anchor2))
    
    temp.sx2 <- sx
    anchorIds(temp.sx2, type="both") <- list(fresh.anchor2, fresh.anchor1)
    expect_identical(temp.sx2, temp.sx)
})

test_that("ReverseStrictGI objects are correctly formed", {
    rsx <- GInteractions(all.anchor1, all.anchor2, all.regions, mode="reverse")
    expect_that(rsx, is_a("ReverseStrictGInteractions"))
    expect_identical(rsx, as(x, "ReverseStrictGInteractions"))
   
    expect_identical(anchors(rsx, id=TRUE, type="first"), do.call(pmax, anchors(x, id=TRUE)))
    expect_identical(anchors(rsx, id=TRUE, type="second"), do.call(pmin, anchors(x, id=TRUE)))
    expect_identical(regions(rsx), regions(x))
    
    temp.rsx <- rsx
    set.seed(70006)
    fresh.anchor1 <- sample(N, Np)
    fresh.anchor2 <- sample(N, Np)
    anchorIds(temp.rsx, type="first") <- fresh.anchor1
    expect_identical(anchors(temp.rsx, id=TRUE, type="first"), pmax(fresh.anchor1, anchors(rsx, id=TRUE, type="second")))
    expect_identical(anchors(temp.rsx, id=TRUE, type="second"), pmin(fresh.anchor1, anchors(rsx, id=TRUE, type="second")))
    
    temp.rsx <- rsx
    anchorIds(temp.rsx, type="second") <- fresh.anchor2
    expect_identical(anchors(temp.rsx, id=TRUE, type="first"), pmax(fresh.anchor2, anchors(rsx, id=TRUE, type="first")))
    expect_identical(anchors(temp.rsx, id=TRUE, type="second"), pmin(fresh.anchor2, anchors(rsx, id=TRUE, type="first")))
    
    temp.rsx <- rsx
    anchorIds(temp.rsx) <- list(fresh.anchor1, fresh.anchor2)
    expect_identical(anchors(temp.rsx, id=TRUE, type="first"), pmax(fresh.anchor1, fresh.anchor2))
    expect_identical(anchors(temp.rsx, id=TRUE, type="second"), pmin(fresh.anchor1, fresh.anchor2))
    
    temp.rsx2 <- rsx
    anchorIds(temp.rsx2, type="both") <- list(fresh.anchor2, fresh.anchor1)
    expect_identical(temp.rsx2, temp.rsx)
})

test_that("combining GI objects of different strictness", {
    rsx <- GInteractions(all.anchor1, all.anchor2, all.regions, mode="reverse")
    sx <- GInteractions(all.anchor1, all.anchor2, all.regions, mode="strict")
    
    expect_identical(c(rsx, sx, x), as(c(x, x, x), "ReverseStrictGInteractions"))
    expect_identical(c(x, sx, rsx), as(c(x, swapAnchors(x), swapAnchors(x, mode="reverse")), "GInteractions"))
    expect_identical(c(sx, rsx, x), as(c(x, x, x), "StrictGInteractions"))

    # Checking for correct coercions between Strict subclasses. 
    expect_identical(rsx, as(sx, "ReverseStrictGInteractions"))
    expect_identical(sx, as(rsx, "StrictGInteractions"))
})

###############################################################
# This defines the findOverlaps method for GInteractions objects.

.get_used <- function(gi)
# Gets the indices for all regions(gi) that are actually used as anchors.
{
    all1 <- anchors(gi, type="first", id=TRUE)
    all2 <- anchors(gi, type="second", id=TRUE)
    used <- logical(length(regions(gi)))
    used[all1] <- TRUE
    used[all2] <- TRUE
    which(used)
}

.fast_overlap <- function(gi, ranges, ..., gi.is.query=TRUE)
# Overlaps regions(gi) with ranges, but only for those regions used as anchors.
{
    regs <- regions(gi)
    subset <- .get_used(gi)
    if (length(subset)!=length(regs)) { regs <- regs[subset] }

    # Need this switch, as type="within" will vary depending on manner of query vs. subject.
    if (gi.is.query) {
        olap <- findOverlaps(regs, ranges, select="all", ...)
        gi.dex <- subset[queryHits(olap)]
        ranges.dex <- subjectHits(olap)
    } else {
        olap <- findOverlaps(ranges, regs, select="all", ...)
        gi.dex <- subset[subjectHits(olap)]
        ranges.dex <- queryHits(olap)
        o <- order(gi.dex, ranges.dex)
        gi.dex <- gi.dex[o]
        ranges.dex <- ranges.dex[o]
    }

    return(list(gi.dex=gi.dex, ranges.dex=ranges.dex))
}

.get_olap_bounds <- function(hits, N)
# Gets the first and last index of the overlap vector for each GI index.
{
    current.rle <- rle(hits)
    first.in.rle <- rep(1L, N)
    last.in.rle <- integer(N)
    cum.end <- cumsum(current.rle$lengths)
    first.in.rle[current.rle$values] <- cum.end - current.rle$lengths + 1L
    last.in.rle[current.rle$values] <- cum.end
    return(list(first=first.in.rle, last=last.in.rle))
}

.decode_region_mode <- function(use.region, possibilities=c("both", "first", "second"))
# Gets the index of possibilities, for easy entry into C++.
# 1 -> both, 2 -> first, 3 -> second.
# This needs to be sync'd with set_mode_values in overlaps.cpp.
{
    use.region <- match.arg(use.region, possibilities)
    match(use.region, possibilities)
}

.linear_olap_finder <- function(gi, ranges, ..., select, gi.is.query=TRUE, use.region="both")
# Identifies linear overlaps, with differing C++ function depending on whether
# all overlaps are desired, or counting should be performed, etc.
{
    olap <- .fast_overlap(gi, ranges, ..., gi.is.query=gi.is.query)
    a1 <- anchors(gi, type="first", id=TRUE)
    a2 <- anchors(gi, type="second", id=TRUE)

    # Getting all combinations of overlaps (zero-indexing for C code).
    bounds <- .get_olap_bounds(olap$gi.dex, length(regions(gi)))
    out <- .Call(cxx_linear_olaps, a1 - 1L, a2 - 1L, bounds$first - 1L, bounds$last,
                 olap$ranges.dex - 1L, length(ranges), .decode_region_mode(use.region),
                 select, gi.is.query)

    # Processing into a Hits object if required.
    final <- out
    if (select=="all") {
        if (!gi.is.query) {
            final <- Hits(out[[2]]+1L, out[[1]]+1L, length(ranges), length(gi), sort.by.query=TRUE)
        } else {
            final <- Hits(out[[1]]+1L, out[[2]]+1L, length(gi), length(ranges), sort.by.query=TRUE)
        }
    } else if (select=="count") {
        if (gi.is.query) {
            names(final) <- names(gi)
        } else {
            names(final) <- names(ranges)
        }
    } else {
        final <- final + 1L # get rid of zero-indexing for first/last/arbitrary.
    }
    return(final)
}

setMethod("findOverlaps", c(query="GInteractions", subject="Vector"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             select=c("all", "first", "last", "arbitrary"),
             ignore.strand=TRUE, ..., use.region="both") {

        type <- match.arg(type)
        select <- match.arg(select)
        .linear_olap_finder(query, subject,
                    maxgap=maxgap, minoverlap=minoverlap, type=type,
                    select=select, ignore.strand=ignore.strand, ...,
                    gi.is.query=TRUE, use.region=use.region)
    }
)

setMethod("findOverlaps", c(query="Vector", subject="GInteractions"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             select=c("all", "first", "last", "arbitrary"),
             ignore.strand=TRUE, ..., use.region="both") {

        type <- match.arg(type)
        select <- match.arg(select)
        .linear_olap_finder(subject, query,
                    maxgap=maxgap, minoverlap=minoverlap, type=type,
                    select=select, ignore.strand=ignore.strand, ...,
                    gi.is.query=FALSE, use.region=use.region)
    }
)

###############################################################

.paired_overlap_finder2 <- function(gi.left, gi.right, ..., select, use.region="both")
# Identifies overlaps between two GI objects (left and right),
# with different C++ functions for different behaviours as before.
{
    used2 <- .get_used(gi.right)
    olap <- .fast_overlap(gi.left, regions(gi.right)[used2], ..., gi.is.query=TRUE)
    olap$ranges.dex <- used2[olap$ranges.dex]

    left.a1 <- anchors(gi.left, type="first", id=TRUE)
    left.a2 <- anchors(gi.left, type="second", id=TRUE)
    left.bounds <- .get_olap_bounds(olap$gi.dex, length(regions(gi.left)))

    right.a1 <- anchors(gi.right, type="first", id=TRUE)
    o1 <- order(right.a1)
    right.bounds1 <- .get_olap_bounds(right.a1[o1], length(regions(gi.right)))

    right.a2 <- anchors(gi.right, type="second", id=TRUE)
    o2 <- order(right.a2)
    right.bounds2 <- .get_olap_bounds(right.a2[o2], length(regions(gi.right)))

    # Getting all 2D overlaps.
    npairs <- length(gi.right)
    out <- .Call(cxx_paired_olaps, left.a1 - 1L, left.a2 - 1L,
                 left.bounds$first - 1L, left.bounds$last, olap$ranges.dex - 1L,
                 right.bounds1$first - 1L, right.bounds1$last, o1 - 1L,
                 right.bounds2$first - 1L, right.bounds2$last, o2 - 1L,
                 .decode_region_mode(use.region, c("both", "same", "reverse")), select)

    # Deciding what output to return.
    final <- out
    if (select=="all") {
        final <- Hits(out[[1]]+1L, out[[2]]+1L, length(gi.left), length(gi.right), sort.by.query=TRUE)
    } else if (select!="count") {
        final <- final + 1L
        names(final) <- names(gi.left)
    }
    return(final)
}

setMethod("findOverlaps", c(query="GInteractions", subject="GInteractions"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             select=c("all", "first", "last", "arbitrary"),
             ignore.strand=TRUE, ..., use.region="both") {

        type <- match.arg(type)
        select <- match.arg(select)
        .paired_overlap_finder2(query, subject, select=select,
                    maxgap=maxgap, minoverlap=minoverlap, type=type,
                    ignore.strand=ignore.strand, ..., use.region=use.region)
    }
)

setMethod("findOverlaps", c(query="GInteractions", subject="missing"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             select=c("all", "first", "last", "arbitrary"),
             ignore.strand=TRUE, ...,
             drop.self=FALSE, drop.redundant=FALSE,
             use.region="both") {

        type <- match.arg(type)
        out <- findOverlaps(query, query, maxgap=maxgap, minoverlap=minoverlap,
                            type=type, select="all", ignore.strand=ignore.strand,
                            ..., use.region=use.region)
        select <- match.arg(select)
        IRanges:::process_self_hits(out, select, drop.self, drop.redundant)
    }
)

###############################################################
# This defines the countOverlaps method.

setMethod("countOverlaps", c(query="GInteractions", subject="Vector"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region="both") {

        type <- match.arg(type)
        .linear_olap_finder(query, subject,
            maxgap=maxgap, minoverlap=minoverlap, type=type,
            ignore.strand=ignore.strand, ..., select="count",
            gi.is.query=TRUE, use.region=use.region)
    }
)

setMethod("countOverlaps", c(query="Vector", subject="GInteractions"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region="both") {

        type <- match.arg(type)
        .linear_olap_finder(subject, query,
            maxgap=maxgap, minoverlap=minoverlap, type=type,
            ignore.strand=ignore.strand, ..., select="count",
            gi.is.query=FALSE, use.region=use.region)
    }
)

setMethod("countOverlaps", c(query="GInteractions", subject="GInteractions"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region="both") {

        type <- match.arg(type)
        .paired_overlap_finder2(query, subject, maxgap=maxgap, minoverlap=minoverlap, type=type,
                    ignore.strand=ignore.strand, ..., select="count", use.region=use.region)
    }
)

setMethod("countOverlaps", c(query="GInteractions", subject="missing"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region='both') {

        type <- match.arg(type)
        hits <- findOverlaps(query, maxgap=maxgap, minoverlap=minoverlap, type=type,
                             ignore.strand=ignore.strand, ..., use.region=use.region)
        ans <- countQueryHits(hits)
        names(ans) <- names(query)
        ans
    }
)

###############################################################
# Defining corresponding functions for InteractionSet objects.

for (sig in c("Vector", "GInteractions")) { # Need to specify GInteractions to avoid ambiguous redirects with Vector.

    # Query is an InteractionSet.
    setMethod("countOverlaps", c("InteractionSet", sig),
        function(query, subject, maxgap=-1L, minoverlap=0L,
                 type=c("any", "start", "end", "within", "equal"),
                 ignore.strand=TRUE, ..., use.region='both') {
            type <- match.arg(type)
            countOverlaps(interactions(query), subject, maxgap=maxgap, minoverlap=minoverlap,
                          type=type, ignore.strand=ignore.strand, ..., use.region=use.region)
        }
    )

    setMethod("findOverlaps", c("InteractionSet", sig),
        function(query, subject, maxgap=-1L, minoverlap=0L,
                 type=c("any", "start", "end", "within", "equal"),
                 select=c("all", "first", "last", "arbitrary"),
                 ignore.strand=TRUE, ..., use.region='both') {
            select <- match.arg(select)
            type <- match.arg(type)
            findOverlaps(interactions(query), subject, maxgap=maxgap,
                minoverlap=minoverlap, type=type, select=select,
                ignore.strand=ignore.strand, ..., use.region=use.region)
        }
    )

    # Subject is an InteractionSet.
    setMethod("countOverlaps", c(sig, "InteractionSet"),
        function(query, subject, maxgap=-1L, minoverlap=0L,
                 type=c("any", "start", "end", "within", "equal"),
                 ignore.strand=TRUE, ..., use.region='both') {
            type <- match.arg(type)
            countOverlaps(query, interactions(subject), maxgap=maxgap, minoverlap=minoverlap,
                          type=type, ignore.strand=ignore.strand, ..., use.region=use.region)
        }
    )

    setMethod("findOverlaps", c(sig, "InteractionSet"),
        function(query, subject, maxgap=-1L, minoverlap=0L,
                 type=c("any", "start", "end", "within", "equal"),
                 select=c("all", "first", "last", "arbitrary"),
                 ignore.strand=TRUE, ..., use.region='both') {
            select <- match.arg(select)
            type <- match.arg(type)
            findOverlaps(query, interactions(subject), maxgap=maxgap,
                         minoverlap=minoverlap, type=type, select=select,
                         ignore.strand=ignore.strand, ..., use.region=use.region)
        }
    )
}

# Both arguments are InteractionSet objects.
setMethod("countOverlaps", c("InteractionSet", "InteractionSet"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region='both') {
        type <- match.arg(type)
        countOverlaps(interactions(query), interactions(subject), maxgap=maxgap, minoverlap=minoverlap,
                      type=type, ignore.strand=ignore.strand, ..., use.region=use.region)
    }
)

setMethod("findOverlaps", c("InteractionSet", "InteractionSet"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             select=c("all", "first", "last", "arbitrary"),
             ignore.strand=TRUE, ..., use.region='both') {
        select <- match.arg(select)
        type <- match.arg(type)
        findOverlaps(interactions(query), interactions(subject), maxgap=maxgap, minoverlap=minoverlap,
                     type=type, select=select, ignore.strand=ignore.strand, ..., use.region=use.region)
    }
)

# Missing the subject.
setMethod("countOverlaps", c("InteractionSet", "missing"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region='both') {
        type <- match.arg(type)
        countOverlaps(interactions(query),maxgap=maxgap, minoverlap=minoverlap, type=type,
                      ignore.strand=ignore.strand, ..., use.region=use.region)
    }
)

setMethod("findOverlaps", c("InteractionSet", "missing"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             select=c("all", "first", "last", "arbitrary"),
             ignore.strand=TRUE, ..., use.region='both') {
        select <- match.arg(select)
        type <- match.arg(type)
        findOverlaps(interactions(query), maxgap=maxgap, minoverlap=minoverlap, type=type,
                     select=select, ignore.strand=ignore.strand, ..., use.region=use.region)
    }
)

###############################################################
# Defining overlapsAny for ContactMatrix objects.

setMethod("overlapsAny", c("ContactMatrix", "Vector"),
    function(query, subject, maxgap=-1L, minoverlap=0L,
        type=c("any", "start", "end", "within", "equal"),
        ignore.strand=TRUE, ...) {
        a1 <- anchors(query, id=TRUE, type="row")
        a2 <- anchors(query, id=TRUE, type="column")

        is.used <- union(a1, a2)
        is.overlapped <- logical(length(regions(query)))
        is.overlapped[is.used] <- overlapsAny(regions(query)[is.used], subject, maxgap=maxgap, minoverlap=minoverlap,
                                              type=type, ignore.strand=ignore.strand, ...)
        return(list(row=is.overlapped[a1], column=is.overlapped[a2]))
})

# Use outer(output$row, output$column, "|" or "&") to get the logical area in the interaction space.
# Not sure it makes a great deal of sense to define 'findOverlaps' here.

for (sig in c("GInteractions", "InteractionSet")) {
    setMethod("overlapsAny", c("ContactMatrix", sig),
        function(query, subject, maxgap=-1L, minoverlap=0L,
             type=c("any", "start", "end", "within", "equal"),
             ignore.strand=TRUE, ..., use.region='both') {

        # It's possible to do this more efficiently by avoiding instantiation of the full object.
        # But it would require a total re-implementation at the C++ level, which is a pain.
        row.a <- rep(anchors(query, type="row", id=TRUE), ncol(query))
        col.a <- rep(anchors(query, type="column", id=TRUE), each=nrow(query))
        new.query <- GInteractions(row.a, col.a, regions(query))
        out <- overlapsAny(new.query, subject, maxgap=maxgap, minoverlap=minoverlap, type=type,
                           ignore.strand=ignore.strand, ..., use.region=use.region)
        dim(out) <- dim(query)
        return(out)
    })
}

# Haven't defined the converse methods, as it's not clear whether you want to consider the entire
# interaction space in the ContactMatrix, or just the non-NA entries.

###############################################################
# End

###############################################################

setValidity2("InteractionSet", function(object) {
    if (nrow(object)!=length(interactions(object))) {
        return("'interactions' length is not equal to the number of rows")
    } 
    if (!is.null(object@NAMES)) { # Using direct slot access, otherwise diverts to slots in GInteractions.
        return("'NAMES' slot must always be NULL")
    }
    if (ncol(object@elementMetadata) != 0L) {
        return("'elementMetadata' slot must always have zero columns")
    }
    return(TRUE)
})

setMethod("parallel_slot_names", "InteractionSet", function(x) {
    c("interactions", callNextMethod()) 
})

setMethod("show", signature("InteractionSet"), function(object) {
    callNextMethod()
    cat(sprintf("type: %s\n", class(interactions(object))))
    cat(sprintf("regions: %i\n", length(regions(interactions(object)))))
})

###############################################################
# Constructors

.new_InteractionSet <- function(assays, interactions, ...) {
    se0 <- SummarizedExperiment(assays, ...)
    new("InteractionSet", se0, interactions=interactions)
}

setMethod("InteractionSet", c("ANY", "GInteractions"), function(assays, interactions, ...) { 
        .new_InteractionSet(assays, interactions, ...)
   }
)

setMethod("InteractionSet", c("missing", "missing"), function(assays, interactions, ...) {
        .new_InteractionSet(list(), GInteractions(), ...)
   }
)

###############################################################
# Subsetting

# Need to define these because SummarizedExperiment doesn't use extract/replaceROWS directly;
# they divert to these functions anyway.
setMethod("[", c("InteractionSet", "ANY", "ANY"), function(x, i, j, ..., drop=TRUE) {
    if (!missing(i)) { unchecked_interactions(x) <- interactions(x)[i] }
    callNextMethod()
})

setMethod("[<-", c("InteractionSet", "ANY", "ANY", "InteractionSet"), function(x, i, j, ..., value) {
    if (!missing(i)) { 
        I <- interactions(x)
        I[i] <- interactions(value) 
        unchecked_interactions(x) <- I
    }
    callNextMethod(x=x, i=i, j=j, ..., value=value)
})

setMethod("subset", "InteractionSet", function(x, i, j) {
    x[i, j]
})

###############################################################
# Combining

setMethod("cbind", "InteractionSet", function(..., deparse.level=1) {
    args <- unname(list(...))
    ans <- args[[1]]
    ref.inters <- interactions(ans)
    ref.length <- length(ans)

    for (x in args[-1]) {
        if (ref.length!=length(x) || any(interactions(x)!=ref.inters)) { 
            # Possible to cbind for different metadata here, but I doubt that gets much use.
            stop("interactions must be identical in 'cbind'") 
        }
    }
    
    base <- do.call(cbind, lapply(args, function(x) { as(x, "SummarizedExperiment") }))
    new("InteractionSet", base, interactions=interactions(ans))
})

setMethod("rbind", "InteractionSet", function(..., deparse.level=1) {
    args <- unname(list(...))
    base <- do.call(rbind, lapply(args, function(x) { as(x, "SummarizedExperiment") }))
    new("InteractionSet", base, interactions=do.call(c, lapply(args, FUN=interactions)))
})

###############################################################
# Other methods

setMethod("order", "InteractionSet", function(..., na.last=TRUE, decreasing=FALSE) {
    do.call(order, c(lapply(list(...), interactions), list(na.last=na.last, decreasing=decreasing)))
})

setMethod("duplicated", "InteractionSet", function(x, incomparables=FALSE, fromLast=FALSE, ...) {
    duplicated(interactions(x))
})

###############################################################
# End

# getset.R

setGeneric("anchors", function(x, ...) standardGeneric("anchors"))
setGeneric("anchorIds", function(x, ...) standardGeneric("anchorIds"))

setGeneric("anchor1", function(x) standardGeneric("anchor1"))
setGeneric("anchor2", function(x) standardGeneric("anchor2"))
setGeneric("unchecked_anchor1<-", function(x, value) standardGeneric("unchecked_anchor1<-"))
setGeneric("unchecked_anchor2<-", function(x, value) standardGeneric("unchecked_anchor2<-"))

setGeneric("regions", function(x, ...) standardGeneric("regions"))
setGeneric("unchecked_regions<-", function(x, value) standardGeneric("unchecked_regions<-"))
setGeneric("regions<-", function(x, value) standardGeneric("regions<-"))
setGeneric("replaceRegions<-", function(x, value) standardGeneric("replaceRegions<-"))
setGeneric("appendRegions<-", function(x, value) standardGeneric("appendRegions<-"))
setGeneric("reduceRegions", function(x) standardGeneric("reduceRegions"))

setGeneric("anchorIds<-", function(x, ..., value) standardGeneric("anchorIds<-"))

setGeneric("interactions", function(x, ...) standardGeneric("interactions"))
setGeneric("unchecked_interactions<-", function(x, value) standardGeneric("unchecked_interactions<-"))
setGeneric("interactions<-", function(x, value) standardGeneric("interactions<-"))

setGeneric("unchecked_matrix<-", function(x, value) standardGeneric("unchecked_matrix<-"))
setGeneric("as.matrix<-", function(x, ..., value) standardGeneric("as.matrix<-"))

# boundingBox.R

setGeneric("boundingBox", function(x, f) standardGeneric("boundingBox"))

# ContactMatrix-class.R

setGeneric("ContactMatrix", function(matrix, anchor1, anchor2, regions, ...) standardGeneric("ContactMatrix"))

# conversion.R

setGeneric("inflate", function(x, ...) standardGeneric("inflate"))
setGeneric("deflate", function(x, ...) standardGeneric("deflate"))

# distances.R

setGeneric("pairdist", function(x, ...) standardGeneric("pairdist"))
setGeneric("intrachr", function(x) standardGeneric("intrachr"))

# GInteractions-class.R

setGeneric("GInteractions", function(anchor1, anchor2, regions, ...) standardGeneric("GInteractions"))

# InteractionSet-class.R

setGeneric("InteractionSet", function(assays, interactions, ...) standardGeneric("InteractionSet"))

# linearize.R

setGeneric("linearize", function(x, ref, ...) standardGeneric("linearize"))

# linkOverlaps.R

setGeneric("linkOverlaps", function(query, subject1, subject2, ...) standardGeneric('linkOverlaps'))

# pairs.R

setGeneric("pairs", function(x, ...) standardGeneric("pairs"))

# swapAnchors.R

setGeneric("swapAnchors", function(x, ...) { standardGeneric("swapAnchors") })

##############################################
# Setting validity and show methods.

setValidity2("ContactMatrix", function(object) {
    if (is.unsorted(regions(object))) {
        return("'regions' should be sorted")
    }
    msg <- .check_inputs(anchor1(object), anchor2(object), regions(object), same.length=FALSE)
    if (is.character(msg)) { return(msg) }

    mat <- as.matrix(object)
    if (length(dim(mat))!=2L) {
        return("'matrix' slot should contain a matrix-like object with 2 dimensions")
    }
    if (nrow(mat)!=length(anchor1(object))) { 
        return("'matrix' nrow must be equal to length of 'anchor1'")
    }
    if (ncol(mat)!=length(anchor2(object))) {
        return("'matrix' ncol must be equal to length of 'anchor2'")
    }
    return(TRUE)
}) 

setMethod("show", signature("ContactMatrix"), function(object) {
    cat("class:", class(object)[1], "\n")
    cat("dim:", dim(as.matrix(object)), "\n")
    cat("type:", class(as.matrix(object))[1], "\n")

    rnames <- rownames(object)
    if (!is.null(rnames)) scat("rownames(%d): %s\n", rnames)
    else cat("rownames: NULL\n")

    cnames <- colnames(object)
    if (!is.null(cnames)) scat("colnames(%d): %s\n", cnames)
    else cat("colnames: NULL\n")

    expt <- names(metadata(object))
    if (is.null(expt))
        expt <- character(length(metadata(object)))
    scat("metadata(%d): %s\n", expt)

    cat(sprintf("regions: %i\n", length(regions(object))))
})

##############################################
# Constructor:

.new_ContactMatrix <- function(matrix, anchor1, anchor2, regions, metadata) {
    anchor1 <- as.integer(anchor1)
    anchor2 <- as.integer(anchor2)
    
    msg <- .check_inputs(anchor1, anchor2, regions, same.length=FALSE)
    if (is.character(msg)) { stop(msg) }
    out <- .resort_regions(anchor1, anchor2, regions)

    new("ContactMatrix", matrix=matrix, anchor1=out$anchor1, anchor2=out$anchor2, 
        regions=out$regions, metadata=metadata)
}

setMethod("ContactMatrix", c("ANY", "numeric", "numeric", "GRanges"), 
    function(matrix, anchor1, anchor2, regions, metadata=list()) { 
        .new_ContactMatrix(matrix, anchor1, anchor2, regions, metadata)
    }
)

setMethod("ContactMatrix", c("ANY", "GRanges", "GRanges", "GenomicRanges_OR_missing"), 
    function(matrix, anchor1, anchor2, regions, metadata=list()) { 

        if (missing(regions)) { 
            collated <- .collate_GRanges(anchor1, anchor2)
            regions <- collated$ranges
            anchor1 <- collated$indices[[1]]
            anchor2 <- collated$indices[[2]]
        } else {
            anchor1 <- match(anchor1, regions)
            anchor2 <- match(anchor2, regions)
            if (any(is.na(anchor1)) || any(is.na(anchor2))) {
                 stop("anchor regions missing in specified 'regions'")
            }
        }
        
        .new_ContactMatrix(matrix, anchor1, anchor2, regions, metadata)
    }
)

setMethod("ContactMatrix", c("missing", "missing", "missing", "GenomicRanges_OR_missing"),
    function(matrix, anchor1, anchor2, regions, metadata=list()) {
        if (missing(regions)) { regions <- GRanges() }
        .new_ContactMatrix(base::matrix(0L, 0, 0), integer(0), integer(0), regions, metadata)
    } 
)

##############################################
# Subsetting

setMethod("[", c("ContactMatrix", "ANY", "ANY"), function(x, i, j, ..., drop=TRUE) {
    if (!missing(i)) { 
        unchecked_anchor1(x) <- anchor1(x)[i]
        unchecked_matrix(x) <- as.matrix(x)[i,,drop=FALSE]
    }
    if (!missing(j)) {
        unchecked_anchor2(x) <- anchor2(x)[j]
        unchecked_matrix(x) <- as.matrix(x)[,j,drop=FALSE]
    }
    return(x)
}) 

setMethod("[<-", c("ContactMatrix", "ANY", "ANY", "ContactMatrix"), function(x, i, j, ..., value) {
    if (!identical(regions(value), regions(x))) { 
        stop("replacement and original 'regions' must be identical")
    }
    
    a1 <- anchor1(x)
    a2 <- anchor2(x)
    m <- as.matrix(x)

    if (!missing(i) && !missing(j)) {
        if (!identical(anchor1(x)[i], anchor1(value))) {
            stop("cannot modify row indices for a subset of columns")
        }
        if (!identical(anchor2(x)[j], anchor2(value))) {
            stop("cannot modify column indices for a subset of rows")
        }
        m[i,j] <- as.matrix(value)
    } else if (!missing(i)) { 
        a1[i] <- anchor1(value)
        m[i,] <- as.matrix(value)
    } else if (!missing(j)) { 
        a2[j] <- anchor2(value)
        m[,j] <- as.matrix(value)
    }

    unchecked_anchor1(x) <- a1
    unchecked_anchor2(x) <- a2
    unchecked_matrix(x) <- m
    return(x)
})

setMethod("subset", "ContactMatrix", function(x, i, j) {
    x[i,j]
})

##############################################
# Combining

setMethod("cbind", "ContactMatrix", function(..., deparse.level=1) {
    incoming <- list(...)
    ref <- incoming[[1]]
    all.regions <- lapply(incoming, FUN=regions)
    all.anchor1 <- lapply(incoming, FUN=slot, name="anchor1")
    all.anchor2 <- lapply(incoming, FUN=slot, name="anchor2")

    # Checking if regions are the same; collating if not.
    unified <- .coerce_to_union(all.regions, all.anchor1, all.anchor2)
    unchecked_regions(ref) <- unified$region
    unchecked_anchor1(ref) <- unified$anchor1[[1]]

    for (x in unified$anchor1[-1]) {
        if (!identical(anchor1(ref), x)) {
            stop("row anchor indices must be identical for 'cbind'")
        }    
    }

    unchecked_matrix(ref) <- do.call(cbind, lapply(incoming, as.matrix))
    unchecked_anchor2(ref) <- unlist(unified$anchor2)
    return(ref)
})

setMethod("rbind", "ContactMatrix", function(..., deparse.level=1) {
    incoming <- list(...)
    ref <- incoming[[1]]
    all.regions <- lapply(incoming, FUN=regions)
    all.anchor1 <- lapply(incoming, FUN=slot, name="anchor1")
    all.anchor2 <- lapply(incoming, FUN=slot, name="anchor2")

    # Checking if regions are the same; collating if not.
    unified <- .coerce_to_union(all.regions, all.anchor1, all.anchor2)
    unchecked_regions(ref) <- unified$region
    unchecked_anchor2(ref) <- unified$anchor2[[1]]

    for (x in unified$anchor2[-1]) { 
        if (!identical(anchor2(ref), x)) { 
            stop("column anchor indices must be identical for 'rbind'")
        }    
    }
    
    unchecked_matrix(ref) <- do.call(rbind, lapply(incoming, as.matrix))
    unchecked_anchor1(ref) <- unlist(unified$anchor1)
    return(ref)
})

setMethod("t", "ContactMatrix", function(x) { 
    unchecked_matrix(x) <- t(as.matrix(x))
    tmp <- anchor1(x)
    unchecked_anchor1(x) <- anchor2(x)
    unchecked_anchor2(x) <- tmp
    return(x)
})

##############################################
# Sorting and ordering

setMethod("order", "ContactMatrix", function(..., na.last=TRUE, decreasing=FALSE) {
    incoming <- list(...)
    all.rows <- lapply(incoming, anchors, type="row", id=TRUE)
    all.columns <- lapply(incoming, anchors, type="column", id=TRUE)
    list(row=do.call(order, c(all.rows, na.last=na.last, decreasing=decreasing)),
         column=do.call(order, c(all.columns, na.last=na.last, decreasing=decreasing)))
})

setMethod("sort", "ContactMatrix", function(x, decreasing=FALSE, ...) {
    out <- order(x, decreasing=decreasing)
    x[out$row, out$column]
})

setMethod("duplicated", "ContactMatrix", function(x, incomparables=FALSE, ...) {
    r1 <- duplicated(anchor1(x), incomparables=incomparables, ...)
    r2 <- duplicated(anchor2(x), incomparables=incomparables, ...)
    return(list(row=r1, column=r2))
})

setMethod("unique", "ContactMatrix", function(x, incomparables=FALSE, ...) {
    is.dup <- duplicated(x, incomparables=incomparables, ...)
    return(x[!is.dup$row,!is.dup$column])
})

##############################################
# End

.linearize <- function(x, ref, internal=TRUE) {
    ref <- as.integer(ref)
    keep.a1 <- anchors(x, type="first", id=TRUE) %in% ref
    keep.a2 <- anchors(x, type="second", id=TRUE) %in% ref

    if (!internal) {
        keep <- keep.a1 != keep.a2
    } else {
        keep <- keep.a1 | keep.a2
    }
    
    x <- x[keep,]
    new.range.x <- anchor1(x)
    keep.a1 <- keep.a1[keep]
    new.range.x[keep.a1] <- anchor2(x)[keep.a1] # Replace anchor1 matches to "ref" with (mostly) non-ref anchor2.
    new.ranges <- regions(x)[new.range.x]

    if (internal) {
        keep.a2 <- keep.a2[keep]
        is.same <- keep.a1 & keep.a2
        expanded <- range(pairs(x[is.same], as.grlist=TRUE)) # replace internal combos with ranges.
        if (any(lengths(expanded)>1L)) { stop("multi-chromosome sets of 'ref' are not supported") }
        new.ranges[is.same] <- unlist(expanded)
    }

    metadata(new.ranges) <- metadata(x)    
    mcols(new.ranges) <- cbind(mcols(new.ranges), mcols(x))   
    return(list(ranges=new.ranges, keep=keep))
}

setMethod("linearize", c("GInteractions", "numeric"), function(x, ref, internal=TRUE) {
    out <- .linearize(x, ref, internal=internal)
    return(out$ranges)
})

.chooseRegion <- function(x, ref, ...) {
    which(overlapsAny(regions(x), ref, ...))
}

setMethod("linearize", c("GInteractions", "GRanges"), function(x, ref, ..., internal=TRUE) {
    i <- .chooseRegion(x, ref, ...)          
    linearize(x, i, internal=internal)
})

.ISetToRSE <- function(x, out) {
    new("RangedSummarizedExperiment", x[out$keep,], rowRanges=out$ranges)
}

setMethod("linearize", c("InteractionSet", "numeric"), function(x, ref, internal=TRUE) {
    y <- interactions(x)
    out <- .linearize(y, ref, internal=internal)
    .ISetToRSE(x, out)
})

setMethod("linearize", c("InteractionSet", "GRanges"), function(x, ref, ..., internal=TRUE) {
    y <- interactions(x)
    i <- .chooseRegion(x, ref, ...)          
    out <- .linearize(y, i, internal=internal)
    .ISetToRSE(x, out)
})

# Tests the construction and manipulation of GInteractions objects.
# library(InteractionSet); library(testthat); source("test-compare.R")

set.seed(8000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
x <- GInteractions(all.anchor1, all.anchor2, all.regions)

ref.match <- function(x, y) {
    match(do.call(paste, c(anchors(x, id=TRUE), sep=".")), 
          do.call(paste, c(anchors(y, id=TRUE), sep=".")))
}

#####################################

test_that("matching between GI objects works", {
    another.x <- x[sample(Np)] 
    
    expect_identical(match(x, another.x), ref.match(x, another.x))
    expect_identical(match(x, another.x[1:5]), ref.match(x, another.x[1:5]))
    expect_identical(match(x[20:10], another.x), ref.match(x[20:10], another.x))
    
    expect_identical(match(x[0], another.x), integer(0))
    expect_identical(match(x, another.x[0]), rep(as.integer(NA), Np))

    # Testing unequal regions, generated by appending regions to the front or end.
    ref <- match(x, another.x)
    more.x <- another.x
    suppressWarnings(appendRegions(more.x) <- GRanges("chrC", IRanges(1, 1)))
    expect_identical(ref, match(x, more.x))

    more.x <- another.x
    suppressWarnings(replaceRegions(more.x) <- c(GRanges("achr", IRanges(1, 1)), all.regions)) # inserts at front.
    expect_identical(ref, match(x, more.x))

    # Testing via the %in% operator.
    expect_identical(x %in% another.x, !is.na(match(x, another.x)))
    expect_identical(x[20:10] %in% another.x, !is.na(match(x[20:10], another.x)))
})

test_that("matching between ISet objects works", {
    iset <- InteractionSet(matrix(runif(Np), dimnames=list(NULL, 1)), x)
    another.x <- x[sample(Np)] 
    iset2 <- InteractionSet(matrix(runif(Np), dimnames=list(NULL, 1)), another.x)
    
    # Testing ISet to GI and vice versa.
    expect_identical(match(iset, x), ref.match(iset, x))
    expect_identical(match(iset, another.x), ref.match(iset, another.x))
    expect_identical(match(iset2, x), ref.match(iset2, x))
    expect_identical(match(x, iset), ref.match(x, iset))
    expect_identical(match(another.x, iset), ref.match(another.x, iset))
    expect_identical(match(x, iset2), ref.match(x, iset2))
    expect_identical(match(iset, iset2), ref.match(iset, iset2))

    # Testing various subsets.
    expect_identical(match(iset[10:15], another.x), ref.match(iset[10:15], another.x))
    expect_identical(match(another.x, iset[10:15]), ref.match(another.x, iset[10:15]))
    expect_identical(match(iset, another.x[20:6]), ref.match(iset, another.x[20:6]))
    expect_identical(match(another.x[20:6], iset), ref.match(another.x[20:6], iset))
    expect_identical(match(iset, iset2[1:6,]), ref.match(iset, iset2[1:6,]))

    # Testing via %in%.
    expect_identical(iset %in% another.x, !is.na(ref.match(iset, another.x)))
    expect_identical(iset %in% iset2, !is.na(ref.match(iset, iset2)))
    expect_identical(another.x %in% iset, !is.na(ref.match(another.x, iset)))
})

#####################################

test_that("comparisons between GI objects works", {
    another.x <- x[sample(Np)] 
    expect_identical(pcompare(x, another.x), ifelse(x@anchor1==another.x@anchor1, x@anchor2-another.x@anchor2, x@anchor1-another.x@anchor1))
    sub.x <- x[3:12] # Recycle the vector...
    expect_identical(pcompare(sub.x, another.x), ifelse(sub.x@anchor1==another.x@anchor1, sub.x@anchor2-another.x@anchor2, sub.x@anchor1-another.x@anchor1))
    expect_identical(pcompare(another.x, sub.x), -pcompare(sub.x, another.x))
    expect_identical(pcompare(x[0], another.x[0]), integer(0))
    
    old <- pcompare(x, another.x)
    expect_identical(x==x, !logical(length(x)))
    expect_identical(x!=x, logical(length(x)))
    expect_identical(x==another.x, old==0L)
    
    # Altering regions.
    more.x <- another.x
    regions(more.x)$whee <- 1
    expect_identical(pcompare(x, more.x), old) # This should be okay, as metadata is ignored.

    ref <- pcompare(x, another.x)
    more.x <- another.x
    suppressWarnings(appendRegions(more.x) <- GRanges("chrC", IRanges(1, 1)))
    expect_identical(ref, pcompare(x, more.x))

    more.x <- another.x
    suppressWarnings(replaceRegions(more.x) <- c(GRanges("achr", IRanges(1, 1)), all.regions)) # inserts at front.
    expect_identical(ref, pcompare(x, more.x))
})

test_that("comparisons between different strictness levels triggers warnings", {
    sx <- as(swapAnchors(x), "StrictGInteractions")
    expect_warning(sx==x, "comparison between GInteractions objects of different strictness")
    rsx <- as(swapAnchors(x, mode="reverse"), "ReverseStrictGInteractions")
    expect_warning(rsx==sx, "comparison between GInteractions objects of different strictness")
    expect_warning(rsx==x, "comparison between GInteractions objects of different strictness")
    expect_warning(match(rsx, x), "comparison between GInteractions objects of different strictness")
})

# Tests the trimming of GInteractions objects.
# library(InteractionSet); library(testthat); source("test-grmeth.R")

set.seed(10000)

N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 50))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
x <- GInteractions(all.anchor1, all.anchor2, all.regions)

test_that("trimming works for all objects", {
    suppressWarnings(seqlengths(x) <- c(chrA=50, chrB=100))
    x2 <- trim(x)
    ref <- trim(regions(x))

    expect_identical(anchors(x2, type="first"), ref[anchors(x, type="first", id=TRUE)])
    expect_identical(anchors(x2, type="second"), ref[anchors(x, type="second", id=TRUE)])
    expect_false(is.unsorted(regions(x2)))
    
    iset <- InteractionSet(matrix(0, nrow=Np, ncol=2), x)
    iset2 <- trim(iset)
    expect_identical(class(iset2), class(iset))
    expect_identical(interactions(iset2), x2)

    # Now for CM objects.
    y <- inflate(x, "chrA", "chrB")
    y2 <- trim(y)

    expect_identical(class(y), class(y2))
    expect_identical(anchors(y2, type="row"), ref[anchors(y, type="row", id=TRUE)])
    expect_identical(anchors(y2, type="column"), ref[anchors(y, type="column", id=TRUE)])
})

test_that("resizing works for GI, IS objects", {
    # Common resizing operation.
    new.size <- 30
    suppressWarnings(x2 <- resize(x, fix="center", width=new.size))
    ref <- resize(regions(x), fix="center", width=new.size)

    expect_identical(anchors(x2, type="first"), ref[anchors(x, type="first", id=TRUE)])
    expect_identical(anchors(x2, type="second"), ref[anchors(x, type="second", id=TRUE)])
    expect_false(is.unsorted(regions(x2)))

    # Checking that this works for ISet objects.    
    iset <- InteractionSet(matrix(0, nrow=Np, ncol=2), x)
    iset2 <- resize(iset, fix="center", width=new.size)
    expect_identical(class(iset2), class(iset))
    expect_identical(interactions(iset2), x2)
    
    # Variable resizing operation.
    new.size <- round(runif(Np, 10, 50))
    suppressWarnings(x2 <- resize(x, fix="center", width=new.size))
    expect_identical(anchors(x2, type="first"), resize(anchors(x, type="first"), width=new.size, fix="center"))
    expect_identical(anchors(x2, type="second"), resize(anchors(x, type="second"), width=new.size, fix="center"))
    expect_false(is.unsorted(regions(x2)))
    
    # Highly variable resizing.
    new.size1 <- round(runif(Np, 10, 50))
    new.size2 <- round(runif(Np, 10, 50))
    suppressWarnings(x2 <- resize(x, fix="center", width=list(new.size1, new.size2)))
    expect_identical(anchors(x2, type="first"), resize(anchors(x, type="first"), width=new.size1, fix="center"))
    expect_identical(anchors(x2, type="second"), resize(anchors(x, type="second"), width=new.size2, fix="center"))
    expect_false(is.unsorted(regions(x2)))  

    # Checking that a warning is raised when we recycle.
    expect_warning(x2 <- resize(x, fix="center", width=1:3), "not a multiple")
    expect_warning(x2 <- resize(x, fix="center", width=1:1001), "not a multiple")
})

test_that("resizing works for CM objects", {
    # Now for CM objects.
    new.size <- 30
    y <- inflate(x, "chrA", "chrB")
    y2 <- resize(y, fix="center", width=new.size)
    ref <- resize(regions(x), fix="center", width=new.size)

    expect_identical(class(y), class(y2))
    expect_identical(anchors(y2, type="row"), ref[anchors(y, type="row", id=TRUE)])
    expect_identical(anchors(y2, type="column"), ref[anchors(y, type="column", id=TRUE)])

    # Variable resizing.
    new.size1 <- round(runif(nrow(y), 10, 50))
    new.size2 <- round(runif(ncol(y), 10, 50))
    suppressWarnings(y2 <- resize(y, fix="center", width=list(new.size1, new.size2)))
    expect_identical(anchors(y2, type="row"), resize(anchors(y, type="row"), width=new.size1, fix="center"))
    expect_identical(anchors(y2, type="column"), resize(anchors(y, type="column"), width=new.size2, fix="center"))
    expect_false(is.unsorted(regions(y2)))  
})

# We won't bother to check variable values for the rest, as the underlying code is exactly the same as resize().

test_that("narrowing works for all objects", {
    suppressWarnings(x2 <- narrow(x, start=3))
    ref <- narrow(regions(x), start=3)

    expect_identical(anchors(x2, type="first"), ref[anchors(x, type="first", id=TRUE)])
    expect_identical(anchors(x2, type="second"), ref[anchors(x, type="second", id=TRUE)])
    expect_false(is.unsorted(regions(x2)))

    iset <- InteractionSet(matrix(0, nrow=Np, ncol=2), x)
    iset2 <- narrow(iset, start=3)
    expect_identical(class(iset2), class(iset))
    expect_identical(interactions(iset2), x2)

    # Now for CM objects.
    y <- inflate(x, "chrA", "chrB")
    y2 <- narrow(y, start=3)

    expect_identical(class(y), class(y2))
    expect_identical(anchors(y2, type="row"), ref[anchors(y, type="row", id=TRUE)])
    expect_identical(anchors(y2, type="column"), ref[anchors(y, type="column", id=TRUE)])
})

test_that("shifting works for all objects", {
    suppressWarnings(x2 <- shift(x, shift=10))
    ref <- shift(regions(x), shift=10)

    expect_identical(anchors(x2, type="first"), ref[anchors(x, type="first", id=TRUE)])
    expect_identical(anchors(x2, type="second"), ref[anchors(x, type="second", id=TRUE)])
    expect_false(is.unsorted(regions(x2)))
    
    iset <- InteractionSet(matrix(0, nrow=Np, ncol=2), x)
    iset2 <- shift(iset, shift=10)
    expect_identical(class(iset2), class(iset))
    expect_identical(interactions(iset2), x2)

    # Now for CM objects.
    y <- inflate(x, "chrA", "chrB")
    y2 <- shift(y, shift=10)

    expect_identical(class(y), class(y2))
    expect_identical(anchors(y2, type="row"), ref[anchors(y, type="row", id=TRUE)])
    expect_identical(anchors(y2, type="column"), ref[anchors(y, type="column", id=TRUE)])
})

test_that("flanking works for all objects", {
    suppressWarnings(x2 <- flank(x, width=10))
    ref <- flank(regions(x), width=10)

    expect_identical(anchors(x2, type="first"), ref[anchors(x, type="first", id=TRUE)])
    expect_identical(anchors(x2, type="second"), ref[anchors(x, type="second", id=TRUE)])
    expect_false(is.unsorted(regions(x2)))
    
    iset <- InteractionSet(matrix(0, nrow=Np, ncol=2), x)
    iset2 <- flank(iset, width=10)
    expect_identical(class(iset2), class(iset))
    expect_identical(interactions(iset2), x2)

    # Now for CM objects.
    y <- inflate(x, "chrA", "chrB")
    y2 <- flank(y, width=10)

    expect_identical(class(y), class(y2))
    expect_identical(anchors(y2, type="row"), ref[anchors(y, type="row", id=TRUE)])
    expect_identical(anchors(y2, type="column"), ref[anchors(y, type="column", id=TRUE)])
})


test_that("width calculations work for all objects", {
    expect_identical(width(x), list(first=width(anchors(x, type="first")), second=width(anchors(x, type="second"))))
    
    iset <- InteractionSet(matrix(0, nrow=Np, ncol=2), x)
    expect_identical(width(iset), width(x))

    y <- inflate(x, "chrA", "chrB")
    expect_identical(width(y), list(row=width(anchors(y, type="row")), column=width(anchors(y, type="column"))))
})

# Tests the linkOverlaps method
# library(InteractionSet); library(testthat); source("test-link.R")

set.seed(9000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.strand <- sample(c("*", "+", "-"), N, replace=TRUE)
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends), strand=all.strand)

Np <- 100
all.anchor1 <- sample(N, Np, replace=TRUE)
all.anchor2 <- sample(N, Np, replace=TRUE)
x <- GInteractions(all.anchor1, all.anchor2, all.regions)

# Generating some random regions to test against.

Ngenes <- 10
gene.starts <- round(runif(Ngenes, 1, 100))
gene.ends <- gene.starts + round(runif(Ngenes, 5, 20))
gene.strand <- sample(c("*", "+", "-"), Ngenes, replace=TRUE)
gene.regions <- GRanges(rep(c("chrA", "chrB"), Ngenes/2), IRanges(gene.starts, gene.ends), strand=gene.strand)

Nenh <- 8
enh.starts <- round(runif(Nenh, 1, 100))
enh.ends <- enh.starts + round(runif(Nenh, 5, 20))
enh.strand <- sample(c("*", "+", "-"), Nenh, replace=TRUE)
enh.regions <- GRanges(rep(c("chrA", "chrB"), Nenh/2), IRanges(enh.starts, enh.ends), strand=enh.strand)

# Testing for ISets and GIs, in a range of scenarios.

for (cls in 1:2) { 
    if (cls==1L) {
        obj <- x
    } else {
        obj <- InteractionSet(matrix(0, Np, 4, dimnames=list(NULL, seq_len(4))), x)
    }
    
    for (param in seq_len(7)) {
        type <- "any"
        maxgap <- -1L
        minoverlap <- 0L
        use.region <- "both"
        ignore.strand <- TRUE

        if (param==2L) { 
            maxgap <- 10L
        } else if (param==3L) {
            minoverlap <- 10L
        } else if (param==4L) {
            type <- "within"
        } else if (param==5L) {
            use.region <- "same"
        } else if (param==6L) {
            use.region <- "reverse"
        } else if (param==7L) {
            ignore.strand <- FALSE
        }

        test_that(sprintf("linking overlaps works between two regions (%i, %i)", cls, param), {
            # Getting the reference results by doing it manually via 'merge'.
            olap1.g <- findOverlaps(anchors(obj, type="first"), gene.regions, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand, type=type)
            olap2.g <- findOverlaps(anchors(obj, type="second"), gene.regions, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand, type=type)
            olap1.e <- findOverlaps(anchors(obj, type="first"), enh.regions, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand, type=type)
            olap2.e <- findOverlaps(anchors(obj, type="second"), enh.regions, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand, type=type)

            combo1 <- base::merge(olap1.g, olap2.e, by.x=1, by.y=1)
            combo2 <- base::merge(olap2.g, olap1.e, by.x=1, by.y=1)
            if (use.region=="both") { 
                combo <- rbind(combo1, combo2)
            } else if (use.region=="same") {
                combo <- combo1
            } else {
                combo <- combo2
            }

            combo <- DataFrame(combo)
            colnames(combo) <- c("query", "subject1", "subject2")
            rownames(combo) <- NULL

            is.dup <- duplicated(paste0(combo$query, ".", combo$subject1, ".", combo$subject2))
            combo <- combo[!is.dup,]
            o <- order(combo$query, combo$subject1, combo$subject2)
            combo <- combo[o,]
    
            expect_identical(combo, linkOverlaps(obj, gene.regions, enh.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                                 ignore.strand=ignore.strand, use.region=use.region))
        })

        test_that(sprintf("linking overlaps works between the same regions (%i, %i)", cls, param), {
            # More manually merging, but just with the gene regions.
            olap1.g <- findOverlaps(anchors(obj, type="first"), gene.regions, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand, type=type)
            olap2.g <- findOverlaps(anchors(obj, type="second"), gene.regions, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand, type=type)

            combo.S <- base::merge(olap1.g, olap2.g, by.x=1, by.y=1)
            combo.S <- DataFrame(combo.S)
            colnames(combo.S) <- c("query", "subject1", "subject2")
            rownames(combo.S) <- NULL

            new.s1 <- pmax(combo.S$subject1, combo.S$subject2)
            new.s2 <- pmin(combo.S$subject1, combo.S$subject2)
            combo.S$subject1 <- new.s1
            combo.S$subject2 <- new.s2
    
            is.dup <- duplicated(paste0(combo.S$query, ".", combo.S$subject1, ".", combo.S$subject2)) | is.na(combo.S$query)
            combo.S <- combo.S[!is.dup,]
            o <- order(combo.S$query, combo.S$subject1, combo.S$subject2)
            combo.S <- combo.S[o,]
    
            expect_identical(combo.S, linkOverlaps(obj, gene.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                                   ignore.strand=ignore.strand, use.region=use.region))
        })

        test_that(sprintf("linking overlaps works with Hits (%i, %i)", cls, param), {
            ref1 <- linkOverlaps(obj, gene.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, use.region=use.region, ignore.strand=ignore.strand)
            olap1 <- findOverlaps(regions(obj), gene.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
            expect_identical(ref1, linkOverlaps(obj, olap1, use.region=use.region))
            
            ref2 <- linkOverlaps(obj, gene.regions, enh.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, use.region=use.region, ignore.strand=ignore.strand)
            olap2 <- findOverlaps(regions(obj), enh.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
            expect_identical(ref2, linkOverlaps(obj, olap1, olap2, use.region=use.region))
        })

        if (!ignore.strand) {
            # Just checking that ignore.strand works as expected.
            test_that("linkOverlaps works as expected without strandedness", {
                olap1 <- linkOverlaps(obj, gene.regions, ignore.strand=TRUE)
                olap2 <- linkOverlaps(obj, gene.regions, enh.regions, ignore.strand=TRUE)

                strand(regions(obj)) <- "*"
                strand(gene.regions) <- "*"
                strand(enh.regions) <- "*"

                expect_identical(olap1, linkOverlaps(obj, gene.regions, ignore.strand=FALSE))
                expect_identical(olap2, linkOverlaps(obj, gene.regions, enh.regions, ignore.strand=FALSE))
           })
        }
    }

    # Testing against empty slots.
    test_that("linking overlaps behaves with empty inputs", {
        expect_identical(DataFrame(query=integer(0), subject1=integer(0), subject2=integer(0)), linkOverlaps(obj, gene.regions[0], gene.regions))
        expect_identical(DataFrame(query=integer(0), subject1=integer(0), subject2=integer(0)), linkOverlaps(obj, gene.regions, gene.regions[0]))
        expect_identical(DataFrame(query=integer(0), subject1=integer(0), subject2=integer(0)), linkOverlaps(obj, gene.regions[0], gene.regions[0]))

        expect_error(linkOverlaps(obj, Hits()), "'nLnode' of 'subject1'")
        okay.hits <- Hits(nLnode=length(regions(obj)))
        expect_identical(DataFrame(query=integer(0), subject1=integer(0), subject2=integer(0)), linkOverlaps(obj, okay.hits))
        expect_error(linkOverlaps(obj, okay.hits, Hits()), "'nLnode' of 'subject2'")
        expect_identical(DataFrame(query=integer(0), subject1=integer(0), subject2=integer(0)), linkOverlaps(obj, okay.hits, okay.hits))
    })
}

# Testing the various overlap methods for InteractionSet objects.
# This implicitly tests the GInteractions methods, because one calls the other.
# library(InteractionSet); library(testthat); source("test-olap.R")

set.seed(300)
N <- 50
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
stranded <- sample(c("-", "+", "*"), N, replace=TRUE)
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-20, 20)), IRanges(all.starts, all.ends), strand=stranded)
all.regions <- unique(all.regions)
N <- length(all.regions)

Np <- 100
all.anchor1 <- sample(N, Np, replace=TRUE)
all.anchor2 <- sample(N, Np, replace=TRUE)
Nlibs <- 4
counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)
x <- InteractionSet(counts, GInteractions(all.anchor1, all.anchor2, all.regions))

#######################################################
#######################################################
# GInterations: 1D overlaps

### LOOP START
for (object in c("GRanges", "GRangesList")) {
###

if (object=="GRanges") {
    # GRanges.
    set.seed(301)
    Nq <- 10
    query.starts <- round(runif(Nq, 1, 100))
    query.ends <- query.starts + round(runif(Nq, 5, 20))
    query.strand <- sample(c("-", "+", "*"), Nq, replace=TRUE)
    query.regions <- GRanges(rep(c("chrA", "chrB"), Nq/2), IRanges(query.starts, query.ends), strand=query.strand)

} else if (object=="GRangesList") {
    # Paired regions in a GRangesList (don't be fooled by the GInteractions below!)
    set.seed(302)
    
    Nq2 <- 100
    query.starts <- round(runif(Nq2, 1, 100))
    query.ends <- query.starts + round(runif(Nq2, 5, 20))
    query.strand <- sample(c("-", "+", "*"), Nq2, replace=TRUE)
    query.regions1 <- GRanges(rep(c("chrA", "chrB"), Nq2/2), IRanges(query.starts, query.ends), strand=query.strand)

    query.starts <- round(runif(Nq2, 1, 100))
    query.ends <- query.starts + round(runif(Nq2, 5, 20))
    query.strand <- sample(c("-", "+", "*"), Nq2, replace=TRUE)
    query.regions2 <- GRanges(rep(c("chrA", "chrB"), Nq2/2), IRanges(query.starts, query.ends), strand=query.strand)

    query.regions <- pairs(GInteractions(query.regions1, query.regions2), as.grlist=TRUE)
}

for (param in seq_len(7)) {
    type <- "any"
    maxgap <- -1L
    minoverlap <- 0L
    use.region <- "both"
    ignore.strand <- TRUE

    if (param==2L) { 
        maxgap <- 10L
    } else if (param==3L) {
        minoverlap <- 10L
    } else if (param==4L) {
        type <- "within"
    } else if (param==5L) {
        use.region <- "first"
    } else if(param==6L) {
        use.region <- "second"
    } else if (param==7L) {
        ignore.strand <- FALSE
    }

    test_that(sprintf("findOverlaps works with 1D overlaps to %s (%i)", object, param), {
        # Manual calculation of overlaps from anchors().              
        expected1 <- findOverlaps(anchors(x, type="first"), query.regions, type=type, 
                                  maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
        expected2 <- findOverlaps(anchors(x, type="second"), query.regions, type=type, 
                                  maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)

        if (use.region=="both") {
            ref <- Hits(c(queryHits(expected1), queryHits(expected2)), c(subjectHits(expected1), subjectHits(expected2)),
                        nLnode=length(x), nRnode=length(query.regions), sort.by.query=TRUE)
        } else if (use.region=="first") {
            ref <- expected1
        } else if (use.region=="second") { 
            ref <- expected2
        }
       
        olap <- findOverlaps(x, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                             use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(olap, unique(sort(ref)))

        # Checking how it behaves with different 'select' specifications. 
        for (select in c("first", "last", "arbitrary")) { 
            selected <- findOverlaps(x, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                     select=select, use.region=use.region, ignore.strand=ignore.strand)
            if (select!="arbitrary") {
                expect_identical(selected, selectHits(olap, select))
            } else {
                is.okay <- !is.na(selected)
                expect_identical(is.okay, !is.na(selectHits(olap, select)))
                arbiter <- Hits(which(is.okay), selected[is.okay], length(x), length(query.regions), sort.by.query=TRUE)
                expect_true(all(!is.na(match(arbiter, olap))))
            }
        }

        # Checking that 'overlapsAny', 'countOverlaps' and 'subsetByOverlaps' works.
        count.lap <- countOverlaps(x, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                   use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(count.lap, selectHits(olap, "count"))
        
        out <- overlapsAny(x, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                           use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(out, !is.na(selected))

        expect_equal(subsetByOverlaps(x, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                      use.region=use.region, ignore.strand=ignore.strand), x[out,])

        # Checking the countOverlaps preserves names.
        x2 <- x
        rownames(x2) <- paste0("Y", seq_along(x2))
        count.lap2 <- countOverlaps(x2, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                    use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(count.lap2, setNames(count.lap, names(x2)))
    })

    test_that(sprintf("findOverlaps works with 1D overlaps to %s (reversed, %i)", object, param), {
        # Manual calculation of overlaps from anchors(), after swapping x and query.regions.
        rexpected1 <- findOverlaps(query.regions, anchors(x, type="first"), type=type, maxgap=maxgap, 
                                   minoverlap=minoverlap, ignore.strand=ignore.strand)
        rexpected2 <- findOverlaps(query.regions, anchors(x, type="second"), type=type, maxgap=maxgap, 
                                   minoverlap=minoverlap, ignore.strand=ignore.strand)

        if (use.region=="both") {
            rref <- Hits(c(queryHits(rexpected1), queryHits(rexpected2)), c(subjectHits(rexpected1), subjectHits(rexpected2)),
                         nLnode=length(query.regions), nRnode=length(x), sort.by.query=TRUE)
        } else if (use.region=="first") {
            rref <- rexpected1
        } else if (use.region=="second") {
            rref <- rexpected2
        }
        
        rolap <- findOverlaps(query.regions, x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                              use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(rolap, unique(sort(rref)))
        if (type!="within") { 
            olap <- findOverlaps(x, query.regions, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                 use.region=use.region, ignore.strand=ignore.strand)
            expect_identical(rolap, t(olap))
        } 
   
        # Seeing how it behaves with 'select' 
        for (select in c("first", "last", "arbitrary")) { 
            selected <- findOverlaps(query.regions, x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                     select=select, use.region=use.region, ignore.strand=ignore.strand)

            if (select!="arbitrary") {
                expect_identical(selected, selectHits(rolap, select))
            } else {
                is.okay <- !is.na(selected)
                expect_identical(is.okay, !is.na(selectHits(rolap, select)))
                arbiter <- Hits(which(is.okay), selected[is.okay], length(query.regions), length(x), sort.by.query=TRUE)
                expect_true(all(!is.na(match(arbiter, rolap))))
            }
        }
        
        # Seeing how the other *Overlaps functions behave. 
        count.lap <- countOverlaps(query.regions, x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                   use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(count.lap, selectHits(rolap, "count"))

        out <- overlapsAny(query.regions, x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                           use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(out, !is.na(selected))
        expect_identical(subsetByOverlaps(query.regions, x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                          use.region=use.region, ignore.strand=ignore.strand), query.regions[out])

        # Checking the countOverlaps preserves names.
        q2 <- query.regions
        names(q2) <- paste0("G", seq_along(q2))
        count.lap2 <- countOverlaps(q2, x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                    use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(count.lap2, setNames(count.lap, names(q2)))
    })

    if (!ignore.strand) {
        # Just checking that ignore.strand for findOverlaps,GenomicRanges does as expected.
        test_that("findOverlaps for 1D overlaps works as expected without strandedness", {
            ref.olap <- findOverlaps(x, query.regions, ignore.strand=TRUE)
            rev.olap <- findOverlaps(query.regions, x, ignore.strand=TRUE)
            strand(regions(x)) <- "*"
            strand(query.regions) <- "*"
            expect_identical(findOverlaps(x, query.regions, ignore.strand=FALSE), ref.olap)
            expect_identical(findOverlaps(query.regions, x, ignore.strand=FALSE), rev.olap)
        })
    }

    test_that(sprintf("findOverlaps behaves sensibly with silly %s", object), {
        expect_equal(findOverlaps(x[0,], query.regions), Hits(nLnode=0, nRnode=length(query.regions), sort.by.query=TRUE))
        expect_equal(findOverlaps(x, query.regions[0]), Hits(nLnode=length(x), nRnode=0L, sort.by.query=TRUE))
        expect_equal(findOverlaps(query.regions, x[0]), Hits(nRnode=0, nLnode=length(query.regions), sort.by.query=TRUE))
        expect_equal(findOverlaps(query.regions[0], x), Hits(nRnode=length(x), nLnode=0L, sort.by.query=TRUE))
        expect_equal(findOverlaps(x[0,], query.regions[0]), Hits(nLnode=0L, nRnode=0L, sort.by.query=TRUE))
        expect_equal(findOverlaps(query.regions[0,], x[0]), Hits(nLnode=0L, nRnode=0L, sort.by.query=TRUE))
        
        expect_equal(findOverlaps(x[0,], query.regions, select="first"), integer(0))
        expect_equal(findOverlaps(query.regions[0], x, select="first"), integer(0))
        expect_equal(findOverlaps(x, query.regions[0], select="first"), rep(as.integer(NA), nrow(x)))
        expect_equal(findOverlaps(query.regions, x[0], select="first"), rep(as.integer(NA), length(query.regions)))
        expect_equal(overlapsAny(x[0,], query.regions), logical(0))
        expect_equal(overlapsAny(query.regions[0]), logical(0))
        expect_equal(overlapsAny(x, query.regions[0]), logical(nrow(x)))
        expect_equal(overlapsAny(query.regions, x[0]), logical(length(query.regions)))
    })
}

### LOOP END
}
###

#######################################################
#######################################################
# GInterations/InteractionSet: 2D overlaps

### LOOP START
for (object in c("InteractionSet", "GInteractions")) {
### 

set.seed(303)
N2 <- 75
next.starts <- round(runif(N2, 1, 100))
next.ends <- next.starts + round(runif(N2, 5, 20))
next.stranded <- sample(c("-", "+", "*"), N2, replace=TRUE)
next.regions <- GRanges(rep(c("chrA", "chrB"), c(N2-20, 20)), IRanges(next.starts, next.ends), strand=next.stranded)
next.regions <- unique(next.regions)
N2 <- length(next.regions)

Np2 <- 120
next.anchor1 <- sample(N2, Np2, replace=TRUE)
next.anchor2 <- sample(N2, Np2, replace=TRUE)
counts <- matrix(rpois(Np2*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)

if (object=="InteractionSet") {
    x2 <- InteractionSet(counts, GInteractions(next.anchor1, next.anchor2, next.regions))
} else {
    x2 <- GInteractions(next.anchor1, next.anchor2, next.regions)
}

for (param in seq_len(6)) {
    type <- "any"
    maxgap <- -1L
    minoverlap <- 0L
    use.region <- "both"
    ignore.strand <- TRUE

    if (param==2L) { 
        maxgap <- 10L
    } else if (param==3L) {
        minoverlap <- 10L
    } else if (param==4L) {
        type <- "within"
    } else if (param==5L) {
        use.region <- "same"
    } else if(param==6L) {
        use.region <- "reverse"
    } else if (param==7L) {
        ignore.strand <- FALSE
    }

    test_that(sprintf("findOverlaps works with 2D overlaps to %s (%i)", object, param), {
        # Manually calculating overlaps based on anchors().
        expected1.A <- findOverlaps(anchors(x, type="first"), anchors(x2, type="first"), type=type, 
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
        expected1.B <- findOverlaps(anchors(x, type="first"), anchors(x2, type="second"), type=type, 
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
        expected2.B <- findOverlaps(anchors(x, type="second"),anchors(x2, type="first"), type=type, # Yes, the A/B switch in order is deliberate.
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand) 
        expected2.A <- findOverlaps(anchors(x, type="second"), anchors(x2, type="second"), type=type, 
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
    
        expected1.A <- paste0(queryHits(expected1.A), ".", subjectHits(expected1.A), ".A")
        expected1.B <- paste0(queryHits(expected1.B), ".", subjectHits(expected1.B), ".B")
        expected2.A <- paste0(queryHits(expected2.A), ".", subjectHits(expected2.A), ".A")
        expected2.B <- paste0(queryHits(expected2.B), ".", subjectHits(expected2.B), ".B")
        if (use.region=="both") {
            expected1 <- c(expected1.A, expected1.B)
            expected2 <- c(expected2.A, expected2.B)
        } else if (use.region=="same") {
            expected1 <- expected1.A
            expected2 <- expected2.A
        } else {
            expected1 <- expected1.B
            expected2 <- expected2.B
        }
    
        expected <- intersect(expected1, expected2)
        harvest <- do.call(rbind, strsplit(expected, "\\."))
        ref <- Hits(as.integer(harvest[,1]), as.integer(harvest[,2]), nLnode=length(x), nRnode=length(x2), sort.by.query=TRUE)
        ref <- sort(unique(ref))
        
        olap <- findOverlaps(x, x2, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                             use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(olap, ref)
    
        # Checking 'select' arguments.
        for (select in c("first", "last", "arbitrary")) { 
            selected <- findOverlaps(x, x2, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                     use.region=use.region, select=select, ignore.strand=ignore.strand)

            if (select!="arbitrary") {
                expect_identical(selected, selectHits(olap, select))
            } else {
                is.okay <- !is.na(selected)
                expect_identical(is.okay, !is.na(selectHits(olap, select)))
                arbiter <- Hits(which(is.okay), selected[is.okay], length(x), length(x2), sort.by.query=TRUE)
                expect_true(all(!is.na(match(arbiter, olap))))
            }
        }
    
        # Checking that 'overlapsAny', 'countOverlaps' and 'subsetByOverlaps' works.
        count.lap <- countOverlaps(x, x2, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                   use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(count.lap, selectHits(olap, "count"))

        out <- overlapsAny(x, x2, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                           use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(out, !is.na(selected))
        expect_equal(subsetByOverlaps(x, x2, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                      use.region=use.region, ignore.strand=ignore.strand), x[out,])
    })

    # Note: No need to flip, it's the same method.
    # However, we do test self-overlaps.

    test_that(sprintf("findOverlaps works with 2D self-overlaps (%i)", param), {
        # Manually calculating overlaps based on anchors().
        expected1.A <- findOverlaps(anchors(x, type="first"), anchors(x, type="first"), type=type, 
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
        expected1.B <- findOverlaps(anchors(x, type="first"), anchors(x, type="second"), type=type, 
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
        expected2.B <- findOverlaps(anchors(x, type="second"), anchors(x, type="first"), type=type, # Again; yes, the A/B switch is deliberate.
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand) 
        expected2.A <- findOverlaps(anchors(x, type="second"), anchors(x, type="second"), type=type, 
                                    maxgap=maxgap, minoverlap=minoverlap, ignore.strand=ignore.strand)
    
        expected1.A <- paste0(queryHits(expected1.A), ".", subjectHits(expected1.A), ".A")
        expected1.B <- paste0(queryHits(expected1.B), ".", subjectHits(expected1.B), ".B")
        expected2.A <- paste0(queryHits(expected2.A), ".", subjectHits(expected2.A), ".A")
        expected2.B <- paste0(queryHits(expected2.B), ".", subjectHits(expected2.B), ".B")
        if (use.region=="both") {
            expected1 <- c(expected1.A, expected1.B)
            expected2 <- c(expected2.A, expected2.B)
        } else if (use.region=="same") {
            expected1 <- expected1.A
            expected2 <- expected2.A
        } else {
            expected1 <- expected1.B
            expected2 <- expected2.B
        }
    
        expected <- intersect(expected1, expected2)
        harvest <- do.call(rbind, strsplit(expected, "\\."))
        ref <- SelfHits(as.integer(harvest[,1]), as.integer(harvest[,2]), nnode=length(x), sort.by.query=TRUE)
        ref <- sort(unique(ref))
        
        self.olap <- findOverlaps(x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                  use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(self.olap, ref)

        # Checking that the function responds to the various drop.* arguments 
        self.olap.noself <- findOverlaps(x, type=type, maxgap=maxgap, minoverlap=minoverlap, use.region=use.region,
                                         drop.self=TRUE, ignore.strand=ignore.strand)
        expect_identical(self.olap.noself, self.olap[!isSelfHit(self.olap)])

        self.olap.nored <- findOverlaps(x, type=type, maxgap=maxgap, minoverlap=minoverlap, use.region=use.region,
                                        drop.redundant=TRUE, ignore.strand=ignore.strand)
        expect_identical(self.olap.nored, self.olap[!isRedundantHit(self.olap)])
    
        # Checking the behaviour of 'select'.
        for (select in c("first", "last", "arbitrary")) { 
            selected <- findOverlaps(x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                     use.region=use.region, select=select, ignore.strand=ignore.strand)

            if (select!="arbitrary") {
                expect_identical(selected, selectHits(self.olap, select))
            } else {
                is.okay <- !is.na(selected)
                expect_identical(is.okay, !is.na(selectHits(self.olap, select)))
                arbiter <- Hits(which(is.okay), selected[is.okay], length(x), length(x), sort.by.query=TRUE)
                expect_true(all(!is.na(match(arbiter, self.olap))))
            }
        }
    
        # Checking that 'overlapsAny', 'countOverlaps' and 'subsetByOverlaps' works.
        count.lap <- countOverlaps(x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                                   use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(count.lap, selectHits(self.olap, "count"))

        out <- overlapsAny(x, type=type, maxgap=maxgap, minoverlap=minoverlap, 
                           use.region=use.region, ignore.strand=ignore.strand)
        expect_identical(out, !is.na(selected))
    })

    if (!ignore.strand) {
        # Just checking that ignore.strand for findOverlaps,GenomicRanges does as expected.
        test_that("findOverlaps for 2D overlaps works as expected without strandedness", {
            ref.olap <- findOverlaps(x, x2, ignore.strand=TRUE)
            self.olap <- findOverlaps(x, ignore.strand=TRUE)
            strand(regions(x)) <- "*"
            strand(regions(x2)) <- "*"
            expect_identical(findOverlaps(x, x2, ignore.strand=FALSE), ref.olap)
            expect_identical(findOverlaps(x, ignore.strand=FALSE), self.olap)
        })
    }
}

### LOOP END
}
###

test_that("findOverlaps in 2D-mode works with silly inputs", {
    expect_equal(findOverlaps(x[0], x2), Hits(nLnode=0, nRnode=Np2, sort.by.query=TRUE))
    expect_equal(findOverlaps(x, x2[0]), Hits(nRnode=0, nLnode=nrow(x), sort.by.query=TRUE))
    expect_equal(findOverlaps(x[0], x2[0]), Hits(nLnode=0L, nRnode=0L, sort.by.query=TRUE))
    
    expect_equal(findOverlaps(x[0], x2, select="first"), integer(0))
    expect_equal(findOverlaps(x, x2[0], select="first"), rep(as.integer(NA), nrow(x)))
    expect_equal(overlapsAny(x[0], x2), logical(0))
    expect_equal(overlapsAny(x, x2[0]), logical(nrow(x)))
})

#######################################################
#######################################################
# ContactMatrix overlaps

set.seed(304)
Nr <- 100
Nc <- 200
all.anchor1 <- sample(N, Nr, replace=TRUE)
all.anchor2 <- sample(N, Nc, replace=TRUE)
counts <- matrix(rpois(Nr*Nc, lambda=10), Nr, Nc)
x <- ContactMatrix(counts, all.anchor1, all.anchor2, all.regions)

Nq <- 6
query.starts <- round(runif(Nq, 1, 100))
query.ends <- query.starts + round(runif(Nq, 5, 20))
query.regions <- GRanges(rep(c("chrA", "chrB"), Nq/2), IRanges(query.starts, query.ends))

test_that("overlapsAny for CM objects works with 1D overlaps", {
    olap <- overlapsAny(x, query.regions)
    expect_identical(olap, list(row=overlapsAny(anchors(x, type="row"), query.regions),
                                column=overlapsAny(anchors(x, type="column"), query.regions)))
    
    olap <- overlapsAny(x, query.regions, type="within")
    expect_identical(olap, list(row=overlapsAny(anchors(x, type="row"), query.regions, type="within"),
                                column=overlapsAny(anchors(x, type="column"), query.regions, type="within")))

    olap <- overlapsAny(x, query.regions, ignore.strand=FALSE)
    expect_identical(olap, list(row=overlapsAny(anchors(x, type="row"), query.regions, ignore.strand=FALSE),
                                column=overlapsAny(anchors(x, type="column"), query.regions, ignore.strand=FALSE)))
    
    expect_equal(overlapsAny(x[0,], query.regions), list(row=logical(0), column=overlapsAny(anchors(x, type="column"), query.regions)))
    expect_equal(overlapsAny(x[,0], query.regions), list(row=overlapsAny(anchors(x, type="row"), query.regions), column=logical(0)))
    expect_equal(overlapsAny(x[0,0], query.regions), list(row=logical(0), column=logical(0)))
})

test_that("overlapsAny for CM objects works with 2D overlaps", {
    Nq3 <- 10
    query.starts <- round(runif(Nq3, 1, 100))
    query.ends <- query.starts + round(runif(Nq3, 5, 20))
    query.regions1 <- GRanges(rep(c("chrA", "chrB"), Nq3/2), IRanges(query.starts, query.ends))
    query.starts <- round(runif(Nq3, 1, 100))
    query.ends <- query.starts + round(runif(Nq3, 5, 20))
    query.regions2 <- GRanges(rep(c("chrA", "chrB"), Nq3/2), IRanges(query.starts, query.ends))
    pairing <- GInteractions(query.regions1, query.regions2)
    
    olap <- overlapsAny(x, pairing)
    temp.iset <- deflate(x, collapse=TRUE)
    ref <- overlapsAny(temp.iset, pairing) # no NAs, everyone's represented here.
    ref <- inflate(temp.iset, anchors(x, type="row", id=TRUE), anchors(x, type="column", id=TRUE), fill=ref)
    expect_identical(olap, unname(as.matrix(as.matrix(ref))))
    
    olap <- overlapsAny(x, pairing, type="within")
    ref <- overlapsAny(temp.iset, pairing, type="within")
    ref <- inflate(temp.iset, anchors(x, type="row", id=TRUE), anchors(x, type="column", id=TRUE), fill=ref)
    expect_identical(olap, unname(as.matrix(as.matrix(ref))))

    olap <- overlapsAny(x, pairing, ignore.strand=FALSE)
    ref <- overlapsAny(temp.iset, pairing, ignore.strand=FALSE)
    ref <- inflate(temp.iset, anchors(x, type="row", id=TRUE), anchors(x, type="column", id=TRUE), fill=ref)
    expect_identical(olap, unname(as.matrix(as.matrix(ref))))
    
    olap <- overlapsAny(x, pairing)
    new.gi <- GInteractions(query.regions1, query.regions2)
    expect_identical(olap, overlapsAny(x, new.gi))
    
    new.iset <- InteractionSet(matrix(runif(10), dimnames=list(NULL, 1)), new.gi)
    expect_identical(olap, overlapsAny(x, new.iset))
    
    # Checking with order enforcement.
    olap <- overlapsAny(x, pairing, use.region="same")
    expect_identical(olap, overlapsAny(x, new.gi, use.region="same"))
    
    olap <- overlapsAny(x, pairing, use.region="reverse")
    expect_identical(olap, overlapsAny(x, new.gi, use.region="reverse"))
})

# This tests the conversion functions of an InteractionSet object.
# library(InteractionSet); library(testthat); source("test-convert.R") 

set.seed(6000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
Nlibs <- 4
counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)
offs <- matrix(rnorm(Np*Nlibs), ncol=Nlibs)
x <- InteractionSet(list(counts, offs), GInteractions(all.anchor1, all.anchor2, all.regions))

##########################################
# Slow and steady implementation of the coercion function.

ref.fun <- function(x, rows, cols, fill, ass=1, sam=1, swap=TRUE) { 
    all.anchors <- anchors(x, id=TRUE)
    if (missing(fill)) { fill <- assay(x, ass)[,sam] }
    ref <- matrix(as(NA, typeof(fill)), length(rows), length(cols))
    for (i in seq_len(nrow(x))){ 
        a1 <- all.anchors$first[i]
        a2 <- all.anchors$second[i]
        ref[rows==a1,cols==a2] <- fill[i]
        if (swap) { ref[rows==a2,cols==a1] <- fill[i] }
    }
    return(ref)
}

##########################################

test_that("Inflation with integers works", {
    chosen.rows <- 1:10
    chosen.cols <- 11:15
    out <- inflate(x, chosen.rows, chosen.cols)
    expect_identical(regions(out), regions(x))
    expect_identical(anchors(out, type="row"), regions(x)[chosen.rows])
    expect_identical(anchors(out, type="column"), regions(x)[chosen.cols])
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    out <- inflate(x, chosen.rows, chosen.cols, assay=2)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols, ass=2))
    out <- inflate(x, chosen.rows, chosen.cols, sample=4)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols, sam=4))

    blah <- runif(Np)
    out <- inflate(x, chosen.rows, chosen.cols, fill=blah)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols, fill=blah))
    
    # Disabling swapping.
    out.nswp <- inflate(x, chosen.rows, chosen.cols, fill=blah, swap=FALSE)
    ref.nswp <- ref.fun(x, chosen.rows, chosen.cols, fill=blah, swap=FALSE)
    expect_equal(as.matrix(out.nswp), ref.nswp)
    
    # Trying out a sparse matrix.
    out2 <- inflate(x, chosen.rows, chosen.cols, fill=blah, sparse=TRUE) 
    expect_is(as.matrix(out2), "dgCMatrix")
    expect_equal(dim(out), dim(out2))
    ref <- as.matrix(out)
    not.missing <- Matrix::which(!is.na(ref))
    expect_equal(as.matrix(out2)[not.missing], ref[not.missing])
    expect_true(all(as.matrix(as.matrix(out2))[!not.missing]==0))

    # Testing default fill.
    outx <- inflate(interactions(x), chosen.rows, chosen.cols) 
    refx <- inflate(interactions(x), chosen.rows, chosen.cols, fill=rep(1, length(x)))
    expect_identical(anchors(outx), anchors(refx))
    expect_identical(as.matrix(outx), as.matrix(refx)!=0)

    outx <- inflate(interactions(x), chosen.rows, chosen.cols, sparse=TRUE)
    refx <- inflate(interactions(x), chosen.rows, chosen.cols, fill=rep(1, length(x)), sparse=TRUE)
    expect_identical(anchors(outx), anchors(refx))
    expect_identical(as.matrix(outx), as.matrix(refx)!=0)
    
    # Dealing with duplication and resorting:
    chosen.rows <- c(1:10, 1:10)
    chosen.cols <- c(11:15, 11:15)
    out <- inflate(x, chosen.rows, chosen.cols)
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    chosen.rows <- as.integer(c(1,3,2,6,7,9,2,2,1))
    chosen.cols <- as.integer(c(11,16,2,2,5))
    out <- inflate(x, chosen.rows, chosen.cols)
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    out2 <- inflate(x, chosen.rows, chosen.cols, sparse=TRUE) # Trying out a sparse matrix, again.
    expect_is(as.matrix(out2), "dgCMatrix")
    expect_equal(dim(out), dim(out2))
    ref <- as.matrix(out)
    not.missing <- Matrix::which(!is.na(ref))
    expect_equal(as.matrix(out2)[not.missing], ref[not.missing])
    expect_true(all(as.matrix(as.matrix(out2))[!not.missing]==0))
    
    # What happens with silly inputs?
    expect_true(nrow(inflate(x, integer(0), 1:10))==0L)
    expect_true(ncol(inflate(x, 1:5, integer(0)))==0L)
    expect_error(inflate(x, 0, 1:10), "positive integer")
    expect_error(inflate(x, as.numeric(NA), 1:10), "positive integer")
    expect_error(inflate(x, 10000, 1:10), "positive integer")
})

##########################################

test_that("Inflation with logical vectors works", {
    chosen.rows <- chosen.cols <- logical(length(regions(x)))
    old.rows <- 1:10
    old.cols <- 11:15
    chosen.rows[old.rows] <- TRUE
    chosen.cols[old.cols] <- TRUE
    out.old <- inflate(x, old.rows, old.cols)
    out <- inflate(x, chosen.rows, chosen.cols)
    expect_identical(out, out.old)
    
    out.old <- inflate(x, old.rows, old.cols, sparse=TRUE)
    out <- inflate(x, chosen.rows, chosen.cols, sparse=TRUE)
    expect_identical(out, out.old)
})

test_that("Inflation with character vectors works", {
    out <- inflate(x, "chrA", "chrA")
    chosen.rows <- which(seqnames(regions(out))=="chrA")
    chosen.cols <- which(seqnames(regions(out))=="chrA")
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    blah <- runif(Np)
    out <- inflate(x, chosen.rows, chosen.cols, fill=blah, swap=FALSE) # Symmetric space, so there's guaranteed to swapping.
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols, fill=blah, swap=FALSE))
    
    out <- inflate(x, "chrA", "chrB")
    chosen.rows <- which(seqnames(regions(out))=="chrA")
    chosen.cols <- which(seqnames(regions(out))=="chrB")
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    out <- inflate(x, "chrA", c("chrA", "chrB")) # Multiple chromosomes.
    chosen.rows <- which(seqnames(regions(out))=="chrA")
    chosen.cols <- which(seqnames(regions(out)) %in%  c("chrA", "chrB"))
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    expect_true(nrow(inflate(x, "whee", 1:10))==0L)
    expect_true(ncol(inflate(x, 1:5, "whee"))==0L)
})

##########################################

test_that("Inflation with GRanges works", {
    of.interest <- GRanges(c("chrA", "chrB"), IRanges(c(1, 10), c(20, 50)))
    
    out <- inflate(x, of.interest, of.interest)
    chosen.rows <- chosen.cols <- which(overlapsAny(regions(x), of.interest))
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    out <- inflate(x, of.interest, of.interest, type="within")
    chosen.rows <- chosen.cols <- which(overlapsAny(regions(x), of.interest, type="within"))
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    out <- inflate(x, of.interest[1], of.interest[2], type="within")
    chosen.rows <- which(overlapsAny(regions(x), of.interest[1], type="within"))
    chosen.cols <- which(overlapsAny(regions(x), of.interest[2], type="within"))
    expect_identical(anchors(out, type="row", id=TRUE), chosen.rows)
    expect_identical(anchors(out, type="column", id=TRUE), chosen.cols)
    expect_equal(as.matrix(out), ref.fun(x, chosen.rows, chosen.cols))
    
    expect_true(nrow(inflate(x, GRanges(), 1:10))==0L)
    expect_true(ncol(inflate(x, 1:5, GRanges()))==0L)
    out.of.range <- GRanges("chrC", IRanges(1, 1))
    expect_true(nrow(suppressWarnings(inflate(x, out.of.range, 1:10)))==0L)
    expect_true(ncol(suppressWarnings(inflate(x, 1:5, out.of.range)))==0L)
    
    all.chr <- range(all.regions)
    expect_identical(inflate(x, all.chr[1], all.chr[2]), inflate(x, "chrA", "chrB"))
})

test_that("Inflation with NULL works", {
    all.regs <- seq_along(regions(x))
    out <- inflate(x, all.regs, all.regs)
    expect_identical(out, inflate(x, NULL, NULL))
    expect_identical(ref.fun(x, all.regs, all.regs), as.matrix(inflate(x, NULL, NULL)))
    
    all.regs <- !logical(length(regions(x)))
    out <- inflate(x, all.regs, all.regs)
    expect_identical(out, inflate(x, NULL, NULL))
    
    all.regs <- unique(as.character(seqlevels(regions(x))))
    out <- inflate(x, all.regs, all.regs)
    expect_identical(out, inflate(x, NULL, NULL))
    
    all.regs <- range(regions(x))
    out <- inflate(x, all.regs, all.regs)
    expect_identical(out, inflate(x, NULL, NULL))
})

##########################################

test_that("Standard deflation methods are working", {
    y <- inflate(x, "chrA", "chrA")
    x2 <- deflate(y)
    x2 <- sort(x2)
    all.chr <- range(all.regions)
    keep.x <- subsetByOverlaps(x, GInteractions(all.chr[1], all.chr[1])) 
    keep.x <- sort(swapAnchors(keep.x))
    expect_identical(anchors(x2), anchors(keep.x))
    expect_equal(assay(x2)[,1], assay(keep.x)[,1]) # Not identical, due to coercion to double.
    
    # What happens when you turn off uniqueness (in this case, we have symmetry):
    x2 <- deflate(y, collapse=FALSE)
    x2 <- sort(x2)
    not.diag <- anchors(keep.x, type="first", id=TRUE)!=anchors(keep.x, type="second", id=TRUE)
    keep.x <- rbind(keep.x[not.diag], swapAnchors(keep.x[not.diag], mode='all'), keep.x[!not.diag])
    keep.x <- sort(keep.x)
    expect_identical(anchors(x2), anchors(keep.x))
    expect_equal(assay(x2)[,1], assay(keep.x)[,1])
    
    # Behaviour for different index sets:
    y <- inflate(x, "chrA", "chrB")
    x2 <- deflate(y)
    x2 <- sort(x2)
    keep.x <- subsetByOverlaps(x, GInteractions(all.chr[1], all.chr[2])) 
    keep.x <- sort(swapAnchors(keep.x))
    expect_identical(anchors(x2), anchors(keep.x))
    expect_equal(assay(x2)[,1], assay(keep.x)[,1])
})

test_that("Advanced deflation optionsare working", {
    # With sparsity:
    y <- inflate(x, "chrA", "chrA", sparse=TRUE)
    xref <- inflate(x, "chrA", "chrA")
    ref <- as.matrix(xref)
    ref[is.na(ref)] <- 0
    expect_identical(as.matrix(y), as(ref, "dgCMatrix"))
    expect_equal(deflate(y), deflate(xref, use.zero=FALSE, use.na=FALSE)) # Getting rid of genuine zeros in there.
    
    # Including NAs to be extracted:
    na.deflated <- deflate(xref, use.na=TRUE)
    expect_identical(length(na.deflated), sum(seq_len(nrow(ref))))

    # Including zeroes to be extracted:
    xref2 <- xref
    as.matrix(xref2) <- ref
    expect_equal(deflate(y, use.zero=TRUE), deflate(xref2, use.zero=TRUE))
    
    # Using explicit extraction.
    ex <- !is.na(as.matrix(xref))
    alt <- deflate(xref, use.na=FALSE, use.zero=TRUE)
    expect_equal(deflate(xref, extract=ex), alt)
    expect_false(isTRUE(all.equal(alt, deflate(xref, use.na=TRUE)))) # definitely different
    expect_equal(deflate(xref, extract=ex, use.na=TRUE), alt) # ... but 'ex' overrides it.

    # Trying out some silliness.
    expect_true(nrow(deflate(ContactMatrix(matrix(0, 4, 0), 1:4, integer(0), all.regions)))==0L)
    expect_true(nrow(deflate(ContactMatrix(matrix(0, 0, 4), integer(0), 1:4, all.regions)))==0L)
})

# Tests the construction and manipulation of InteractionSet objects.
# library(InteractionSet); library(testthat); source("test-IS.R")

set.seed(1000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
Nlibs <- 4
counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)
x <- InteractionSet(counts, GInteractions(all.anchor1, all.anchor2, all.regions))

############################## 

test_that("show method works for IS objects", {
    expect_is(x, "InteractionSet")
    expect_output(show(x), "class: InteractionSet 
dim: 20 4 
metadata(0):
assays(1): ''
rownames: NULL
rowData names(0):
colnames(4): 1 2 3 4
colData names(0):
type: GInteractions
regions: 30", 
    fixed=TRUE)
})

test_that("new slot access works for IS objects", {
    expect_is(interactions(x), "GInteractions")
    expect_equivalent(assay(x), counts)
    ref <- GInteractions(all.anchor1, all.anchor2, all.regions)
    expect_identical(interactions(x), ref)
    
    scores <- Nlibs:1 + 50L
    x2 <- InteractionSet(counts, ref, colData=DataFrame(score=scores))
    expect_identical(x2$score, scores)
    expect_identical(colData(x2)$score, scores)
    expect_identical(mcols(x2)$score, NULL) # Dollar doesn't assign here.
                         
    x3 <- InteractionSet(counts, ref, metadata=list(whee=5L))
    expect_identical(metadata(x3)$whee, 5L)
})

test_that("IS constructor behaves correctly with crappy inputs", {
    expect_identical(dim(InteractionSet(matrix(0, 4, 0), GInteractions(1:4, 1:4, all.regions))), c(4L, 0L)) # No columns
    expect_identical(dim(InteractionSet(matrix(0, 0, 4, dimnames=list(NULL, seq_len(4))), GInteractions(integer(0), numeric(0), GRanges()))), c(0L, 4L))
    expect_error(InteractionSet(matrix(0, 3, 0), GInteractions(1:4, 1:4, all.regions)), "'interactions' length is not equal to the number of rows")
})

############################## 

test_that("delegated getters/setters work for IS objects", {
    set.seed(1001)
    shuffled <- sample(100, N, replace=TRUE)
    ref <- interactions(x)
    expect_identical(regions(x), regions(ref))
    expect_identical(anchors(x), anchors(ref))
    expect_identical(anchors(x, id=TRUE), anchors(ref, id=TRUE))
    expect_identical(anchors(x, type="first"), anchors(ref, type="first"))
    expect_identical(anchors(x, type="second"), anchors(ref, type="second"))

    expect_identical(anchorIds(x), anchorIds(ref))
    expect_identical(anchorIds(x, type="first"), anchorIds(ref, type="first"))
    expect_identical(anchorIds(x, type="second"), anchorIds(ref, type="second"))

    expect_identical(first(x), first(ref))
    expect_identical(second(x), second(ref))
    
    regions(x)$score <- shuffled
    regions(ref)$score <- shuffled
    expect_identical(regions(x)$score, regions(ref)$score)
    regions(ref)$score <- regions(x)$score <- NULL # Restoring.
    
    orig.x <- x
    fresh.anchor1 <- sample(N, Np)
    fresh.anchor2 <- sample(N, Np)
    anchorIds(x) <- list(fresh.anchor1, fresh.anchor2)
    anchorIds(ref) <- list(fresh.anchor1, fresh.anchor2)
    expect_identical(anchors(x), anchors(ref))
    expect_identical(anchors(x, id=TRUE), anchors(ref, id=TRUE))
    expect_identical(anchors(x, type="first"), anchors(ref, type="first"))
    expect_identical(anchors(x, type="second"), anchors(ref, type="second"))
    expect_identical(first(x), first(ref))
    expect_identical(second(x), second(ref))
    
    original.i <- anchors(orig.x, id=TRUE) # Reverting back to original indices, to check individual assignments work.
    anchorIds(x, type="first") <- original.i$first
    anchorIds(ref, type="first") <- original.i$first
    expect_identical(anchors(x, id=TRUE, type="first"), anchors(ref, id=TRUE, type="first"))
    anchorIds(x, type="second") <- original.i$second
    anchorIds(ref, type="second") <- original.i$second
    expect_identical(anchors(x, id=TRUE, type="second"), anchors(ref, id=TRUE, type="second"))
    anchorIds(x, type="both") <- original.i
    anchorIds(ref, type="both") <- original.i
    expect_identical(anchors(x), anchors(ref))
    
    lib.sizes <- 1:4*1000L
    x$totals <- lib.sizes
    expect_identical(x$totals, lib.sizes)
    expect_identical(colData(x)$totals, lib.sizes)
    
    x.dump <- x
    ref.dump <- interactions(x)
    mod.ranges <- resize(regions(x), fix="center", width=50)
    new.ranges <- c(regions(x), mod.ranges) 
    expect_error(regions(x.dump) <- new.ranges, "assigned value must be of the same length")
    replaceRegions(x.dump) <- new.ranges
    replaceRegions(ref.dump) <- new.ranges
    expect_identical(anchors(x.dump), anchors(ref.dump))
    expect_identical(regions(x.dump), regions(ref.dump))
    expect_error(replaceRegions(x.dump) <- mod.ranges, "some existing ranges do not exist in replacement GRanges")
    
    x.dump <- x
    ref.dump <- interactions(x)
    appendRegions(x.dump) <- mod.ranges
    appendRegions(ref.dump) <- mod.ranges
    expect_identical(regions(x.dump), regions(ref.dump))
    
    expect_identical(interactions(reduceRegions(x)), reduceRegions(interactions(x)))
    
    x.dump <- x
    interactions(x.dump) <- rev(interactions(x))
    expect_identical(interactions(x.dump), rev(interactions(x)))
    expect_identical(interactions(rev(x.dump)), interactions(x))
    
    new.scores <- 1:Np*10 + 50
    mcols(x.dump)$fire <- new.scores
    expect_identical(mcols(x.dump)$fire, new.scores)
    expect_identical(interactions(x.dump)$fire, new.scores)
    
    new.si <- Seqinfo(seqnames=c("chrA", "chrB"), seqlengths=c(1000, 2000))
    new.x <- x
    seqinfo(new.x) <- new.si
    expect_identical(seqinfo(new.x), seqinfo(interactions(new.x)))
})

############################## 

test_that("subsetting works for IS objects", {
    set.seed(1002)
    x$totals <- runif(ncol(x))          
    rchosen <- 1:10
    xsub <- x[rchosen,]
    expect_output(show(xsub), "class: InteractionSet 
dim: 10 4 
metadata(0):
assays(1): ''
rownames: NULL
rowData names(0):
colnames(4): 1 2 3 4
colData names(1): totals
type: GInteractions
regions: 30", 
fixed=TRUE)
    expect_equal(xsub, x[rchosen])
    expect_identical(assay(xsub), assay(x)[rchosen,])
    expect_identical(interactions(xsub), interactions(x)[rchosen])

    cchosen <- c(2,4)
    xsub <- x[,cchosen]
    expect_output(show(xsub), "class: InteractionSet 
dim: 20 2 
metadata(0):
assays(1): ''
rownames: NULL
rowData names(0):
colnames(2): 2 4
colData names(1): totals
type: GInteractions
regions: 30", 
fixed=TRUE)
    expect_identical(assay(xsub), assay(x)[,cchosen])
    expect_identical(xsub$totals, x$totals[cchosen])
    expect_identical(interactions(xsub), interactions(x))

    xsub <- x[rchosen,cchosen]
expect_output(show(xsub), "class: InteractionSet 
dim: 10 2 
metadata(0):
assays(1): ''
rownames: NULL
rowData names(0):
colnames(2): 2 4
colData names(1): totals
type: GInteractions
regions: 30", 
fixed=TRUE)
    expect_equal(xsub, subset(x, rchosen, cchosen))
    lrchosen <- logical(nrow(x)); lrchosen[rchosen] <- TRUE
    lcchosen <- logical(ncol(x)); lcchosen[cchosen] <- TRUE
    expect_equal(xsub, x[lrchosen,lcchosen])

    expect_identical(assay(xsub), assay(x)[rchosen,cchosen])
    expect_identical(xsub$totals, x$totals[cchosen])
    expect_identical(interactions(xsub), interactions(x)[rchosen])

    expect_identical(nrow(x[0,]), 0L)
    expect_identical(ncol(x[0,]), as.integer(Nlibs))
    expect_identical(ncol(x[,0]), 0L)
    expect_identical(nrow(x[,0]), as.integer(Np))
})

test_that("subset assignment works for IS objects", {
    temp.x <- x
    temp.x[1:5+10,] <- x[1:5,]
    new.index <- seq_len(nrow(x))
    new.index[1:5+10] <- 1:5
    expect_equal(assay(temp.x), assay(x)[new.index,])
    expect_identical(interactions(temp.x), interactions(x)[new.index,])
    
    temp.x <- x  
    c.from <- 2:3
    c.to <- 1:2
    tmp <- x[,c.from]
    colnames(tmp) <- colnames(x)[c.to] # Avoid errors from duplicated column names.
    temp.x[,c.to] <- tmp
    new.index <- seq_len(ncol(x))
    new.index[c.to] <- c.from
    ref.x <- x[,new.index]
    colnames(ref.x) <- colnames(x)
    expect_equal(assay(temp.x), assay(ref.x))
    expect_identical(interactions(temp.x), interactions(x))
    
    temp.x <- x
    temp.x[0,] <- x[0,]
    expect_equal(temp.x, x)
    temp.x[,0] <- x[,0]
    expect_equal(temp.x, x)
})

############################## 

test_that("combining works for IS objects", {
    xsub <- x[1:5,]
    xsub2 <- x[6:20,]
    expect_equal(rbind(xsub, xsub2), x)
    xsub <- x[5:10,]
    xsub2 <- x[1:3,]
    expect_equal(rbind(xsub, xsub2), x[c(5:10, 1:3),])
    expect_error(rbind(xsub, xsub2[,1:2]), "objects must have the same colnames")
    
    xsub <- x[,1]
    xsub2 <- x[,2:4]
    expect_equal(cbind(xsub, xsub2), x)
    
    first.cols <- 3 
    other.cols <- 1:4
    xsub <- x[,first.cols]
    xsub2 <- x[,other.cols]
    xsub.comb <- cbind(xsub, xsub2)
    xsub.ref <- x[,c(first.cols, other.cols)]
    colnames(xsub.comb) <- colnames(xsub.ref) # Keeping the column names happy again.
    expect_equal(xsub.comb, xsub.ref)
    
    expect_error(cbind(xsub, xsub2[1:10,]), "interactions must be identical in 'cbind'")
    
    expect_identical(nrow(rbind(x[0,], x[0,])), 0L) # Behaviour with empties.
    expect_identical(ncol(rbind(x[0,], x[0,])), ncol(x))
    expect_equal(rbind(x, x[0,]), x)
    expect_identical(nrow(cbind(x[,0], x[,0])), nrow(x))
    expect_identical(ncol(cbind(x[,0], x[,0])), 0L)
    expect_equal(cbind(x, x[,0]), x)
    
    set.seed(1003)
    next.starts <- round(runif(N, 1, 100))
    next.ends <- next.starts + round(runif(N, 5, 20))
    next.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(next.starts, next.ends))
    
    next.anchor1 <- sample(N, Np)
    next.anchor2 <- sample(N, Np)
    counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
    colnames(counts) <- colnames(x)
    next.x <- InteractionSet(counts, GInteractions(next.anchor1, next.anchor2, next.regions))
    
    c.x <- rbind(x, next.x)
    expect_equivalent(assay(c.x), rbind(assay(x), assay(next.x)))
    expect_identical(interactions(c.x), c(interactions(x), interactions(next.x)))
    
    expect_identical(nrow(rbind(x[0,], next.x[0,])), 0L) # Behaviour with empties.
    expect_identical(ncol(rbind(x[0,], next.x[0,])), ncol(x))
    expect_identical(nrow(rbind(x, next.x[0,])), nrow(x)) # Not fully equal, as regions have changed.
})

############################## 

test_that("sorting and deduplication work for IS objects", {
    o.x <- order(x)
    expect_identical(o.x, order(interactions(x)))
    expect_equal(sort(x), x[o.x,])

    set.seed(1004)
    x.1 <- x[sample(nrow(x), 100, replace=TRUE),]
    x.2 <- x[sample(nrow(x), 100, replace=TRUE),]
    o.x2 <- order(x.1, x.2)
    expect_identical(o.x2, order(interactions(x.1), interactions(x.2)))
    
    expect_identical(duplicated(x), duplicated(interactions(x)))
    temp.x <- rbind(x, x)    
    expect_identical(duplicated(temp.x), duplicated(interactions(temp.x)))
    expect_equal(x, unique(temp.x))
    
    expect_identical(duplicated(x, fromLast=TRUE), duplicated(interactions(x), fromLast=TRUE))
    expect_equal(x, unique(temp.x, fromLast=TRUE))
    
    expect_identical(order(x[0,]), integer(0))
    expect_identical(duplicated(x[0,]), logical(0))
})

test_that("anchor swapping works for IS objects", {        
    expect_identical(interactions(swapAnchors(x)), swapAnchors(interactions(x)))
    expect_identical(interactions(swapAnchors(x, mode='reverse')), swapAnchors(interactions(x), mode='reverse'))
    expect_identical(interactions(swapAnchors(x, mode='all')), swapAnchors(interactions(x), mode='all'))
})

test_that("splitting works for IS objects", {
    flen <- c(5L, 10L, 5L)
    f <- rep(1:3, flen)
    out <- split(x, f)
    expect_that(sapply(out, nrow), is_equivalent_to(flen))
    for (i in seq_along(flen)) {
        expect_equal(out[[i]], x[f==i])
    }
})

############################## 

test_that("name handling works properly for IS objects", {
    temp.x <- x
    ref.names <- paste0("X", seq_along(temp.x))
    names(temp.x) <- ref.names
    expect_identical(names(temp.x), ref.names)
    expect_identical(names(temp.x), names(interactions(temp.x)))
    expect_identical(names(temp.x[2:5]), ref.names[2:5])
    expect_identical(names(temp.x[2:5]), names(interactions(temp.x[2:5])))
    expect_identical(names(temp.x[2:5]), names(interactions(temp.x)[2:5]))
    
    combined <- rbind(temp.x, temp.x)
    expect_identical(names(combined), names(interactions(combined)))
    expect_identical(names(combined), names(c(interactions(temp.x), interactions(temp.x))))
    combined <- rbind(temp.x, x)
    expect_identical(names(combined), c(ref.names, character(length(x))))
    expect_identical(names(interactions(combined)), c(ref.names, character(length(x))))
})

############################## 

test_that("strictness is handled in IS objects", {
    sx <- InteractionSet(counts, GInteractions(all.anchor1, all.anchor2, all.regions, mode="strict"))
    expect_output(show(sx), "class: InteractionSet 
dim: 20 4 
metadata(0):
assays(1): ''
rownames: NULL
rowData names(0):
colnames(4): 1 2 3 4
colData names(0):
type: StrictGInteractions
regions: 30", 
    fixed=TRUE)
    
    expect_is(interactions(sx), "StrictGInteractions")
    expect_is(interactions(sx[,1:2]), "StrictGInteractions")
    expect_is(interactions(sx[1:3,]), "StrictGInteractions")
})

############################## 

test_that("pair coercion works for IS objects", {
    expect_identical(pairs(x), pairs(interactions(x)))
    expect_identical(pairs(x, id=TRUE), pairs(interactions(x), id=TRUE))
    expect_identical(pairs(x, as.grlist=TRUE), pairs(interactions(x), as.grlist=TRUE))
})

# Testing the linearize function for InteractionSet objects.
# library(InteractionSet); library(testthat); source("test-linear.R")

set.seed(200)
N <- 50
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))
all.regions <- unique(all.regions)
N <- length(all.regions)

Np <- 100
all.anchor1 <- sample(N, Np, replace=TRUE)
all.anchor2 <- sample(N, Np, replace=TRUE)
Nlibs <- 4
counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)
x <- InteractionSet(counts, GInteractions(all.anchor1, all.anchor2, all.regions))

####################################

test_that("conversion via linearization is working", {
    o <- order(all.regions)
    new.regions <- all.regions[o]
    new.pos <- integer(length(o))
    new.pos[o] <- seq_along(new.pos)
    new.anchor1 <- new.pos[all.anchor1]
    new.anchor2 <- new.pos[all.anchor2]

    for (interest in seq_len(N)) {
        cur.reg <- regions(x)[interest]
        out <- linearize(x, interest)
        chosen <- new.anchor1==interest | new.anchor2==interest
        expect_identical(assay(out), assay(x[chosen,]))
        expect_output(show(out), sprintf("class: RangedSummarizedExperiment 
dim: %i 4 
metadata(0):
assays(1): ''
rownames: NULL
rowData names(0):
colnames(4): 1 2 3 4
colData names(0):", sum(chosen)), fixed=TRUE)
    
        new.ranges <- anchors(x, type="first")
        not1 <- new.ranges!=cur.reg
        new.ranges[!not1] <- anchors(x, type="second")[!not1]
        new.ranges <- new.ranges[chosen]
        expect_identical(rowRanges(out), new.ranges)
    
        # Comparing with equivalent GRanges method.
        out2 <- linearize(x, cur.reg, type="equal")
        expect_equal(out, out2)
    
        # Also checking with GInteractions methods.
        out3 <- linearize(interactions(x), cur.reg, type="equal")
        expect_identical(out3, rowRanges(out2))
        out3 <- linearize(interactions(x), interest)
        expect_identical(out3, rowRanges(out2))
    }
})

test_that("linearization behaves correctly with silly inputs", {
    expect_identical(nrow(linearize(x, 0)), 0L)
    expect_identical(nrow(linearize(x[0,], 1)), 0L)
    expect_identical(nrow(suppressWarnings(linearize(x, GRanges("chrC", IRanges(1,1))))), 0L)
})

####################################

test_that("linearization works with multiple overlaps", {
    lenA <- max(end(regions(x)[seqnames(regions(x))=="chrA"]))
    x2 <- x
    mcols(x2)$Index <- seq_len(nrow(x2))
    out <- linearize(x2, GRanges("chrA", IRanges(1, lenA)))
    all.a <- anchors(x2[mcols(out)$Index])
    mcols(out)$Index <- NULL
   
    # Testing for proper merging of internal ranges. 
    is.same <- as.logical(seqnames(all.a$first)==seqnames(all.a$second))
    expect_identical(pmin(start(all.a$first), start(all.a$second))[is.same], start(rowRanges(out))[is.same])
    expect_identical(pmax(end(all.a$first), end(all.a$second))[is.same], end(rowRanges(out))[is.same])
    expect_identical(seqnames(all.a$first)[is.same], seqnames(rowRanges(out))[is.same])
    
    is.B1 <- as.logical(seqnames(all.a$first)=="chrB")
    expect_identical(all.a$first[is.B1], rowRanges(out)[is.B1])
    is.B2 <- as.logical(seqnames(all.a$second)=="chrB")
    expect_identical(all.a$second[is.B2], rowRanges(out)[is.B2])
    
    lenB <- max(end(regions(x)[seqnames(regions(x))=="chrB"]))
    expect_error(linearize(x2, GRanges(c("chrA", "chrB"), IRanges(1, c(lenA, lenB)))), "multi-chromosome sets of 'ref' are not supported")
})

test_that("linearization correctly ignores internal interactions", {
    lenA <- max(end(regions(x)[seqnames(regions(x))=="chrA"]))
    x2 <- x
    mcols(x2)$Index <- seq_len(nrow(x2))
    out <- linearize(x2, GRanges("chrA", IRanges(1, lenA)), internal=FALSE)
    all.a <- anchors(x2[mcols(out)$Index])
    mcols(out)$Index <- NULL
    
    is.diff <- as.logical(seqnames(all.a$first)!=seqnames(all.a$second))
    expect_true(all(is.diff))
    is.B1 <- as.logical(seqnames(all.a$first)=="chrB")
    expect_identical(all.a$first[is.B1], rowRanges(out)[is.B1])
    is.B2 <- as.logical(seqnames(all.a$second)=="chrB")
    expect_identical(all.a$second[is.B2], rowRanges(out)[is.B2])
})

# Testing what happens with region and interaction-specific metadata.

set.seed(100)
test_that("linearization preserves metadata", {
    out <- linearize(x, 1)
    mcols(x) <- DataFrame(Blah=runif(nrow(x)), Index=seq_len(nrow(x)))
    mcols(regions(x)) <- DataFrame(Foo=runif(length(regions(x))))

    out2 <- linearize(x, 1)
    expect_identical(colnames(mcols(out2)), c("Foo", "Blah", "Index"))
    expect_identical(mcols(out2)$Blah, mcols(x)$Blah[mcols(out2)$Index])

    a1 <- anchors(x, type="first", id=TRUE)
    a2 <- anchors(x, type="second", id=TRUE)
    index <- ifelse(a1!=1L, a1, a2)[mcols(out2)$Index]
    expect_identical(mcols(out2)$Foo, mcols(regions(x))$Foo[index])
})

# Tests the distance calculation methods for an InteractionSet.
# library(InteractionSet); library(testthat); source("test-dist.R")

set.seed(100)
N <- 50
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 100
all.anchor1 <- sample(N, Np, replace=TRUE)
all.anchor2 <- sample(N, Np, replace=TRUE)
Nlibs <- 4
counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)
x <- InteractionSet(counts, GInteractions(all.anchor1, all.anchor2, all.regions))

test_that("Paired distances are correct for IS objects", {
    a1 <- all.regions[all.anchor1]
    a2 <- all.regions[all.anchor2]
    swap <- a1 < a2
    temp <- a2[swap]
    a2[swap] <- a1[swap]
    a1[swap] <- temp
    
    is.intra <- intrachr(x)
    expect_identical(is.intra, as.logical(seqnames(a1)==seqnames(a2)))
    expect_identical(is.intra, pairdist(x, type="intra"))
    expect_identical(pairdist(x), ifelse(is.intra, as.integer(abs(start(a1)+end(a1)-start(a2)-end(a2))/2L), as.integer(NA))) # Don't use 'mid', it does its own rounding.
    expect_identical(pairdist(x,type="gap"), ifelse(is.intra, pmax(start(a1), start(a2)) - pmin(end(a1), end(a2)) -1L, as.integer(NA))) 
    expect_identical(pairdist(x,type="span"), ifelse(is.intra, pmax(end(a1), end(a2)) - pmin(start(a1), start(a2)) +1L, as.integer(NA))) 
    
    ax1 <- anchors(x, type="first", id=TRUE)
    ax2 <- anchors(x, type="second", id=TRUE)
    expect_identical(pairdist(x, type="diag"), ifelse(is.intra, pmax(ax1, ax2) - pmin(ax1, ax2), as.integer(NA)))
    
    # What happens with empty inputs?
    expect_identical(pairdist(x[0,]), integer(0))
    expect_identical(pairdist(x[0,], type="intra"), logical(0))
    expect_identical(pairdist(x[!is.intra,]), rep(as.integer(NA), sum(!is.intra)))
})

####################################################

set.seed(101)
Nr <- 100
Nc <- 200
all.anchor1 <- sample(N, Nr, replace=TRUE)
all.anchor2 <- sample(N, Nc, replace=TRUE)
counts <- matrix(rpois(Nr*Nc, lambda=10), Nr, Nc)
x <- ContactMatrix(counts, all.anchor1, all.anchor2, all.regions)

test_that("Paired distances are correct for CM objects", {
    a1 <- anchors(x, type="row")
    a2 <- anchors(x, type="column")
    is.intra <- intrachr(x) 
    expect_identical(is.intra, matrix(outer(as.character(seqnames(a1)), as.character(seqnames(a2)), "=="), nrow=Nr, ncol=Nc))
    expect_identical(is.intra, pairdist(x, type="intra"))
    
    ref <- abs(outer(start(a1)+end(a1), start(a2)+end(a2), "-"))
    ref <- ref/2L
    ref[!is.intra] <- NA
    storage.mode(ref) <- "integer"
    expect_identical(pairdist(x), ref)
                                                    
    ref <- abs(outer(anchors(x, type="row", id=TRUE), anchors(x, type="column", id=TRUE), "-"))
    ref[!is.intra] <- NA
    storage.mode(ref) <- "integer"
    expect_identical(pairdist(x, type="diag"), ref)
    
    as1 <- start(a1)
    as2 <- start(a2)
    ae1 <- end(a1)
    ae2 <- end(a2)
    ref <- outer(as1, as2, pmax) - outer(ae1, ae2, pmin) - 1L
    ref[!is.intra] <- NA                    
    expect_identical(pairdist(x,type="gap"), ref)
    
    ref <- outer(ae1, ae2, pmax) - outer(as1, as2, pmin) + 1L
    ref[!is.intra] <- NA                    
    expect_identical(pairdist(x,type="span"), ref)
    
    # Don't need to worry about NA's in 'x@matrix', they don't affect the calculation.
    # Still checking silly inputs, though:   
    expect_identical(pairdist(x[0,]), matrix(0L, 0, Nc))
    expect_identical(pairdist(x[,0], type="intra"), matrix(FALSE, Nr, 0))
    is.ra <- which(seqnames(anchors(x, type="row"))=="chrA")
    is.cb <- which(seqnames(anchors(x, type="column"))=="chrB")
    expect_identical(pairdist(x[is.ra,is.cb]), matrix(as.integer(NA), length(is.ra), length(is.cb)))
})

# Tests the construction and manipulation of ContactMatrix objects.
# library(InteractionSet); library(testthat); source("test-CM.R")

### Start of loop across multiple matrix types.
for (type in c("normal", "sparse", "dense")) { 
if (type=="sparse") { 
    mattype <- "dgCMatrix"
    makeMatrix <- function(x, r, c) {
        Matrix::sparseMatrix(rep(seq_len(r), c), rep(seq_len(c), each=r), x=x, dims=c(r, c))
    }
} else if (type=="normal") {
    makeMatrix <- base::matrix
    mattype <- "matrix"
} else {
    makeMatrix <- Matrix::Matrix
    mattype <- "dgeMatrix"
}
###

set.seed(4000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)),
                           IRanges(all.starts, all.ends))

Nr <- 10
Nc <- 20
all.anchor1 <- sample(N, Nr)
all.anchor2 <- sample(N, Nc)
counts <- makeMatrix(rpois(Nr*Nc, lambda=10), Nr, Nc)
x <- ContactMatrix(counts, all.anchor1, all.anchor2, all.regions)

###################################

test_that("show methods for CM objects are correct", {
    expect_output(show(x), sprintf("class: ContactMatrix 
dim: %i %i 
type: %s 
rownames: NULL
colnames: NULL
metadata(0):
regions: %i", Nr, Nc, mattype, N), 
    fixed=TRUE)
})

###################################

# Various common values needed in many individual test chunks. 
o <- order(all.regions)
ref.regions <- all.regions[o]
new.pos <- integer(length(o))
new.pos[o] <- seq_along(new.pos)
ref.anchor1 <- new.pos[all.anchor1]
ref.anchor2 <- new.pos[all.anchor2]

test_that("slots contain valid data and getters work for CM objects", {
    expect_s4_class(x, "ContactMatrix")
    expect_identical(as.matrix(x), counts)
    expect_true(!is.unsorted(regions(x)))
    expect_identical(regions(x), ref.regions)    

    expect_identical(anchors(x, id=TRUE, type="row"), ref.anchor1)
    expect_identical(anchors(x, id=TRUE, type="column"), ref.anchor2)
    expect_identical(anchors(x, id=TRUE), list(row=ref.anchor1, column=ref.anchor2))
    
    expect_identical(anchors(x, type="row"), ref.regions[ref.anchor1])
    expect_identical(anchors(x, type="column"), ref.regions[ref.anchor2])
    expect_identical(anchors(x), list(row=ref.regions[ref.anchor1], column=ref.regions[ref.anchor2]))
})

###################################

test_that("alternative constructors are correct", {
    x2 <- ContactMatrix(counts, all.regions[all.anchor1], all.regions[all.anchor2])
    was.used <- sort(unique(all.regions[union(all.anchor1, all.anchor2)])) # Only includes the regions actually used.
    expect_identical(regions(x2), was.used)
    expect_identical(anchors(x2), anchors(x))
    
    x3 <- ContactMatrix(counts, all.regions[all.anchor1], all.regions[all.anchor2], was.used)
    expect_identical(anchors(x3, id=TRUE), (anchors(x2, id=TRUE)))
    expect_identical(regions(x3), regions(x2))
})

test_that("errors are thrown with crappy inputs for CM objects", {
    expect_s4_class(ContactMatrix(makeMatrix(0, 4, 0), 1:4, integer(0), all.regions), "ContactMatrix") # No columns.
    four.peat <- GRanges("chrA", IRanges(1:4, 1:4))
    expect_s4_class(ContactMatrix(makeMatrix(0, 0, 4), integer(0), 1:4, four.peat), "ContactMatrix") # No rows.
    expect_s4_class(ContactMatrix(makeMatrix(0, 0, 4), GRanges(), four.peat), "ContactMatrix") # Nothing at all
    
    expect_error(ContactMatrix(makeMatrix(0, 3, 1), 1:4, 1, all.regions), "'matrix' nrow must be equal to length of 'anchor1'")
    expect_error(ContactMatrix(makeMatrix(0, 4, 0), 1:4, 1, all.regions), "'matrix' ncol must be equal to length of 'anchor2'")
    expect_error(ContactMatrix(makeMatrix(0, 4, 0), 0:3, 1:4, all.regions), "all anchor indices must be positive integers")
    expect_error(ContactMatrix(makeMatrix(0, 4, 0), c(1,2,3,NA), 1:4, all.regions), "all anchor indices must be finite integers")
    expect_error(ContactMatrix(makeMatrix(0, 4, 0), c(1,2,3,-1), 1:4, all.regions), "all anchor indices must be positive integers")
    expect_error(ContactMatrix(makeMatrix(0, 4, 0), c(1,2,3,length(all.regions)+1L), 1:4, all.regions), "all anchor indices must refer to entries in 'regions'")
    
    expect_identical(dim(ContactMatrix()), integer(2))
    if (type=="normal") { expect_identical(ContactMatrix(), ContactMatrix(makeMatrix(0L,0,0), integer(0), integer(0), GRanges())) }
})

###################################

test_that("setters on the anchors are functional for CM objects", {
    set.seed(4001)
    fresh.anchor1 <- sample(N, Nr)
    fresh.anchor2 <- sample(N, Nc)
    anchorIds(x) <- list(fresh.anchor1, fresh.anchor2)
    expect_identical(anchors(x, id=TRUE, type="row"), fresh.anchor1)
    expect_identical(anchors(x, id=TRUE, type="column"), fresh.anchor2)
    expect_error(anchorIds(x) <- list(fresh.anchor1, fresh.anchor2, fresh.anchor1), "must be a list of 2 numeric vectors")
    expect_error(anchorIds(x) <- list(fresh.anchor2, fresh.anchor2), "nrow must be equal to length of 'anchor1'")
    expect_error(anchorIds(x) <- list(fresh.anchor1, fresh.anchor1), "ncol must be equal to length of 'anchor2'")
    
    mod.x <- x
    anchorIds(x, type="row") <- ref.anchor1 # Restoring; checking that these calls also work.
    expect_identical(anchors(x, id=TRUE, type="row"), ref.anchor1)
    anchorIds(x, type="column") <- ref.anchor2
    expect_identical(anchors(x, id=TRUE, type="column"), ref.anchor2)
    anchorIds(mod.x, type="both") <- list(ref.anchor1, ref.anchor2) # Restoring.
    expect_identical(x, mod.x)
})
 
test_that("setters on the regions are functional for CM objects", {
    set.seed(4002)
    shuffled <- sample(100, N, replace=TRUE)
    regions(x)$score <- shuffled
    expect_identical(regions(x)$score, shuffled)
    expect_false(identical(regions(x), ref.regions))
    regions(x) <- ref.regions # Restoring.
    expect_true(identical(regions(x), ref.regions))
   
    x.dump <- x
    mod.ranges <- resize(regions(x), fix="center", width=50)
    new.ranges <- c(regions(x), mod.ranges) 
    expect_error(regions(x.dump) <- new.ranges, "assigned value must be of the same length")
    replaceRegions(x.dump) <- new.ranges
    expect_identical(anchors(x.dump), anchors(x))
    expect_identical(sort(new.ranges), regions(x.dump))
    expect_error(replaceRegions(x.dump) <- mod.ranges, "some existing ranges do not exist in replacement GRanges")
    
    x.dump2 <- x
    appendRegions(x.dump2) <- mod.ranges
    expect_identical(regions(x.dump), regions(x.dump2))
    expect_identical(anchors(x.dump), anchors(x.dump2))
    
    x.dump <- reduceRegions(x)
    expect_identical(anchors(x), anchors(x.dump))
    expect_identical(regions(x)[sort(unique(unlist(anchors(x, id=TRUE))))], regions(x.dump))
})

test_that("setters on the matrix are functional for CM objects", {
    x.dump <- x
    new.mat <- makeMatrix(sample(N, Nr*Nc, replace=TRUE), Nr, Nc)
    as.matrix(x.dump) <- new.mat
    expect_identical(new.mat, as.matrix(x.dump))
    as.matrix(x.dump) <- 1:Nr
    expect_identical(as.matrix(x.dump), makeMatrix(1:Nr, Nr, Nc))
    
    if (type=="sparse") { expect_error(as.matrix(x.dump) <- makeMatrix(1:Nr, Nr, 1), "replacement Matrix must have same dimensions as 'x'") }
})

###################################

test_that("subsetting works correctly for CM objects", {
    rchosen <- 1:5
    xsub <- x[rchosen,]
    expect_identical(nrow(xsub), length(rchosen))
    expect_identical(ncol(xsub), ncol(x))

    expect_identical(as.matrix(xsub), as.matrix(x)[rchosen,])
    expect_identical(regions(xsub), regions(x))
    expect_identical(anchors(xsub, type="row"), ref.regions[ref.anchor1][rchosen])
    expect_identical(anchors(xsub, type="column"), ref.regions[ref.anchor2])

    cchosen <- 10:20
    xsub <- x[,cchosen]
    expect_identical(ncol(xsub), length(cchosen))
    expect_identical(nrow(xsub), nrow(x))

    expect_identical(as.matrix(xsub), as.matrix(x)[,cchosen])
    expect_identical(regions(xsub), regions(x))
    expect_identical(anchors(xsub, type="row"), ref.regions[ref.anchor1])
    expect_identical(anchors(xsub, type="column"), ref.regions[ref.anchor2][cchosen])

    xsub <- subset(x,rchosen,cchosen)
    expect_identical(nrow(xsub), length(rchosen))
    expect_identical(ncol(xsub), length(cchosen))

    expect_identical(as.matrix(xsub), as.matrix(x)[rchosen,cchosen])
    expect_identical(regions(xsub), regions(x))
    expect_identical(anchors(xsub, type="row"), ref.regions[ref.anchor1][rchosen])
    expect_identical(anchors(xsub, type="column"), ref.regions[ref.anchor2][cchosen])

    expect_identical(nrow(x[0,]), 0L)
    expect_identical(ncol(x[,0]), 0L)
})

###################################

test_that("subset assignment works correctly for CM objects", {
    rchosen <- 1:5
    cchosen <- 10:20

    temp.x <- x
    temp.x[rchosen+5,] <- x[rchosen,]
    new.index <- seq_len(nrow(x))
    new.index[rchosen+5] <- rchosen
    expect_identical(as.matrix(temp.x), as.matrix(x)[new.index,])
    expect_identical(anchors(temp.x, type="row"), anchors(x, type="row")[new.index,])
    expect_identical(anchors(temp.x, type="column"), anchors(x, type="column"))
    
    temp.x <- x
    temp.x[,cchosen-9,] <- x[,cchosen]
    new.index <- seq_len(ncol(x))
    new.index[cchosen-9] <- cchosen
    expect_identical(as.matrix(temp.x), as.matrix(x)[,new.index])
    expect_identical(anchors(temp.x, type="row"), anchors(x, type="row"))
    expect_identical(anchors(temp.x, type="column"), anchors(x, type="column")[new.index,])
    
    temp.x <- x
    temp.x[0,] <- x[0,]
    expect_identical(temp.x, x)
    temp.x[,0] <- x[,0]
    expect_identical(temp.x, x)
    
    temp.x <- x
    anchorIds(temp.x[1:5,], type="row") <- 1:5
    expect_identical(anchors(temp.x, type="row", id=TRUE)[1:5], 1:5)
    anchorIds(temp.x[,1:5], type="column") <- 1:5
    expect_identical(anchors(temp.x, type="column", id=TRUE)[1:5], 1:5)
    expect_error(anchorIds(temp.x[1:5,1:5], type="row") <- 1:5+1L, "cannot modify row indices for a subset of columns")
    expect_error(anchorIds(temp.x[1:5,1:5], type="column") <- 1:5+1L, "cannot modify column indices for a subset of rows")
})

###################################

test_that("combining works correctly for CM objects", {
    xsub <- x[1:5,]
    xsub2 <- x[6:10,]
    expect_identical(rbind(xsub, xsub2), x)
    expect_error(rbind(xsub, xsub2[,1:2]), "column anchor indices must be identical")
    xsub <- x[,1:5]
    xsub2 <- x[,6:20]
    expect_identical(cbind(xsub, xsub2), x)
    expect_error(cbind(xsub, xsub2[1:5,]), "row anchor indices must be identical")
    
    xsub.mod <- xsub <- x[1:5,]
    chosen.cols <- setdiff(anchors(xsub.mod, type="column", id=TRUE), anchors(xsub.mod, type="row", id=TRUE))
    regions(xsub.mod)[chosen.cols] <- resize(regions(xsub.mod)[chosen.cols], width=5)
    byrow <- cbind(xsub, xsub.mod)
    expect_identical(anchors(byrow, type="row"), anchors(xsub, type="row"))
    expect_identical(anchors(byrow, type="column"), c(anchors(xsub, type="column"), anchors(xsub.mod, type="column")))
    expect_identical(as.matrix(byrow), cbind(as.matrix(xsub), as.matrix(xsub.mod)))
    
    xsub.mod <- xsub <- x[,1:5]
    chosen.cols <- setdiff(anchors(xsub.mod, type="row", id=TRUE), anchors(xsub.mod, type="column", id=TRUE))
    regions(xsub.mod)[chosen.cols] <- resize(regions(xsub.mod)[chosen.cols], width=5)
    bycol <- rbind(xsub, xsub.mod)
    expect_identical(anchors(bycol, type="row"), c(anchors(xsub, type="row"), anchors(xsub.mod, type="row")))
    expect_identical(anchors(bycol, type="column"), anchors(xsub, type="column"))
    expect_identical(as.matrix(bycol), rbind(as.matrix(xsub), as.matrix(xsub.mod)))
    
    expect_identical(nrow(rbind(x[0,], x[0,])), 0L) # Behaviour with empties.
    expect_identical(ncol(rbind(x[0,], x[0,])), ncol(x))
    expect_identical(rbind(x, x[0,]), x)
    expect_identical(nrow(cbind(x[,0], x[,0])), nrow(x))
    expect_identical(ncol(cbind(x[,0], x[,0])), 0L)
    expect_identical(cbind(x, x[,0]), x)
})

###################################

test_that("sorting and deduplication works correctly for CM objects", {
    o.x <- list(row=order(anchors(x, type="row")), column=order(anchors(x, type="column")))
    expect_identical(o.x, order(x))
    expect_identical(sort(x), x[o.x$row,o.x$column])
    
    temp.x <- rbind(x, x)    
    temp.x2 <- temp.x
    anchorIds(temp.x2) <- list(seq_len(nrow(temp.x2)), anchors(temp.x2, type="column", id=TRUE))
    o.x2 <- list(row=order(anchors(temp.x, type="row"), anchors(temp.x2, type="row")),
                 column=order(anchors(temp.x, type="column"), anchors(temp.x2, type="column")))
    expect_identical(o.x2, order(temp.x, temp.x2))
    
    is.dup <- list(row=duplicated(anchors(x, type="row")), column=duplicated(anchors(x, type="column")))
    expect_identical(is.dup, duplicated(x))
    
    temp.x <- rbind(x, x)    
    is.dup <- list(row=duplicated(anchors(temp.x, type="row")), column=duplicated(anchors(temp.x, type="column")))
    expect_identical(is.dup, duplicated(temp.x))
    expect_true(all(tail(is.dup$row, nrow(x)))) # if ordering is stable; only the first occurrence should be true.
    expect_identical(x, unique(temp.x))
    
    temp.x <- cbind(x, x)    
    is.dup <- list(row=duplicated(anchors(temp.x, type="row")), column=duplicated(anchors(temp.x, type="column")))
    expect_identical(is.dup, duplicated(temp.x))
    expect_true(all(tail(is.dup$column, ncol(x)))) 
    expect_identical(x, unique(temp.x))
    
    temp.x <- rbind(temp.x, temp.x)
    is.dup <- list(row=duplicated(anchors(temp.x, type="row"), fromLast=TRUE), column=duplicated(anchors(temp.x, type="column"), fromLast=TRUE))
    expect_identical(is.dup, duplicated(temp.x, fromLast=TRUE))
    expect_true(all(head(is.dup$column, ncol(x)))) # if ordering is stable; only the first occurrence should be true.
    expect_true(all(head(is.dup$row, nrow(x)))) # if ordering is stable; only the first occurrence should be true.
    expect_identical(x, unique(temp.x, fromLast=TRUE))
    
    dedupped <- duplicated(unique(temp.x))
    expect_false(any(dedupped$row))
    expect_false(any(dedupped$column))
})

# Testing the tranposition

test_that("transposition works correctly for CM objects", {
    tx <- t(x)
    expect_identical(anchors(x, type="row"), anchors(tx, type="column"))
    expect_identical(anchors(x, type="column"), anchors(tx, type="row"))
    expect_identical(t(x[0,]), tx[,0])
    expect_identical(t(x[,0]), tx[0,])
})

# Testing name setting and extraction.

test_that("name setting and getting work correctly for CM objects", {
    temp.x <- x
    rowref <- paste0("X", seq_len(nrow(temp.x)))
    rownames(temp.x) <- rowref
    colref <- paste0("Y", seq_len(ncol(temp.x)))
    colnames(temp.x) <- colref

    expect_identical(rownames(temp.x), rowref)
    expect_identical(colnames(temp.x), colref)
    expect_identical(rownames(temp.x[1:5,]), rowref[1:5])
    expect_identical(colnames(temp.x[,1:3]), colref[1:3])
    
    rcombined <- cbind(x, temp.x)
    expect_identical(rownames(rcombined), rowref)
    expect_identical(colnames(rcombined), c(character(ncol(x)), colref))
    ccombined <- rbind(x, temp.x)
    expect_identical(rownames(ccombined), c(character(nrow(x)), rowref))
    expect_identical(colnames(ccombined), colref)
    
    for (id in c(TRUE, FALSE)) {
        expect_identical(names(anchors(temp.x, id=id)[[1]]), rowref)
        expect_identical(names(anchors(temp.x, id=id)[[2]]), colref)
        expect_identical(names(anchors(temp.x, id=id, type="row")), rowref)
        expect_identical(names(anchors(temp.x, id=id, type="column")), colref)
    }
})

### End of loop.
}
###

# Testing the bounding box behaviour.
# library(InteractionSet); library(testthat); source("test-box.R")

set.seed(10000)
N <- 30
all.starts <- round(runif(N, 1, 100))
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)), IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
x <- GInteractions(all.anchor1, all.anchor2, all.regions)

# Setting up factors so that, within one level, all entries belong to the same chromosome.
all.chrs <- as.character(seqnames(regions(x)))

test_that("boundingBox works with a factor", {
    for (i in 1:3) { 
        f <- paste0(all.chrs[anchors(x, type="first", id=TRUE)],
                    all.chrs[anchors(x, type="second", id=TRUE)],
                    sample(i, Np, replace=TRUE))
    
        ref1 <- unlist(range(split(anchors(x, type="first"), f)))
        ref2 <- unlist(range(split(anchors(x, type="second"), f)))
        ref <- GInteractions(unname(ref1), unname(ref2))
        names(ref) <- names(ref1)
        expect_identical(ref, boundingBox(x, f))
    }
})

test_that("boundingBox works without a factor", {
    only.A <- all.chrs[anchors(x, type="first", id=TRUE)] == "chrA" & all.chrs[anchors(x, type="second", id=TRUE)] == "chrA"
    x.A <- x[only.A]
    ref1 <- range(anchors(x.A, type="first"))
    ref2 <- range(anchors(x.A, type="second"))
    ref <- GInteractions(unname(ref1), unname(ref2))
    names(ref) <- 1
    expect_identical(boundingBox(x.A), ref)

    only.AB <- all.chrs[anchors(x, type="first", id=TRUE)] == "chrA" & all.chrs[anchors(x, type="second", id=TRUE)] == "chrB"
    x.AB <- x[only.AB]
    ref1 <- range(anchors(x.AB, type="first"))
    ref2 <- range(anchors(x.AB, type="second"))
    ref <- GInteractions(unname(ref1), unname(ref2))
    names(ref) <- 1
    expect_identical(boundingBox(x.AB), ref)
})

test_that("boundingBox breaks with silly inputs", {
    expect_error(boundingBox(x), "multiple chromosomes for group '1'")
    f <- rep("whee", Np)
    f[-1] <- "YAY"
    expect_error(boundingBox(x,f), "multiple chromosomes for group 'YAY'")
    
    ref <- GInteractions(all.regions[0], all.regions[0])
    names(ref) <- character(0)
    expect_identical(boundingBox(x[0]), ref)
})

\name{Interaction distances}

\alias{pairdist}
\alias{pairdist,GInteractions-method}
\alias{pairdist,InteractionSet-method}
\alias{intrachr}
\alias{intrachr,GInteractions-method}
\alias{intrachr,InteractionSet-method}

\title{Get the linear distance for each interaction}

\description{
Compute the distance between interacting regions on the linear genome, for each pairwise interaction contained in a GInteractions or InteractionSet object.
}

\usage{
\S4method{pairdist}{GInteractions}(x, type="mid")
\S4method{pairdist}{InteractionSet}(x, type="mid")

\S4method{intrachr}{GInteractions}(x)
\S4method{intrachr}{InteractionSet}(x)
}

\arguments{
\item{x}{A \link{GInteractions} or \link{InteractionSet} object.}

\item{type}{
    A character string specifying the type of distance to compute.
    Can take values of \code{"mid"}, \code{"gap"}, \code{"span"}, \code{"diag"} or \code{"intra"}.
}
}

\value{
An integer or logical vector of distances.
}

\details{
For each interaction in \code{x}, the \code{pairdist} method computes the distance between the two interacting regions.
An integer vector is returned, with values computed according to the specified value of \code{type}:
\describe{
\item{\code{"mid"}:}{
    The distance between the midpoints of the two regions (rounded down to the nearest integer) is returned.
}

\item{\code{"gap"}:}{
    The length of the gap between the closest points of the two regions is computed - 
    negative lengths are returned for overlapping regions, indicating the length of the overlap.
}

\item{\code{"span"}:}{
    The distance between the furthermost points of the two regions is computed.
}

\item{\code{"diag"}:}{
    The difference between the anchor indices is returned.
    This corresponds to a diagonal on the interaction space when bins are used in the \code{regions} slot of \code{x}.
}
}

Interchromosomal interactions are marked with \code{NA}.
Alternatively, if \code{type="intra"}, a logical vector is returned indicating whether the interaction occurs between two regions on the same chromosome.
\code{intrachr(x)} is an alias for \code{pairdist(x, type="intra")}.

The return values of the assorted methods are the same for both GInteractions and InteractionSet objects.
Methods for the latter operate on the GInteractions object in the \code{interactions} slot.
}

\examples{
example(GInteractions, echo=FALSE)

pairdist(gi)
pairdist(gi, type="gap")
pairdist(gi, type="span")
pairdist(gi, type="diag")
intrachr(gi)

example(InteractionSet, echo=FALSE)

pairdist(iset)
pairdist(iset, type="gap")
pairdist(iset, type="span")
pairdist(iset, type="diag")
intrachr(iset)
}

\author{
Aaron Lun
}

\name{Interaction binding}
\alias{interaction-bind}

\alias{cbind,InteractionSet-method}
\alias{rbind,InteractionSet-method}
\alias{c,GInteractions-method}

\title{Interaction combining}

\description{
Methods to combine InteractionSet or GInteractions objects.
}

\usage{
\S4method{c}{GInteractions}(x, ..., recursive=FALSE)

\S4method{rbind}{InteractionSet}(..., deparse.level=1)
\S4method{cbind}{InteractionSet}(..., deparse.level=1)
}

\arguments{
\item{x}{A \link{GInteractions} or \link{InteractionSet} object.}

\item{...}{  
    For \code{rbind} and \code{c}, \code{...} contains GInteractions or InteractionSet objects to be combined row-wise.
    All objects must be of the same class.
    For \code{c}, any objects are additional to that already specified in \code{x}.
    For \code{cbind}, \code{...} contains InteractionSet objects to be combined column-wise.
}

\item{deparse.level}{An integer scalar; see \code{?base::\link[base]{cbind}} for a description of this argument.}

\item{recursive}{An integer scalar, ignored.}
}

\value{
A combined object of the same class as \code{x}.
}

\section{Details for GInteractions}{
\code{c} will concatenate GInteractions objects.
It will check whether the \code{regions} slot of all supplied objects are the same, in which case the regions and anchor indices are used directly.
Otherwise, the \code{regions} slot is set to a new GRanges object consisting of the (sorted) union of all \code{regions} across the input objects.
Anchor indices in each object are refactored appropriately to refer to the relevant entries in the new GRanges. 

Note that the column names in \code{mcols} must be identical across all supplied objects.
The column names of \code{mcols} for the \code{regions} slot must also be identical across objects.
If GInteractions objects of different strictness (i.e., StrictGInteractions and ReverseGInteractions) are concatenated,
    the returned object will be of the same class as the first supplied object.
}

\section{Details for InteractionSet}{
\code{cbind} will combine objects with the same interactions but different samples.
Errors will occur if the interactions are not identical between objects (i.e., must have same values in the \code{interactions} slots).
Additional restrictions apply on the column and assay names - see \code{\link[SummarizedExperiment]{cbind,SummarizedExperiment-method}} for details.

\code{rbind} will combine objects with the same samples but different interactions.
In this case, the interactions need not be identical, and will be concatenated using the methods described above for GInteractions objects.
Again, additional restrictions apply - see \code{\link[SummarizedExperiment]{rbind,SummarizedExperiment-method}} for details.
}

\examples{
example(GInteractions, echo=FALSE)
c(gi, gi)

new.gi <- gi
regions(new.gi) <- resize(regions(new.gi), width=20, fix="start")
c(gi, new.gi)

#################
# Same can be done for an InteractionSet object:

example(InteractionSet, echo=FALSE)
cbind(iset, iset)
rbind(iset, iset)

new.iset <- iset
regions(new.iset) <- resize(regions(new.iset), width=20, fix="start")
rbind(iset, new.iset)
}

\author{
Aaron Lun
}

\name{Linearize interactions}

\alias{linearize}
\alias{linearize,GInteractions,GRanges-method}
\alias{linearize,GInteractions,numeric-method}
\alias{linearize,InteractionSet,GRanges-method}
\alias{linearize,InteractionSet,numeric-method}

\title{Linearize 2D interactions into 1D ranges}

\description{
Convert interactions in two-dimensional space to one-dimensional ranges on the linear genome.
}

\usage{
\S4method{linearize}{GInteractions,numeric}(x, ref, internal=TRUE)

\S4method{linearize}{GInteractions,GRanges}(x, ref, ..., internal=TRUE)

\S4method{linearize}{InteractionSet,numeric}(x, ref, internal=TRUE)

\S4method{linearize}{InteractionSet,GRanges}(x, ref, ..., internal=TRUE)
}

\arguments{
\item{x}{A \link{GInteractions} or \link{InteractionSet} object.}

\item{ref}{ 
    A numeric vector or a \link[GenomicRanges]{GRanges} object, specifying the reference region(s) to use for linearization.
    If numeric, the entries should be indices pointing to a genomic interval in \code{regions(x)}.
}

\item{internal}{
    A logical scalar specifying whether interactions within \code{ref} should be reported. 
}

\item{...}{
    Other arguments, passed to \code{\link{overlapsAny}} in the GRanges methods.
}
}

\details{
This method identifies all interactions with at least one anchor region overlapping the specified region(s) in \code{ref}.
When \code{x} is a GInteractions object, the method returns a GRanges object with one entry per identified interaction, 
    where the coordinates are defined as the \emph{other} anchor region, i.e., the one that does \emph{not} overlap the reference region.

If both of the anchor regions for an interaction overlap the reference regions, the genomic interval spanned by both anchor regions is returned.
This is because it is not clear which region should be defined as the "other" anchor in such circumstances.
Note that this will fail if the reference regions occur across multiple chromosomes.
If \code{internal=FALSE}, interactions with both overlapping anchor regions are removed from the output.

When \code{x} is an InteractionSet object, a RangedSummarizedExperiment object is returned.
Each entry corresponds to an identified interaction with the non-overlapping anchor region stored in the \code{rowRanges}.
Experimental data associated with each identified interaction is stored in the various assays.

This method effectively converts two-dimesional interaction data into one-dimensional coverage across the linear genome.
It is useful when a particular genomic region is of interest - this can be used as \code{ref}, to examine the behaviour of all other regions relative to it.
For example, Hi-C data in \code{x} can be converted into pseudo-4C contact frequencies after linearization.

Disjoint ranges across multiple chromosomes are supported when \code{ref} is a GRanges object.
However, it usually only makes sense to use continguous ranges as a single bait region.
Similarly, if \code{ref} is numeric, it should refer to consecutive entries in \code{regions(x)} to specify the bait region.
}

\value{
A GRanges when \code{x} is a GInteractions object, and a RangedSummarizedExperiment when \code{x} is an InteractionSet object.
}

\examples{
example(InteractionSet, echo=FALSE)

# With integers
out <- linearize(iset, 1)
linearize(iset, 10)
linearize(iset, 20)

# With ranges
linearize(iset, regions(iset)[1], type="equal")
linearize(iset, regions(iset)[10], type="equal")
linearize(iset, regions(iset)[20], type="equal")
}

\name{Interaction compare}
\alias{interaction-compare}

\alias{order,InteractionSet-method}
\alias{sort,InteractionSet-method}
\alias{duplicated,InteractionSet-method}
\alias{unique,InteractionSet-method}

\alias{order,GInteractions-method}
\alias{sort,GInteractions-method}
\alias{duplicated,GInteractions-method}
\alias{unique,GInteractions-method}

\alias{match,GInteractions,GInteractions-method}
\alias{match,InteractionSet,GInteractions-method}
\alias{match,GInteractions,InteractionSet-method}
\alias{match,InteractionSet,InteractionSet-method}

\alias{swapAnchors}
\alias{swapAnchors,GInteractions-method}
\alias{swapAnchors,InteractionSet-method}

\alias{pcompare,GInteractions,GInteractions-method}

\title{Interaction comparisons}

\description{
Methods to order, compare and de-duplicate GInteractions or InteractionSet objects, based on the anchor indices.
}

\usage{
\S4method{order}{GInteractions}(..., na.last=TRUE, decreasing=FALSE)

\S4method{sort}{GInteractions}(x, decreasing=FALSE, ...)

\S4method{duplicated}{GInteractions}(x, incomparables=FALSE, fromLast=FALSE, ...)

\S4method{unique}{GInteractions}(x, incomparables=FALSE, fromLast=FALSE, ...)

\S4method{swapAnchors}{GInteractions}(x, mode=c("order", "reverse", "all"))

## Each of the above methods has an identical equivalent for
## InteractionSet objects (not shown for brevity).

\S4method{match}{GInteractions,GInteractions}(x, table, nomatch=NA_integer_, incomparables=NULL, ...)

\S4method{match}{GInteractions,InteractionSet}(x, table, nomatch=NA_integer_, incomparables=NULL, ...)

\S4method{match}{InteractionSet,GInteractions}(x, table, nomatch=NA_integer_, incomparables=NULL, ...)

\S4method{match}{InteractionSet,InteractionSet}(x, table, nomatch=NA_integer_, incomparables=NULL, ...)

\S4method{pcompare}{GInteractions,GInteractions}(x, y)
}

\arguments{
\item{...}{
    For \code{order}, one or more \link{InteractionSet} or \link{GInteractions} objects with the same number of rows.
    Otherwise, ignored in all other methods.
}

\item{x}{
    An \link{InteractionSet} or \link{GInteractions} object.
    For \code{pcompare}, a GInteractions object only.
}

\item{na.last}{
    A logical scalar indicating whether \code{NA} values should be ordered last.
    This should not be relevant as anchor indices should be finite.
}

\item{decreasing}{
    A logical scalar indicating whether rows should be sorted in decreasing order.
}

\item{incomparables}{
    A logical scalar.
    See \code{?\link[base]{match}} for a description of this in \code{match}.
    Otherwise, it is ignored.
}

\item{fromLast}{
    A logical scalar indicating whether the last entry of a repeated set in \code{x} should be considered as a non-duplicate.
}

\item{mode}{
    A string indicating what type of swapping should be performed in \code{swapAnchors}.
}

\item{table}{A GInteractions or InteractionSet object.}

\item{nomatch}{An integer scalar indicating the value to use for unmatched entries.}

\item{y}{A GInteractions object, of the same length as \code{x}.}
}

\value{
For \code{sort} and \code{unique}, a GInteractions or InteractionSet object is returned, depending on the class of \code{x}.

For \code{order} and \code{duplicated}, an integer vector of permutations or a logical vector of duplicate specifications is returned, respectively.
}

\section{Sorting and ordering}{
Sorting is performed based on the anchor indices of the GInteraction object.
Rows are ordered for increasing values of the \code{anchor1} slot - if these are equal, ordering is performed with values of the \code{anchor2} slot.
This equates to ordering by the coordinates directly, as the GRanges in the \code{regions} slot is always sorted.
Based on this, \code{sort} will return a sorted copy of \code{x} with permuted rows in increasing order.

The \code{order} method returns an integer vector indicating the permutation to rearrange \code{x} in increasing order.
If multiple objects are supplied to \code{order}, ordering will be computed using anchor indices from successive objects.
For example, ordering is first performed using anchor indices from the first object; 
    any rows with the same \code{anchor1} and \code{anchor2} will be split using the second object; and so on.

For both of these methods, the sorting can be reversed by setting \code{decreasing=TRUE}.
This will sort or order for decreasing values of \code{anchor1} and \code{anchor2}, rather than for increasing values.
}

\section{Removing duplicates}{
The \code{duplicated} method returns a logical vector indicating whether the rows of \code{x} are duplicated.
Duplicated rows are identified on the basis of identical entries in the \code{anchor1} and \code{anchor2} slots.
Values in other slots (e.g., in \code{mcols}) are ignored.
For a set of duplicated rows, the first occurrence in \code{x} is marked as the non-duplicate if \code{fromLast=FALSE}, and the last entry otherwise.

\code{unique} returns a GInteractions object where all duplicate rows have been removed from \code{x}.
This is equivalent to \code{x[!duplicated(x),]}, with any additional arguments passed to \code{duplicated} as specified.
}

\section{Matching and comparing}{
The \code{match} function will return an integer vector of length equal to that of \code{x}.
Each entry of the vector corresponds to an interaction in \code{x} and contains the index of the first interaction \code{table} with the same anchor regions.
Interactions in \code{x} without any matches are assigned \code{NA} values by default.
If the \code{regions} slot is not the same between \code{x} and \code{table}, \code{match} will call \code{\link{findOverlaps}} to identify exact two-dimensional overlaps.

The \code{pcompare} function will return an integer vector of length equal to \code{x} and \code{y}.
This performs parallel comparisons between corresponding entries in two GInteractions objects, based on the values of the anchor indices 
(\code{anchor1} first, and then \code{anchor2} if \code{anchor1} is tied).
Negative, zero and positive values indicate that the corresponding interaction in \code{x} is \dQuote{lesser}, equal or \dQuote{greater} than the corresponding interaction in \code{y}.
If the \code{regions} slot is not the same between the two objects, the union of regions for both objects will be used to obtain comparable indices.
}

\section{Swapping anchors}{
For GInteractions objects, \code{swapAnchors} returns a GInteractions object where \code{anchor1} and \code{anchor2} values are swapped.
If \code{mode="order"}, this is done so that all values in the \code{anchor2} slot are not less than values in \code{anchor1} in the returned object.
If \code{mode="reverse"}, all values in \code{anchor1} are not less than all values in \code{anchor2}.
If \code{mode="all"}, the anchor indices in \code{x} are directly swapped without consideration of ordering.

It is recommended to apply this method before running methods like \code{order} or \code{duplicated}.
This ensures that redundant permutations are coerced into the same format for a valid comparison.
In many applications, permutations of pairwise interactions are not of interest, i.e., an interaction between regions 1 and 2 is the same as that between 2 and 1.
Application of \code{swapAnchors} with \code{mode="order"} ensures that all indices are arranged in a comparable manner.
Alternatively, users can use a (Reverse)StrictGInteractions object which enforces a consistent arrangement of indices across interactions.
}

\section{Methods for InteractionSet objects}{
For all InteractionSet methods, the corresponding method is called on the GInteractions object in the \code{interactions} slot of the InteractionSet object.
Return values for each InteractionSet method is the same as those for the corresponding GInteractions method 
    - except for \code{sort} and \code{unique}, which return a row-permuted or row-subsetted InteractionSet, respectively, instead of a GInteractions object;
      and \code{swapAnchors}, which returns an InteractionSet object where the internal GInteractions has its anchor indices swapped around.      

Note that no additional information from the InteractionSet (beyond that in \code{interactions}) is used for sorting or duplicate marking,
    i.e., the assay or metadata values for each interaction are \emph{not} used in distinguishing rows with the same interaction.
For this reason, the \code{pcompare} method is not implemented for InteractionSet objects.
It makes little sense to do a parallel comparison in an InteractionSet without examining the data.
}

\examples{
example(GInteractions, echo=FALSE)

anchors(gi, id=TRUE)
anchors(swapAnchors(gi, mode="all"), id=TRUE)
gi <- swapAnchors(gi)

anchors(gi)
gi2 <- sort(gi)
gi2
anchors(gi2)

# Can also order them.
o <- order(gi)
o
stopifnot(all.equal(gi[o], gi2))

# Checking duplication.
summary(duplicated(gi))
summary(duplicated(c(gi, gi)))
stopifnot(all.equal(unique(gi), unique(c(gi, gi))))

# Matching and comparing.
another.gi <- gi[sample(length(gi))]
match(gi, another.gi)
match(gi, another.gi[1:5])

pcompare(gi, another.gi)

#################
# Same can be done for an InteractionSet object:

example(InteractionSet, echo=FALSE)
iset <- swapAnchors(iset)

anchors(iset)
anchors(sort(iset))
order(iset)
summary(duplicated(iset))
unique(iset)
}

\author{
Aaron Lun
}

\seealso{
\code{\link[base]{match}},
\code{\link[S4Vectors]{pcompare}}
}

\name{boundingBox}

\alias{boundingBox}
\alias{boundingBox,GInteractions-method}
\alias{boundingBox,InteractionSet-method}

\title{Get the bounding box}

\description{Computing a minimum bounding box for groups of pairwise interactions.}

\usage{
\S4method{boundingBox}{GInteractions}(x, f)

# Note, the same call is used for the InteractionSet method.
}

\arguments{
\item{x}{A \link{GInteractions} or \link{InteractionSet} object.}

\item{f}{
A factor or vector of length equal to that of \code{x}, indicating the group to which each pairwise interaction belongs.
}
}

\value{
A GInteractions object containing the coordinates of each bounding box.
}

\details{
For any group of pairwise interactions, the minimum bounding box is the smallest rectangle in the interaction space that contains all interactions in the group.
Each side of the box has coordinates spanning the most extreme anchor regions on the corresponding chromosome.
This is often useful for summarizing clusters of interactions.

Grouping of interactions is specified using \code{f}, where interactions in \code{x} with the same level of \code{f} are considered to be in the same group.
If \code{f} is not specified, all interactions in \code{x} are assumed to be in a single group (named as ``1'').
An error will be raised if a group spans multiple chromosomes for either the first or second anchor regions.

The function returns a GInteractions object containing the coordinates of the bounding boxes for all groups.
Each interaction represents a bounding box for a group, where the anchor regions represent the sides of the box.
Entries are named according to the levels of \code{f}, in order to specify which bounding box corresponds to which group.

It is recommended to run \code{\link{swapAnchors}} prior to computing the bounding box for intra-chromosomal groups.
If all \code{anchor1 >= anchor2} or all \code{anchor1 <= anchor2}, all interactions will lie on one side of the diagonal of the intra-chromosomal interaction space.
This results in the smallest possible minimum bounding box, which will only increase in size if interactions are placed on the other side of the diagonal.
Alternatively, users can specify a StrictGInteractions object as an input into \code{x}, in which \code{anchor1 <= anchor2} is enforced automatically.

% Bit of a pain to prove, but basically, if you flip a point to be above the diagonal, the Chebyshev distance to a point below the diagonal will always increase.
% This means that you must increase the size of one of your sides of your bounding box.
}

\author{
Aaron Lun
}

\examples{
example(GInteractions, echo=FALSE)

# Making up a sensible grouping.
gi <- sort(gi)
all.chrs <- as.character(seqnames(regions(gi)))
f <- paste0(all.chrs[anchors(gi, type="first", id=TRUE)], ".",
            all.chrs[anchors(gi, type="second", id=TRUE)])

boundingBox(gi, f)
boundingBox(swapAnchors(gi), f)

# Fails for multiple chromosomes
try(out <- boundingBox(gi))
in.A <- f=="chrA.chrA"
out <- boundingBox(gi[in.A])
}

\name{ContactMatrix accessors}
\alias{ContactMatrix-access}

\alias{anchors,ContactMatrix-method}
\alias{anchorIds,ContactMatrix-method}
\alias{anchors<-,ContactMatrix-method}
\alias{anchorIds<-,ContactMatrix-method}

\alias{regions,ContactMatrix-method}
\alias{regions<-,ContactMatrix-method}

\alias{replaceRegions<-,ContactMatrix-method}
\alias{appendRegions<-,ContactMatrix-method}
\alias{reduceRegions,ContactMatrix-method}

\alias{show,ContactMatrix-method}
\alias{as.matrix,ContactMatrix-method}
\alias{t,ContactMatrix-method}

\alias{as.matrix<-}
\alias{as.matrix<-,ContactMatrix-method}

\alias{dim,ContactMatrix-method}
\alias{dimnames,ContactMatrix-method}
\alias{dimnames<-,ContactMatrix-method}
\alias{dimnames<-,ContactMatrix,ANY-method}
\alias{length,ContactMatrix-method}

\alias{seqinfo,ContactMatrix-method}
\alias{seqinfo<-,ContactMatrix-method}

\title{ContactMatrix accessors}

\description{
Methods to get and set fields in an ContactMatrix object.
} 

\usage{
\S4method{anchors}{ContactMatrix}(x, type="both", id=FALSE)
\S4method{anchorIds}{ContactMatrix}(x, type="both")
\S4method{anchorIds}{ContactMatrix}(x, type="both") <- value

\S4method{regions}{ContactMatrix}(x)
\S4method{regions}{ContactMatrix}(x) <- value
\S4method{replaceRegions}{ContactMatrix}(x) <- value
\S4method{appendRegions}{ContactMatrix}(x) <- value
\S4method{reduceRegions}{ContactMatrix}(x)

\S4method{show}{ContactMatrix}(x)
\S4method{as.matrix}{ContactMatrix}(x)
\S4method{as.matrix}{ContactMatrix}(x) <- value
\S4method{t}{ContactMatrix}(x)

\S4method{dim}{ContactMatrix}(x)
\S4method{dimnames}{ContactMatrix}(x)
\S4method{dimnames}{ContactMatrix}(x) <- value
\S4method{length}{ContactMatrix}(x)

\S4method{seqinfo}{ContactMatrix}(x)
\S4method{seqinfo}{ContactMatrix}(x, new2old = NULL,
    pruning.mode = c("error", "coarse", "fine", "tidy")) <- value
}

\arguments{
\item{x}{A \link{ContactMatrix} object.}

\item{type}{A string specifying which anchors are to be extracted or replaced.}

\item{id}{A scalar logical indicating whether indices should be returned.
If \code{FALSE}, \link[GenomicRanges]{GRanges} are returned instead.}

\item{value}{
    For \code{anchorIds<-}, a list of two integer vectors when \code{type="both"}.
    The first and second vectors must have length equal to the number of rows and columns of \code{x}, respectively.
    For \code{type="row"} or \code{"column"}, only one vector needs to be supplied corresponding to either the rows or columns.

    For \code{regions<-}, a GRanges object of length equal to that of \code{regions(x)}.
    For \code{newRegions<-}, a GRanges object that is a superset of all entries in \code{regions(x)} involved in interactions.
    For \code{appendRegions<-}, a GRanges of any length containing additional regions.

    For \code{as.matrix<-}, a matrix-like object of the same dimensions as that in the \code{matrix} slot.
    
    For \code{dimnames<-}, a list of two character vectors corresponding to the row and column names, respectively.
    These can also be passed separately via \code{rownames<-} and \code{colnames<-}.

    For \code{seqinfo<-}, a \link[Seqinfo]{Seqinfo} object containing the sequence information.
}

\item{new2old, pruning.mode}{See \code{?"\link[Seqinfo]{seqinfo<-}"} for details.}
}

\details{
The return value of \code{anchors} varies depending on \code{type} and \code{id}:
\itemize{
\item If \code{id=FALSE}, a GRanges object is returned containing the regions specified by the \code{anchor1} or \code{anchor2} slots in \code{x}, 
    for \code{type=="row"} or \code{"column"}, respectively.
\item If \code{id=FALSE} and \code{type="both"}, a list is returned with two entries \code{row} and \code{column}, containing regions specified by \code{anchor1} and \code{anchor2} respectively.
\item If \code{id=TRUE}, the integer vectors in the \code{anchor1} or \code{anchor2} slots of \code{x} are returned directly, depending on \code{type}. 
    A list of length two is returned for \code{type="both"}, containing both of these vectors.
}
Note that \code{anchorIds} is equivalent to calling \code{anchors} with \code{id=TRUE}.

Replacement in \code{anchorIds<-} can only be performed using anchor indices.
If \code{type="both"}, a list of two integer vectors is required in \code{value}, specifying the indices of the row- and column-wise interacting regions in \code{x}.
If \code{type="row"} or \code{"column"}, an integer vector is required to replace the existing row- or column-wise indices in the \code{anchor1} or \code{anchor2} slot, respectively.
   
For \code{regions}, a GRanges is returned equal to the \code{regions} slot in \code{x}.
For \code{regions<-}, the GRanges in \code{value} is used to replace the \code{regions} slot.
Resorting of the replacement GRanges is performed automatically, along with re-indexing of the anchors.    
In addition, the input GRanges must be of the same length as the existing object in the \code{regions} slot.
The \code{newRegions} replacement method can take variable length GRanges, but requires that the replacement contain (at least) all ranges contained in \code{anchors(x)}.
The \code{appendRegions} replacement method appends extra intervals to the existing \code{regions} slot of \code{x}.
The \code{reduceRegions} method removes unused entries in the \code{regions} slot, to save memory -- see \code{\link{reduceRegions,GInteractions-method}} for more details.

The \code{show} method will print out various details of the object, such as the dimensions of the \code{matrix} slot and the length of the \code{regions} slot.
The \code{as.matrix} method will return the value of the \code{matrix} slot, containing a matrix-like object of interaction data.
Replacement with a matrix-like object of the same dimensions can be performed using the \code{as.matrix<-} function.
The \code{t} method will transpose the matrix, i.e., switch the rows and columns (and switch the vectors in the \code{anchor1} and \code{anchor2} slots).

The \code{dim} method will return a vector of length 2, containing the dimensions of the \code{matrix} slot.
The \code{dimnames} method will return a list of length 2, containing the row and column names of \code{matrix} (these can be modified with the \code{dimnames<-} method).
The \code{length} method will return an integer scalar corresponding to the total number of entries in the \code{matrix} slot.

The \code{seqinfo} method will return the sequence information of the \link[GenomicRanges]{GRanges} in the \code{regions} slot.
This can be replaced with the \code{seqinfo<-} method. 

As the ContactMatrix class inherits from the \link[S4Vectors]{Annotated} class, additional metadata can be stored in the \code{metadata} slot.
This can be accessed or modified with \code{\link[S4Vectors]{metadata<-}}.
}

\value{
For the getters, values in various slots of \code{x} are returned, while for the setters, the slots of \code{x} are modified accordingly -- see Details.
}

\examples{
example(ContactMatrix, echo=FALSE) # Generate a nice object.
show(x)

# Various matrix methods:
as.matrix(x)
t(x)

nrow(x)
ncol(x)
length(x)

# Accessing anchor ranges or indices:
anchors(x)
anchors(x, type="row")
anchors(x, id=TRUE)

anchors(x, id=TRUE, type="row")
anchors(x, id=TRUE, type="column")

# Modifying anchor indices:
nregs <- length(regions(x))
anchorIds(x) <- list(sample(nregs, nrow(x), replace=TRUE),
                   sample(nregs, ncol(x), replace=TRUE))
anchors(x, id=TRUE, type="row")
anchors(x, id=TRUE, type="column")

# Accessing or modifying regions:
regions(x)
regions(x)$score <- runif(length(regions(x)))

new.ranges <- c(regions(x), resize(regions(x), fix="center", width=50))
try(regions(x) <- new.ranges) # Fails
replaceRegions(x) <- new.ranges # Succeeds

length(regions(x))
appendRegions(x) <- GRanges("chrA", IRanges(5:10+1000, 1100+5:10), score=runif(6))
length(regions(x))

reduceRegions(x)

# Setting metadata
metadata(x)$name <- "I am a contact matrix"
metadata(x)
}

\author{
Aaron Lun
}

\name{linkOverlaps}
\alias{linkOverlaps}

\alias{linkOverlaps,GInteractions,Vector,Vector-method}
\alias{linkOverlaps,InteractionSet,Vector,Vector-method}
\alias{linkOverlaps,GInteractions,Vector,missing-method}
\alias{linkOverlaps,InteractionSet,Vector,missing-method}

\alias{linkOverlaps,GInteractions,Hits,Hits-method}
\alias{linkOverlaps,InteractionSet,Hits,Hits-method}
\alias{linkOverlaps,GInteractions,Hits,missing-method}
\alias{linkOverlaps,InteractionSet,Hits,missing-method}

\title{Link overlapping regions}

\description{
Identify interactions that link two sets of regions by having anchor regions overlapping one entry in each set.
}

\usage{
\S4method{linkOverlaps}{GInteractions,Vector,Vector}(query, subject1, subject2, ..., 
    ignore.strand=TRUE, use.region="both")

\S4method{linkOverlaps}{InteractionSet,Vector,Vector}(query, subject1, subject2, ..., 
    ignore.strand=TRUE, use.region="both")
}

\arguments{
\item{query}{A \link{GInteractions} or \link{InteractionSet} object.}

\item{subject1, subject2}{A \link[S4Vectors]{Vector} object defining a set of genomic intervals, such as a \link[GenomicRanges]{GRanges} or \link[GenomicRanges]{GRangesList}.
\code{subject2} may be missing.
Alternatively, both \code{subject1} and \code{subject2} may be Hits objects, see below.}
 
\item{..., ignore.strand}{Additional arguments to be passed to \code{\link[GenomicRanges]{findOverlaps}}.
Note that \code{ignore.strand=TRUE} by default, as genomic interactions are usually unstranded.}

\item{use.region}{
A string specifying which \code{query} regions should be used to overlap which \code{subject}.
Ignored if \code{subject2} is missing.
}
}

\details{
This function identifies all interactions in \code{query} where one anchor overlaps an entry in \code{subject1} and the other anchor overlaps an entry in \code{subject2}.
It is designed to be used to identify regions that are linked by interactions in \code{query}.
For example, one might specify genes as \code{subject1} and enhancers as \code{subject2}, to identify all gene-enhancer contacts present in \code{query}.
This is useful when the exact pairings between \code{subject1} and \code{subject2} are undefined.

The function returns a DataFrame specifying the index of the interaction in \code{query}; the index of the overlapped region in \code{subject1};
    and the index of the overlapped region in \code{subject2}.
If multiple regions in \code{subject1} and/or \code{subject2} are overlapping the anchor regions of a particular interaction,
    all combinations of two overlapping regions (one from each \code{subject*} set) are reported for that interaction.

By default, \code{use.region="both"} such that overlaps will be considered between any first/second interacting region in \code{query} and either \code{subject1} or \code{subject2}.
If \code{use.region="same"}, overlaps will only be considered between the first interacting region in \code{query} and entries in \code{subject1}, 
    and between the second interacting region and \code{subject2}.
The opposite applies with \code{use.region="reverse"}, where the first and second interacting regions are overlapped with \code{subject2} and \code{subject1} respectively.

If \code{subject2} is not specified, links within \code{subject1} are identified instead, i.e., \code{subject2} is set to \code{subject1}.
In such cases, the returned DataFrame is such that the first subject index is always greater than the second subject index, to avoid redundant permutations.
}

\section{Using Hits as input}{
Hits objects can be used for the \code{subject1} and \code{subject2} arguments.
These should be constructed using \code{\link[GenomicRanges]{findOverlaps}} with \code{regions(query)} as the query and the genomic regions of interest as the subject. 
For example, the calls below:
\preformatted{> linkOverlaps(query, subject1) # 1
> linkOverlaps(query, subject1, subject2) # 2}

will yield exactly the same output as:
\preformatted{> olap1 <- findOverlaps(regions(query), subject1)
> linkOverlaps(query, olap1) # 1
> olap2 <- findOverlaps(regions(query), subject2)
> linkOverlaps(query, olap1, olap2) # 2}

This is useful in situations where \code{regions(query)} and the genomic regions in \code{subject1} and \code{subject2} are constant across multiple calls to \code{linkOverlaps}.
In such cases, the overlaps only need to be calculated once, avoiding redundant work within \code{linkOverlaps}.
}

\value{
A DataFrame of integer indices indicating which elements of \code{query} link which elements of \code{subject1} and \code{subject2}.
}

\seealso{
\code{\link{findOverlaps,GInteractions,Vector-method}}
}

\author{
Aaron Lun
}

\examples{
example(GInteractions, echo=FALSE)

all.genes <- GRanges("chrA", IRanges(0:9*10, 1:10*10))
all.enhancers <- GRanges("chrB", IRanges(0:9*10, 1:10*10))

out <- linkOverlaps(gi, all.genes, all.enhancers)
head(out)

out <- linkOverlaps(gi, all.genes)
head(out)

# Same methods apply for InteractionSet objects.

example(InteractionSet, echo=FALSE)
out <- linkOverlaps(iset, all.genes, all.enhancers)
out <- linkOverlaps(iset, all.genes)
}

\name{updateObject}
\alias{updateObject}
\alias{updateObject,GInteractions-method}
\title{Update a GInteractions object}
\description{Update an old GInteractions object to the latest format.}
\usage{
\S4method{updateObject}{GInteractions}(object, ..., verbose = FALSE)
}
\arguments{
\item{object}{A old \link{GInteractions} object.}

\item{...}{Additional arguments that are ignored.}

\item{verbose}{Logical scalar indicating whether a message should be emitted as the object is updated.}
}
\value{
An updated version of \code{object}.
}

\name{ContactMatrix sorting}

\alias{ContactMatrix-sort}
\alias{order,ContactMatrix-method}
\alias{sort,ContactMatrix-method}
\alias{duplicated,ContactMatrix-method}
\alias{unique,ContactMatrix-method}

\title{ContactMatrix sorting and ordering}

\description{
Methods to sort and order ContactMatrix objects, based on the anchor indices.
Also, methods to remove duplicate rows or columns in each ContactMatrix.
}

\usage{
\S4method{order}{ContactMatrix}(..., na.last=TRUE, decreasing=FALSE)
\S4method{sort}{ContactMatrix}(x, decreasing=FALSE, ...)
\S4method{duplicated}{ContactMatrix}(x, incomparables=FALSE, fromLast=FALSE, ...)
\S4method{unique}{ContactMatrix}(x, incomparables=FALSE, fromLast=FALSE, ...)
}

\arguments{
\item{...}{
    For \code{sort}, one or more \link{ContactMatrix} objects with the same dimensions.
    Otherwise, ignored in all other methods.
}

\item{x}{A \link{ContactMatrix} object.}

\item{na.last}{
    A logical scalar indicating whether \code{NA} values should be ordered last.
    This should not be relevant as anchor indices should be finite.
}

\item{decreasing}{
    A logical scalar indicating whether sorting should be performed in decreasing order.
}

\item{incomparables}{
    A logical scalar, ignored.
}

\item{fromLast}{
    A logical scalar indicating whether the last entry of a repeated set in \code{x} should be considered as a non-duplicate.
}
}

\value{
For \code{sort} and \code{unique}, a ContactMatrix object is returned with sorted or unique rows/columns, respectively.

For \code{order}, a list of two integer vectors is returned containing row/column permutations.

For \code{duplicated}, a list of logical vectors is returned specifying which rows/columns are duplicated.
}

\details{
Sorting is performed based on the anchor indices of the ContactMatrix object.
Rows are ordered for increasing values of the \code{anchor1} slot, while columns are ordered for increasing values of the \code{anchor2} slot.
This equates to ordering by the coordinates directly, as the GRanges in the \code{regions} slot is always sorted.
Based on this, \code{sort,ContactMatrix-method} will return a sorted copy of \code{x} with permuted rows/columns in increasing order.
This can be set to decreasing order by specifying \code{decreasing=TRUE}.

\code{order,ContactMatrix-method} returns a list of 2 integer vectors.
The first vector contains the permutation to rearrange the rows of \code{x} in increasing order, while the second vector does the same for the columns of \code{x}.
If multiple objects are supplied to \code{order}, ordering will be computed using anchor indices from successive objects.
In other words, ordering will be performed using anchor indices from the first object; 
    any rows with the same \code{anchor1} or columns with the same \code{anchor2} will be split using the corresponding indices in the second object; and so on.

\code{duplicated,ContactMatrix-method} returns a list of two logical vectors.
The first vector indicates whether rows are duplicated, based on identical values in the \code{anchor1} slot.
The second does the same for columns based on the \code{anchor2} slot.
For a set of duplicated rows or columns, the first occurrence of that row/column is marked as the non-duplicate if \code{fromLast=FALSE}, and the last entry otherwise.

\code{unique,ContactMatrix-method} returns an ContactMatrix object where all duplicate rows and columns have been removed from \code{x}.
This is equivalent to subsetting based on the non-duplicate rows and columns defined in \code{duplicated,ContactMatrix-method}.

Note that sorting and duplicate identification only use the anchor indices.
The values of the \code{matrix} slot are \emph{not} used in distinguishing between rows or columns with the same index.
}

\examples{
example(ContactMatrix, echo=FALSE)

anchors(x)
x2 <- sort(x)
x2
anchors(x2)

# Can also order them.
o <- order(x)
o
stopifnot(all.equal(x[o$row,o$column], x2))

# Checking duplication.
duplicated(x)
duplicated(rbind(x, x))
stopifnot(all.equal(unique(x), unique(rbind(x, x))))
}

\author{
Aaron Lun
}

\name{Convert classes}

\alias{inflate}
\alias{inflate,GInteractions-method}
\alias{inflate,InteractionSet-method}
\alias{deflate}
\alias{deflate,ContactMatrix-method}

\title{Convert between classes}

\description{
Inflate a GInteractions or InteractionSet into a ContactMatrix, or deflate a ContactMatrix to an InteractionSet.
}

\usage{
\S4method{inflate}{GInteractions}(x, rows, columns, fill=TRUE, swap=TRUE, sparse=FALSE, ...)

\S4method{inflate}{InteractionSet}(x, rows, columns, assay=1L, sample=1L, fill, swap=TRUE, sparse=FALSE, ...)

\S4method{deflate}{ContactMatrix}(x, collapse=TRUE, extract, use.zero, use.na, ...)
}

\arguments{
\item{x}{
    A \link{GInteractions} or \link{InteractionSet} object for \code{inflate}, or a \link{ContactMatrix} object for \code{deflate}.
}

\item{rows, columns}{
    An integer, logical or character vector, a GRanges object or \code{NULL}, indicating the regions of interest to be used as the rows or columns of the ContactMatrix.
}

\item{assay}{
    A numeric scalar indicating the assay of the InteractionSet object, from which values are extracted to fill the ContactMatrix.
}

\item{sample}{
    A numeric scalar indicating the sample (i.e., column) of the assay to extract values to fill the ContactMatrix.
}

\item{fill}{
    A vector (usually logical or numeric) of length equal to \code{nrow(x)}, containing values with which to fill the ContactMatrix.
    If specified, this overrides extraction of assay values for \code{inflate,InteractionSet-method}.
}

\item{swap}{
    A logical scalar indicating whether filling should also be performed after swapping anchor indices.
}

\item{sparse}{ 
    A logical scalar indicating whether the inflated matrix should use a sparseMatrix representation.
}

\item{collapse}{
    A logical scalar indicating whether duplicated interactions should be removed from \code{x} prior to deflation.
}

\item{extract}{
    A logical vector or matrix indicating which entries of \code{x} to convert into pairwise interactions.
}

\item{use.zero, use.na}{ 
    A logical scalar indicating whether to convert zero or \code{NA} entries into pairwise interactions.
}

\item{...}{
    For \code{inflate}, additional arguments to pass to \code{\link{overlapsAny}} when \code{rows} or \code{columns} is a GRanges.

    For \code{deflate}, additional arguments to pass to the InteractionSet constructor.
}
}

\value{
For \code{inflate}, a ContactMatrix is returned.

For \code{deflate}, an InteractionSet object is returned.
}

\section{Inflating to a ContactMatrix}{
The \code{inflate} method will return a ContactMatrix where the rows and columns correspond to specified regions of interest in \code{rows} and \code{columns}.
Regions can be specified by supplying an object of various types: 
\itemize{
\item{If it is an integer vector, it is assumed to refer to intervals in the \code{regions} slot of the input object \code{x}.
Values of the vector need not be sorted or unique, but must lie within \code{[1, regions(x)]}.}
\item{If it is a logical vector, it will subset to retain intervals in \code{regions(x)} that are \code{TRUE}.}
\item{If it is a character vector, it is assumed to contain the names of the reference sequences of interest (i.e., chromosome names).}
\item{If it is a GRanges object, \code{\link{overlapsAny}} will be called to identify the overlapping intervals of \code{regions(x)}.}
\item{If it is \code{NULL}, all regions in \code{regions(x)} will be used to construct that dimension of the ContactMatrix.}
}

For the GInteractions method, values in the matrix are filled based on user-supplied values in \code{fill}.
Each element of \code{fill} corresponds to an interaction in \code{x} and is used to set the matrix entry at the matching row/column.
Some entries of the matrix will correspond to pairwise interactions that are not present in \code{x} - these are filled with \code{NA} values.

By default, filling is repeated after swapping the anchor indices.
This means that the value of the matrix at (1, 2) will be the same as that at (2, 1), i.e., the matrix is symmetric around the diagonal of the interaction space.
However, if \code{swap=FALSE}, filling is performed so that the first and second anchor indices correspond strictly to rows and columns, respectively.
This may be preferable if the order of the anchors contains some relevant information. 
In all cases, if duplicated interactions are present in \code{x} (and redundant permutations, when \code{swap=TRUE}), one will be arbitrarily chosen to fill the matrix.
 
For the InteractionSet \code{inflate} method, entries in the matrix are filled in based on the values in the first sample of the first assay when \code{fill} is missing.
For more complex \code{x}, values from different assays and samples can be extracted using the \code{assay} and \code{sample} arguments.
Note that if \code{fill} is specified, it will override any extraction of values from the assays.

If \code{sparse=TRUE}, \code{inflate} will return a ContactMatrix containing a \link[Matrix]{sparseMatrix} in the \code{matrix} slot.
Here, entries without a corresponding interaction in \code{x} are set to zero, not \code{NA}.
See below for some considerations when interpreting zeroes and \code{NA}s in contact matrices.

The default \code{fill=TRUE} has the effect of producing a logical sparse matrix in the output ContactMatrix, indicating which pairs of regions were present in \code{x}.
}

\section{Deflating from a ContactMatrix}{
The \code{deflate} method will return an InteractionSet where each relevant entry in the ContactMatrix is converted into a pairwise interaction.
Relevant entries are defined as those that are non-zero, if \code{use.zero} is \code{FALSE}; and non-\code{NA}, if \code{use.na} is \code{FALSE}.
If \code{x} contains a sparseMatrix representation, the former is set to \code{FALSE} while the latter is set to \code{TRUE}, if either are not specified.
For all other matrices, \code{use.zero=TRUE} and \code{use.na=FALSE} by default.

If \code{extract} is specified, this overrides all values of \code{use.zero} and \code{use.na}.
A typical application would be to \code{deflate} a number of ContactMatrix objects with the same \code{extract} matrix.
This ensures that the resulting InteractionSet objects can be easily combined with \code{cbind}, as the interactions are guaranteed to be the same.
Otherwise, different interactions may be extracted depending on the presence of zero or \code{NA} values.

The values of all matrix entries are stored as a one-sample assay, with each value corresponding to its pairwise interaction after conversion.
Duplicate interactions are removed by default, along with redundant permutations of the anchor indices.
These can be included in the returned object by setting \code{collapse=FALSE}.
This setting will also store the pairs as a GInteractions object, rather than using the default StrictGInteractions object where duplicates are not stored.

Additional arguments can be used to specify the \code{colData} and \code{metadata}, which are stored in the ContactMatrix itself.
}

\section{Interpreting zeroes in a sparse matrix}{
Storing data as a sparseMatrix may be helpful as it is more memory-efficient for sparse areas of the interaction space.
However, users should keep in mind that the zero values in the sparseMatrix may not represent zeroes in \code{fill}.
The majority of these values are likely to be zero just because there was no corresponding interaction in \code{x} to set it to a non-zero value.

Whether or not this is a problem depends on the application.
For example, if \code{fill} represents count data and only interactions with non-zero counts are stored in \code{x}, then setting all other entries to zero is sensible.
However, in other cases, it is not appropriate to fill entries corresponding to missing interactions with zero.
If \code{fill} represents, e.g., log-fold changes, then setting missing entries to a value of zero will be misleading.
One could simply ignore zeroes altogether, though this will also discard entries that are genuinely zero. 

These problems are largely avoided with the default dense matrices, where missing entries are simply set to \code{NA}.
}

\examples{
example(InteractionSet, echo=FALSE)

inflate(iset, 1:10, 1:10)
inflate(iset, 1:10, 1:10, sparse=TRUE)
inflate(iset, 1:10, 1:5+10)
inflate(iset, "chrA", 1:5+10)
inflate(iset, "chrA", "chrB")
inflate(iset, "chrA", GRanges("chrB", IRanges(1, 10)))

y <- inflate(iset, 1:10, 1:10)
iset2 <- deflate(y)
iset2
assay(iset2)

y <- inflate(iset, 1:10, 1:10, swap=FALSE)
iset2 <- deflate(y)
iset2
assay(iset2)

# Testing with different fillings:
y <- inflate(iset, 1:10, 1:10, sample=2)
iset2 <- deflate(y)
assay(iset2)

y <- inflate(iset, 1:10, 1:10, fill=rowSums(assay(iset)))
iset2 <- deflate(y)
assay(iset2)

y2 <- inflate(interactions(iset), 1:10, 1:10, rowSums(assay(iset)))
identical(y, y2) # should be TRUE

# Effect of 'collapse'
y <- inflate(iset, c(8, 1:10), 1:10)
deflate(y)
deflate(y, collapse=FALSE)
}

\author{
Aaron Lun
}

\name{Interaction subsetting}
\alias{interaction-subset}

\alias{[,InteractionSet,ANY-method}
\alias{[,InteractionSet,ANY,ANY-method}
\alias{[,InteractionSet,ANY,ANY,ANY-method}
\alias{[<-,InteractionSet,ANY,ANY,InteractionSet-method}
\alias{subset,InteractionSet-method}

\alias{[<-,GInteractions,ANY,GInteractions-method}

\title{Interaction subsetting and combining}

\description{
Methods to subset or combine InteractionSet or GInteractions objects.
}

\usage{
\S4method{[}{InteractionSet,ANY,ANY}(x, i, j, ..., drop=TRUE)

\S4method{[}{InteractionSet,ANY,ANY,InteractionSet}(x, i, j) <- value

\S4method{subset}{InteractionSet}(x, i, j)

\S4method{[}{GInteractions,ANY,GInteractions}(x, i) <- value
}

\arguments{
\item{x}{A \link{GInteractions} or \link{InteractionSet} object.}

\item{i, j}{
    A vector of logical or integer subscripts.
    For InteractionSet objects, these indicate the rows and columns to be subsetted for \code{i} and \code{j}, respectively.
    Rows correspond to pairwise interactions while columns correspond to samples.
    For GInteractions objects, \code{i} indicates the genomic interactions to be retained.
    \code{j} is ignored as there is no concept of samples in this class.
}

\item{..., drop}{Additional arguments that are ignored.}

\item{value}{
    A GInteractions or InteractionSet object with length or number of rows equal to length of \code{i} (or that of \code{x}, if \code{i} is not specified).
    For InteractionSet objects, the number of columns must be equal to the length of \code{j} (or number of columns in \code{x}, if \code{j} is not specified).
}
}

\value{
A subsetted object of the same class as \code{x}.
}

\section{Details for GInteractions}{
Subsetting operations are not explicitly listed above as they inherit from the \link[S4Vectors]{Vector} class.
They will return a GInteractions object containing the specified interactions.
Values of the \code{anchor1} and \code{anchor2} slots will be appropriately subsetted in the returned object, along with any metadata in \code{mcols}.
However, note that the value of \code{regions} will not be modified by subsetting.

For short index vectors, subsetting a GInteractions object prior to calling \code{\link{anchors}} may be much faster than the reverse procedure.
This is because the \code{\link{anchors}} getter will construct a GRanges(List) containing the genomic loci for all pairwise interactions.
Subsetting beforehand ensures that only loci for the desired interactions are included.
This avoids constructing the entire object just to subset it later.

Subset assignment will check if the \code{regions} are identical between \code{x} and \code{value}.
If not, the \code{regions} slot in the output object will be set to a sorted union of all \code{regions} from \code{x} and \code{value}.
Indices are refactored appropriately to point to the entries in the new \code{regions}.
}

\section{Details for InteractionSet}{
Subsetting behaves in much the same way as that for the SummarizedExperiment class.
Interactions are treated as rows and will be subsetted as such.
All subsetting operations will return an InteractionSet with the specified interactions (rows) or samples (columns).
Again, note that the value of \code{regions} will not be modified by subsetting.
}

\examples{
example(GInteractions, echo=FALSE)

# Subsetting:    
gi[1,]
gi[1:2,]
gi[3]
gi[3:4]

temp.gi <- gi
temp.gi[3:4] <- gi[1:2]

# Splitting:
f <- sample(4, length(gi), replace=TRUE)
out <- split(gi, f)
out[[1]]

#################
# Same can be done for an InteractionSet object:

example(InteractionSet, echo=FALSE)

# Subsetting:    
iset[1,]
iset[1:2,]
iset[,1]
iset[,1:2]
iset[3,3]
iset[3:4,3:4]

# Splitting:
out <- split(iset, f)
out[[1]]
}

\author{
Aaron Lun
}

\name{Interaction accessors}
\alias{interaction-access}

\alias{anchors}
\alias{anchors,InteractionSet-method}
\alias{anchors,GInteractions-method}

\alias{anchorIds}
\alias{anchorIds,InteractionSet-method}
\alias{anchorIds,GInteractions-method}

\alias{first}
\alias{first,InteractionSet-method}
\alias{first,GInteractions-method}
\alias{second}
\alias{second,InteractionSet-method}
\alias{second,GInteractions-method}

\alias{anchors<-}
\alias{anchors<-,InteractionSet-method}
\alias{anchors<-,GInteractions-method}
\alias{anchors<-,StrictGInteractions-method}
\alias{anchors<-,ReverseStrictGInteractions-method}
\alias{anchorIds<-}
\alias{anchorIds<-,InteractionSet-method}
\alias{anchorIds<-,GInteractions-method}
\alias{anchorIds<-,StrictGInteractions-method}
\alias{anchorIds<-,ReverseStrictGInteractions-method}

\alias{regions}
\alias{regions,InteractionSet-method}
\alias{regions,GInteractions-method}
\alias{regions<-}
\alias{regions<-,InteractionSet-method}
\alias{regions<-,GInteractions-method}

\alias{show,GInteractions-method}
\alias{show,InteractionSet-method}

\alias{replaceRegions<-}
\alias{replaceRegions<-,InteractionSet-method}
\alias{replaceRegions<-,GInteractions-method}

\alias{appendRegions<-}
\alias{appendRegions<-,InteractionSet-method}
\alias{appendRegions<-,GInteractions-method}

\alias{reduceRegions}
\alias{reduceRegions,InteractionSet-method}
\alias{reduceRegions,GInteractions-method}

\alias{seqinfo,InteractionSet-method}
\alias{seqinfo,GInteractions-method}
\alias{seqinfo<-,InteractionSet-method}
\alias{seqinfo<-,GInteractions-method}

\alias{interactions}
\alias{interactions,InteractionSet-method}
\alias{interactions<-}
\alias{interactions<-,InteractionSet-method}

\alias{length,GInteractions-method}
\alias{as.data.frame,GInteractions-method}
\alias{$,GInteractions-method}
\alias{$<-,GInteractions-method}

\alias{mcols,InteractionSet-method}
\alias{mcols<-,InteractionSet-method}

\alias{names,GInteractions-method}
\alias{names<-,GInteractions-method}
\alias{names,InteractionSet-method}
\alias{names<-,InteractionSet-method}

\title{Interaction accessors}

\description{
Methods to get and set fields in an InteractionSet or GInteractions object.
} 

\usage{
\S4method{anchors}{GInteractions}(x, type="both", id=FALSE)
\S4method{anchors}{InteractionSet}(x, type="both", id=FALSE)

\S4method{anchorIds}{GInteractions}(x, type="both")
\S4method{anchorIds}{InteractionSet}(x, type="both")

\S4method{first}{GInteractions}(x)
\S4method{first}{InteractionSet}(x)

\S4method{second}{GInteractions}(x)
\S4method{second}{InteractionSet}(x)

\S4method{regions}{GInteractions}(x)
\S4method{regions}{InteractionSet}(x)

\S4method{seqinfo}{GInteractions}(x)
\S4method{seqinfo}{InteractionSet}(x)

\S4method{show}{GInteractions}(x)
\S4method{show}{InteractionSet}(x)

\S4method{anchorIds}{GInteractions}(x, type="both") <- value
\S4method{anchorIds}{InteractionSet}(x, type="both") <- value

\S4method{regions}{GInteractions}(x) <- value
\S4method{regions}{InteractionSet}(x) <- value

\S4method{seqinfo}{GInteractions}(x, new2old = NULL, 
    pruning.mode = c("error", "coarse", "fine", "tidy")) <- value
\S4method{seqinfo}{InteractionSet}(x, new2old = NULL, 
    pruning.mode = c("error", "coarse", "fine", "tidy")) <- value

\S4method{replaceRegions}{GInteractions}(x) <- value
\S4method{replaceRegions}{InteractionSet}(x) <- value

\S4method{appendRegions}{GInteractions}(x) <- value
\S4method{appendRegions}{InteractionSet}(x) <- value

\S4method{reduceRegions}{GInteractions}(x)
\S4method{reduceRegions}{InteractionSet}(x)

\S4method{names}{GInteractions}(x)
\S4method{names}{InteractionSet}(x)

\S4method{names}{GInteractions}(x) <- value
\S4method{names}{InteractionSet}(x) <- value

\S4method{anchors}{StrictGInteractions}(x, type="both") <- value
\S4method{anchors}{ReverseStrictGInteractions}(x, type="both") <- value

\S4method{length}{GInteractions}(x)
\S4method{as.data.frame}{GInteractions}(x, row.names=NULL, optional=FALSE, ...)
\S4method{$}{GInteractions}(x, name)
\S4method{$}{GInteractions}(x, name) <- value

\S4method{interactions}{InteractionSet}(x)
\S4method{interactions}{InteractionSet}(x) <- value

\S4method{mcols}{InteractionSet}(x, use.names=FALSE)
\S4method{mcols}{InteractionSet}(x) <- value
}

\arguments{
\item{x}{An \link{InteractionSet} or \link{GInteractions} object.}

\item{type}{a string specifying which anchors are to be extracted or replaced.}

\item{id}{a scalar logical indicating whether indices or GRanges should be returned.
If \code{FALSE}, \link[GenomicRanges]{GRanges} are returned instead.}

\item{new2old, pruning.mode}{Additional arguments to pass to \code{\link[Seqinfo]{seqinfo}}.}

\item{name}{a string indicating the field of \code{mcols} to be accessed or modified for a GInteractions object.}

\item{value}{
    For \code{anchorIds<-} and \code{type="first"} or \code{"second"}, an integer vector of length equal to the number of rows in \code{x}.
    For \code{type="both"}, a list of two such vectors must be supplied.

    For \code{regions<-}, a GRanges object of length equal to that of \code{regions(x)}.
    For \code{replaceRegions<-}, a GRanges object that is a superset of all entries in \code{regions(x)} involved in interactions.
    For \code{appendRegions<-}, a GRanges of any length containing additional regions.

    For \code{seqinfo<-}, a SeqInfo object containing chromosome length data for all regions.
    For \code{interactions<-}, a GInteractions object of length equal to that of \code{interactions(x)}.  
    For \code{mcols<-}, a DataFrame with number of rows equal to the length of \code{x}.
    For \code{names<-}, a character vector of length equal to that of \code{x}.
}

\item{row.names, optional, ...}{
    Additional arguments, see \code{?\link[base]{as.data.frame}} for more details.
}

\item{use.names}{
    A logical scalar, see \code{?\link[S4Vectors]{mcols}} for more details.
}
}

\section{Anchor manipulations for GInteractions}{
The return value of \code{anchors} varies depending on \code{type} and \code{id}:
\itemize{
\item If \code{id=FALSE}, a GRanges object is returned containing the regions specified by the \code{anchor1} or \code{anchor2} slots in \code{x}, 
    for \code{type=="first"} or \code{"second"}, respectively.
The \code{first} and \code{second} methods are synonyms for \code{anchors} in these respective cases.
\item If \code{id=FALSE} and \code{type="both"}, a list is returned with two entries \code{first} and \code{second}, containing regions specified by \code{anchor1} and \code{anchor2} respectively.
\item If \code{id=TRUE} and \code{type="both"}, a list is returned containing the two integer vectors in the \code{anchor1} or \code{anchor2} slots of \code{x}.
Otherwise, each vector is returned corresponding to the requested value of \code{type}.
}
Note that \code{anchorIds} is the same as calling \code{anchors} with \code{id=TRUE}.

Replacement in \code{anchorIds<-} requires anchor indices rather than a GRanges object. 
If \code{type="both"}, a list of two integer vectors is required in \code{value}, specifying the indices of the interacting regions in \code{regions(x)}.
If \code{type="first"} or \code{"second"}, an integer vector is required to replace the existing values in the \code{anchor1} or \code{anchor2} slot, respectively.
If the object is a StrictGInteractions, indices are automatically swapped so that \code{anchor1 >= anchor2} for each interaction.
The opposite applies if the object is a ReverseStrictGInteractions.
}

\section{Region manipulations for GInteractions}{
For \code{regions}, a GRanges is returned equal to the \code{regions} slot in \code{x}.
For \code{regions<-}, the GRanges in \code{value} is used to replace the \code{regions} slot.
Resorting of the replacement GRanges is performed automatically, along with re-indexing of the anchors.    
In addition, the replacement must be of the same length as the existing object in the \code{regions} slot.

The \code{replaceRegions} replacement method can take variable length GRanges, but requires that the replacement contain (at least) all ranges contained in \code{anchors(x)}.
The \code{appendRegions} replacement method appends extra intervals to the existing \code{regions} slot of \code{x}.
This is more efficient than calling \code{replaceRegions} on a concatenated object with \code{regions(x)} and the extra intervals.

The \code{reduceRegions} method will return a GInteractions object where the \code{regions} slot is reduced to only those entries used in \code{anchors(x)}.
This may save some memory in each object by removing unused regions.
However, this is not recommended for large workflows with many GInteractions objects.
R uses a copy-on-write memory management scheme, so only one copy of the GRanges in \code{regions} should be stored so long as it is not modified in different objects.
}

\section{Other methods for GInteractions}{
For access and setting of all other slots, see \link[S4Vectors]{Vector} for details on the appropriate methods.
This includes \code{mcols} or \code{metadata}, to store interactions-specific or experiment-wide metadata, respectively.
The \code{length} method will return the number of interactions stored in \code{x}.

The \code{show} method will print out the class, the number of pairwise interactions, and the total number of regions in the GInteractions object.
The number and names of the various metadata fields will also be printed.
The \code{as.data.frame} method will return a data.frame object containing the coordinates for the two anchor regions as well as any element-wise metadata.

The \code{seqinfo} method will return the sequence information of the \link[GenomicRanges]{GRanges} in the \code{regions} slot.
This can be replaced with the \code{seqinfo<-} method. 
}

\section{Details for InteractionSet}{
Almost all InteractionSet methods operate by calling the equivalent method for the GInteractions object, and returning the resulting value.
The only exception is \code{interactions}, which returns the GInteractions object in the \code{interactions} slot of the InteractionSet object.
This slot can also be set by supplying a valid GInteractions object in \code{interactions<-}.

The \code{show} method will print information equivalent to that done for a SummarizedExperiment object.
An additional line is added indicating the number of regions in the \code{regions} slot of the object.

For access and setting of all other slots, see the documentation for the \link[SummarizedExperiment]{SummarizedExperiment} class.
This includes \code{assays}, \code{colData}, \code{mcols} or \code{metadata}, which can all be applied to InteractionSet objects.
}

\section{Handling different metadata}{
Note that there are several options for metadata - experiment-wide metadata, which goes into \code{metadata(x)<-};
    region-specific metadata (e.g., adjacent genes, promoter/enhancer identity, GC content), which goes into \code{mcols(regions(x))<-};
    and interaction-specific metadata (e.g., interaction distance, interaction type), which goes directly into \code{mcols(x)<-}.
This is applicable to both GInteractions and InteractionSet objects.
In addition, library-specific data (e.g., library size) should be placed into \code{colData(x)<-} for InteractionSet objects.

Users should take care with the differences in the \code{$} and \code{$<-} operators between these two classes.
For GInteractions objects, this will access or modify fields in the \code{mcols} slot, i.e., for interaction-specific metadata.
For InteractionSet objects, this will access or modify fields in the \code{colData} slot, i.e., for library-specific data.
The difference in behaviour is due to the concept of libraries in the InteractionSet, which is lacking in the GInteractions class.

For InteractionSet objects, the \code{mcols} getter and setter functions operate on the GInteractions object stored in \code{interactions} slot,
    rather than accessing the \code{elementMetadata} slot of the SummarizedExperiment base class.
This makes no difference for practical usage in the vast majority of cases, 
    except that any metadata stored in this manner is carried over when the GInteractions object is extracted with \code{interactions(x)}.
Similarly, the \code{names} getter and setter will operate the names of the internal GInteractions object.
However, the \code{metadata} getter and setter will operate on the SummarizedExperiment base class, \emph{not} on the internal GInteractions object.
}

\value{
For the getters, values in various slots of \code{x} are returned, while for the setters, the slots of \code{x} are modified accordingly -- see Details.
}

\examples{
example(GInteractions, echo=FALSE) # Generate a nice object.
show(gi)

# Accessing anchor ranges or indices:
anchors(gi)
anchors(gi, type="first")
anchors(gi, id=TRUE)

anchors(gi, id=TRUE, type="first")
anchors(gi, id=TRUE, type="second")

# Modifying anchor indices:
nregs <- length(regions(gi))
mod <- list(sample(nregs, length(gi), replace=TRUE),
            sample(nregs, length(gi), replace=TRUE))
anchorIds(gi) <- mod
anchors(gi, id=TRUE, type="first")
anchors(gi, id=TRUE, type="second")

anchorIds(gi, type="both") <- mod
anchorIds(gi, type="first") <- mod[[1]]
anchorIds(gi, type="first") <- mod[[2]]

# Accessing or modifying regions:
regions(gi)
reduceRegions(gi)
regions(gi)$score <- runif(length(regions(gi)))

new.ranges <- c(regions(gi), resize(regions(gi), fix="center", width=50))
try(regions(gi) <- new.ranges) # Fails
replaceRegions(gi) <- new.ranges # Succeeds

length(regions(gi))
appendRegions(gi) <- GRanges("chrA", IRanges(5:10+1000, 1100+5:10), score=runif(6))
length(regions(gi))

seqinfo(gi)
seqinfo(gi) <- Seqinfo(seqnames=c("chrA", "chrB"), seqlengths=c(1000, 2000))

# Accessing or modifying metadata.
gi$score <- runif(length(gi))
mcols(gi)
as.data.frame(gi)

#################
# Same can be done for an InteractionSet object:

example(InteractionSet, echo=FALSE)

anchors(iset)
regions(iset)
reduceRegions(iset)
regions(iset)$score <- regions(gi)$score
replaceRegions(iset) <- new.ranges

seqinfo(iset)
seqinfo(iset) <- Seqinfo(seqnames=c("chrA", "chrB"), seqlengths=c(1000, 2000))

# Standard SE methods also available:
colData(iset)
metadata(iset)
mcols(iset)

# Note the differences in metadata storage:
metadata(iset)$name <- "metadata for SE0"
metadata(interactions(iset))$name <- "metadata for GI"

iset$lib.size <- runif(ncol(iset))*1e6
colData(iset)
mcols(iset) # untouched by "$" operator

mcols(iset)$whee <- runif(nrow(iset))
mcols(iset)
mcols(interactions(iset)) # preserved

names(iset) <- paste0("X", seq_along(iset))
names(iset)
names(interactions(iset))
}

\author{
Aaron Lun
}

\name{InteractionSet class}
\docType{class}

\alias{InteractionSet-class}
\alias{InteractionSet}
\alias{InteractionSet,ANY,GInteractions-method}
\alias{InteractionSet,missing,missing-method}

\title{InteractionSet class and constructors}

\description{
The InteractionSet class stores information about pairwise genomic interactions, and is intended for use in data analysis from Hi-C or ChIA-PET experiments.
Each row of the InteractionSet corresponds to a pairwise interaction between two loci, as defined in the GInteractions object.
Each column corresponds to a library or sample.
Each InteractionSet also contains one or more assays, intended to hold experimental data about interaction frequencies for each interaction in each sample.
}

\usage{
\S4method{InteractionSet}{ANY,GInteractions}(assays, interactions, ...)

\S4method{InteractionSet}{missing,missing}(assays, interactions, ...)
}

\arguments{
\item{assays}{
    A numeric matrix or a list or \code{SimpleList} of matrices, containing data for each interaction.
}

\item{interactions}{
    A GInteractions object of length equal to the number of rows in \code{assays}.
}

\item{...}{Other arguments to be passed to the \code{\link[SummarizedExperiment]{SummarizedExperiment}} constructor.}
}

\value{
For the constructors, an InteractionSet object is returned.
}

\details{
The InteractionSet class inherits from the \link[SummarizedExperiment]{SummarizedExperiment} class and has access to all of its data members and methods.
It also contains an additional \code{interactions} slot which holds a GInteractions object (or an object from any derived classes, e.g., StrictGInteractions).
Each row of the InteractionSet object corresponds to a pairwise interaction between two genomic loci in \code{interactions}.

The constructor will return an InteractionSet object containing all of the specified information 
    - for \code{InteractionSet,missing,missing-method}, an empty InteractionSet object is returned.
Note that any \code{metadata} arguments will be placed in the metadata of the internal SummarizedExperiment object, 
    \emph{not} the metadata of the internal GInteractions object.
This is consistent with the behaviour of similar classes like RangedSummarizedExperiment.
}

\examples{
example(GInteractions, echo=FALSE)
Nlibs <- 4
counts <- matrix(rpois(Np*Nlibs, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)

iset <- InteractionSet(counts, gi)
iset <- InteractionSet(counts, gi, colData=DataFrame(lib.size=1:Nlibs*1000))
iset <- InteractionSet(counts, gi, metadata=list(name="My Hi-C data"))

# Note differences in metadata storage:
metadata(iset)
metadata(interactions(iset))
}

\author{
Aaron Lun
}

\seealso{
\code{?"\link{interaction-access}"},
\code{?"\link{interaction-subset}"},
\code{?"\link{interaction-compare}"}
}

\name{pairs}
\alias{pairs}
\alias{pairs,GInteractions-method}
\alias{pairs,InteractionSet-method}
\alias{makeGInteractionsFromGRangesPairs}

\title{Extract paired ranges}

\description{
Represent interactions in a GInteractions or Interaction object as a Pairs, SelfHits or GRangesList object.
}

\usage{
\S4method{pairs}{GInteractions}(x, id=FALSE, as.grlist=FALSE)

\S4method{pairs}{InteractionSet}(x, id=FALSE, as.grlist=FALSE)

makeGInteractionsFromGRangesPairs(x)
}

\arguments{
\item{x}{
For \code{pairs}, a \link{GInteractions} or \link{InteractionSet} object.
For \code{makeGInteractionsFromGRangesPairs}, a \link[S4Vectors]{Pairs} object containing two parallel \link[GenomicRanges]{GRanges}.
}

\item{id}{A logical scalar specifying whether indices should be returned instead of regions.}

\item{as.grlist}{A logical scalar specifying whether a GRangesList should be returned.}
}

\value{
For \code{pairs}, if \code{id=TRUE}, a SelfHits object is returned. 
Otherwise, if \code{as.grlist=TRUE}, a \link[GenomicRanges]{GRangesList} object is returned.
Otherwise, a Pairs object is returned.

For \code{makeGInteractionsFromGRangesPairs}, a GInteractions object is returned.
}

\details{
Recall that the GInteractions object stores anchor regions for each interaction in two parallel GRanges, 
    where corresponding entries between the two GRanges constitute the pair of regions for one interaction.
These parallel ranges can be extracted and stored as a Pairs object for further manipulation.
This is similar to the GRangesList reported by \code{\link{anchors}} with \code{type="both"} and \code{id=FALSE}.
The reverse conversion is performed using \code{makeGInteractionsFromGRangesPairs}.

An alternative representation involves storing the two anchors for each interaction in a single GRanges of length 2.
Multiple interactions are then stored as a GRangesList, along with any metadata and sequence information.
This is returned if \code{as.grlist=FALSE}, may be more useful in some applications where the two interacting regions must be in the same GRanges.
Finally, if \code{id=TRUE}, the anchor indices are extracted and returned as a SelfHits object.
This may be useful for graph construction.
}

\examples{
example(GInteractions, echo=FALSE)
y <- pairs(gi)
y
makeGInteractionsFromGRangesPairs(y)

pairs(gi, id=TRUE)
pairs(gi, as.grlist=TRUE)

example(InteractionSet, echo=FALSE)
pairs(iset)
pairs(iset, id=TRUE)
pairs(iset, as.grlist=TRUE)
}

\author{
Aaron Lun
}

\name{GInteractions class}
\docType{class}

\alias{GInteractions-class}
\alias{StrictGInteractions-class}
\alias{ReverseStrictGInteractions-class}
\alias{GInteractions}
\alias{GInteractions,numeric,numeric,GRanges-method}
\alias{GInteractions,GRanges,GRanges,GenomicRanges_OR_missing-method}
\alias{GInteractions,missing,missing,GenomicRanges_OR_missing-method}

\title{GInteractions class and constructors}

\description{
The GInteractions class stores pairwise genomic interactions, and is intended for use in data analysis from Hi-C or ChIA-PET experiments.
Each row of the GInteractions corresponds to a pairwise interaction between two loci, with indexing to improve computational efficiency.
}

\usage{
\S4method{GInteractions}{numeric,numeric,GRanges}(anchor1, anchor2, regions, metadata=list(), mode="normal", ...)

\S4method{GInteractions}{GRanges,GRanges,GenomicRanges_OR_missing}(anchor1, anchor2, regions, metadata=list(), mode="normal", ...)

\S4method{GInteractions}{missing,missing,GenomicRanges_OR_missing}(anchor1, anchor2, regions, metadata=list(), mode="normal", ...)
}

\arguments{
\item{anchor1, anchor2}{
    Either a pair of numeric vectors containing indices to \code{regions}, or a pair of GRanges objects specifying the interacting loci.
    Lengths of both \code{anchor1} and \code{anchor2} must be equal.
}

\item{regions}{
    A GRanges object containing the coordinates of the interacting regions.
    This is only mandatory if \code{anchor1} and \code{anchor2} are numeric vectors.
} 

\item{metadata}{
    An optional \code{list} of arbitrary content describing the overall experiment.
}

\item{mode}{
    A string indicating what type of GInteractions object should be constructed.
}

\item{...}{
    Optional metadata columns.
}
}

\value{
For the constructors, a GInteractions (or StrictGInteractions, or ReverseStrictGInteractions) object is returned.
}

\section{Description of the class}{
The GInteractions class inherits from the \link[S4Vectors]{Vector} class and has access to all of its data members and methods, e.g, \code{metadata} and \code{elementMetadata}.
It also contains several additional slots:
\describe{
\item{\code{anchor1}:}{An integer vector specifying the index of the first interacting region.}
\item{\code{anchor2}:}{An integer vector specifying the index of the second interacting region.}
\item{\code{regions}:}{A sorted GRanges object containing the coordinates of all interacting regions.}
}
Each interaction is defined by the corresponding entries in the \code{anchor1} and \code{anchor2} slots, which point to the relevant coordinates in \code{regions} for each locus.

The StrictGInteractions class inherits from the GInteractions class and has the same slots.
The only difference is that, for each interaction, \code{anchor1} must be less than or equal to \code{anchor2}.
This means that the first interacting region has a start position that is "lower" than the second interacting region.
This condition is useful for comparing interactions within and between objects, as it ensures that redundant permutations of the same interaction are not being overlooked.
However, it is not used by default as there may conceivably be instances where the order of interactions is informative.
The ReverseStrictGInteractions class has the opposite behaviour, where \code{anchor1} must be greater than or equal to \code{anchor2}.
}

\section{Class construction}{
GInteractions objects can be constructed by specifying integer vectors to define the pairwise interactions in the \code{GInteractions} call.
For entry \code{x}, the corresponding interaction is defined between \code{regions[anchor1[x]]} and \code{regions[anchor2[x]]}.
Obviously, coordinates of all of the interacting loci must be specified in the \code{regions} argument.
Any metadata in \code{regions} will be preserved.
Note that \code{regions} will be resorted in the returned object, so the anchor indices may not be equal to the input \code{anchor1} and \code{anchor2}.

Alternatively, GInteractions objects can be constructed by directly supplying the GRanges of the interacting loci to the \code{GInteractions} function.
If \code{regions} is not specified, this will be constructed automatically from the two sets of supplied GRanges.
If \code{regions} is supplied, exact matching will be performed to identify the indices in \code{regions} corresponding to the regions in the supplied anchor GRanges.
Missing values are not tolerated and will cause an error to be raised.
In both cases, any metadata in the input GRanges will be transferred to the \code{mcols} of the output GInteractions object.

All constructors will return a GInteractions object containing all of the specified information.
Sorting of \code{regions} is performed automatically, with re-indexing of all anchor indices to preserve the correct pairings between regions.
If \code{mode="strict"}, a StrictGInterctions object is returned with anchor indices swapped such that \code{anchor1 <= anchor2} for all interactions.
If \code{mode="reverse"}, a ReverseStrictGInterctions object is returned with anchor indices swapped such that \code{anchor1 >= anchor2}.
If both anchors are missing, the constructor will return an empty GInteractions object.
}

\examples{
set.seed(1000)
N <- 30
all.starts <- sample(1000, N)
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)),
    IRanges(all.starts, all.ends))

Np <- 20
all.anchor1 <- sample(N, Np)
all.anchor2 <- sample(N, Np)
gi <- GInteractions(all.anchor1, all.anchor2, all.regions)

# Equivalent construction:
GInteractions(all.regions[all.anchor1], all.regions[all.anchor2])
GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], 
    all.regions)

# Putting in metadata, elementMetadata
temp.gi <- gi
metadata(temp.gi)$name <- "My GI object"
mcols(temp.gi)$score <- runif(Np)

# Strict construction
sgi <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], 
        all.regions, mode="strict")
rsgi <- GInteractions(all.regions[all.anchor1], all.regions[all.anchor2], 
        all.regions, mode="reverse")
}

\author{
Aaron Lun,
with contributions from Malcolm Perry and Liz Ing-Simmons.
}

\seealso{
\code{?"\link{interaction-access}"},
\code{?"\link{interaction-subset}"},
\code{?"\link{interaction-compare}"}
}

\name{ContactMatrix subsetting}
\alias{ContactMatrix-subset}

\alias{[,ContactMatrix,ANY-method}
\alias{[,ContactMatrix,ANY,ANY-method}
\alias{[,ContactMatrix,ANY,ANY,ANY-method}
\alias{[<-,ContactMatrix,ANY,ANY,ContactMatrix-method}
\alias{subset,ContactMatrix-method}

\alias{cbind,ContactMatrix-method}
\alias{rbind,ContactMatrix-method}

\title{ContactMatrix subsetting and combining}

\description{
Methods to subset or combine ContactMatrix objects.
}

\usage{
### Subsetting

\S4method{[}{ContactMatrix,ANY,ANY}(x, i, j, ..., drop=TRUE)
\S4method{[}{ContactMatrix,ANY,ANY,ContactMatrix}(x, i, j) <- value
\S4method{subset}{ContactMatrix}(x, i, j)

### Combining

\S4method{cbind}{ContactMatrix}(..., deparse.level=1)
\S4method{rbind}{ContactMatrix}(..., deparse.level=1)
}

\arguments{
\item{x}{A \link{ContactMatrix} object.}

\item{i, j}{A vector of subscripts, indicating the rows and columns to be subsetted for \code{i} and \code{j}, respectively.}

\item{...}{    
    For \code{cbind}, \code{rbind} and \code{c}, \code{...} contains \link{ContactMatrix} objects to be combined.
    Otherwise, this argument is ignored during subsetting.
}

\item{drop}{
    A logical scalar, ignored by \code{[,ContactMatrix,ANY,ANY-method}.
}

\item{value}{
    A ContactMatrix object with dimensions equal to the length of the two subscripts (or the corresponding dimensions of \code{x}, if either subscript is missing.
}

\item{deparse.level}{An integer scalar; see \code{?base::\link[base]{cbind}} for a description of this argument.}
}

\value{
A subsetted or combined ContactMatrix object.
}

\details{
Subsetting of ContactMatrix objects behaves like that for standard matrices.
Rows and columns can be selected and rearranged, with concomitant changes to the \code{anchor1} and \code{anchor2} slots.
All subsetting operations will return an ContactMatrix with the specified rows and columns.
However, note that the value of \code{regions} will not be modified by subsetting.

\code{cbind} will combines objects with the same rows but different columns.
Errors will occur if the row regions are not identical between objects (i.e., must have same values in the slots for \code{regions} and \code{anchor1}).
Conversely, \code{rbind} will combines objects with the same columns but different rows.
Again, errors will occur if the columns are not identical (i.e., must have same values in the slots for \code{regions} and \code{anchor2}).

If subsetted anchors are required, see \code{?"\link{interaction-subset}"} for why subsetting should be done before calling the \code{\link{anchors}} method.
}

\examples{
example(ContactMatrix, echo=FALSE)

# Subsetting:
x[1:5,]
x[,10:15]
x[1:5,10:15]

x2 <- x
x2[1:5,] <- x[6:10,]
as.matrix(x2[,1]) <- 20

# Combining
cbind(x, x)
rbind(x, x)
}

\author{
Aaron Lun
}

\name{ContactMatrix distances}
\alias{ContactMatrix-dist}

\alias{pairdist,ContactMatrix-method}
\alias{intrachr,ContactMatrix-method}

\title{Compute linear distances from ContactMatrix objects}

\description{
Methods to compute linear distances between pairs of interacting regions in a ContactMatrix object.
}

\usage{
\S4method{pairdist}{ContactMatrix}(x, type="mid")

\S4method{intrachr}{ContactMatrix}(x)
}

\arguments{
\item{x}{A \link{ContactMatrix} object.}

\item{type}{
    A character string specifying the type of distance to compute.
    See \code{?"\link{pairdist,GInteractions-method}"} for possible values.
}
}

\value{
An integer or logical matrix of the same dimensions as \code{x}, containing the specified distances.
}

\details{
\code{pairdist,ContactMatrix-method} will return a matrix of integer (or, if \code{type="intra"}, logical) values.
Each entry of this matrix specifies the distance between the interacting loci that are represented by the corresponding row and column.
If \code{type="intra"}, each entry indicates whether the corresponding interaction is intra-chromosomal.
Running \code{intrachr(x)} is equivalent to \code{pairdist(x, type="intra")} for any ContactMatrix object \code{x}.
See \code{\link{pairdist,InteractionSet-method}} for more details on the type of distances that can be computed.
}

\examples{
example(ContactMatrix, echo=FALSE)

pairdist(x)
pairdist(x, type="gap")
pairdist(x, type="span")
pairdist(x, type="diag")

intrachr(x)
}

\author{
Aaron Lun
}

\name{ContactMatrix class}
\docType{class}

\alias{ContactMatrix-class}
\alias{ContactMatrix}
\alias{ContactMatrix,ANY,numeric,numeric,GRanges-method}
\alias{ContactMatrix,ANY,GRanges,GRanges,GenomicRanges_OR_missing-method}
\alias{ContactMatrix,missing,missing,missing,GenomicRanges_OR_missing-method}

\title{ContactMatrix class and constructors}

\description{
The ContactMatrix class contains a matrix where rows and columns represent genomic loci.
Each entry of the matrix contains information about the interaction between the loci represented by the corresponding row/column, e.g., contact frequencies.
Coordinates of the loci are also contained within this class.
}

\usage{
\S4method{ContactMatrix}{ANY,numeric,numeric,GRanges}(matrix, anchor1, anchor2, regions, metadata=list())

\S4method{ContactMatrix}{ANY,GRanges,GRanges,GenomicRanges_OR_missing}(matrix, anchor1, anchor2, regions, metadata=list())

\S4method{ContactMatrix}{missing,missing,missing,GenomicRanges_OR_missing}(matrix, anchor1, anchor2, regions, metadata=list())
}

\arguments{
\item{matrix}{
    Any matrix-like object containing interaction data.
}

\item{anchor1, anchor2}{
    Either a pair of numeric vectors containing indices to \code{regions} or a pair of GRanges objects.
    In both cases, \code{anchor1} and \code{anchor2} should have lengths equal to the number of rows and columns in \code{matrix}, respectively.
}

\item{regions}{
    A GRanges object containing the coordinates of the interacting regions.
    This argument is optional for \code{InteractionSet,ANY,GRanges,GRanges-method}.
} 

\item{metadata}{
    A list containing experiment-wide metadata -  see \code{?\link[S4Vectors]{Annotated}} for more details.
}
}

\value{
For the constructors, a ContactMatrix object is returned.
}

\details{
The ContactMatrix class inherits from the \link[S4Vectors]{Annotated} class, with several additional slots:
\describe{
\item{\code{matrix}:}{A matrix or matrix-like object from, e.g., the \pkg{Matrix} or \pkg{DelayedArray} packages.}
\item{\code{anchor1}:}{An integer vector specifying the index of the first interacting region.}
\item{\code{anchor2}:}{An integer vector specifying the index of the second interacting region.}
\item{\code{regions}:}{A sorted GRanges object containing the coordinates of all interacting regions.}
}
Each entry of \code{anchor1} corresponds to a row in \code{matrix}, while each entry of \code{anchor2} corresponds to a column.
Each entry of \code{matrix} represents an interaction between the corresponding entries in \code{anchor1} and \code{anchor2}, i
    which point to the relevant coordinates in \code{regions} for each locus.

ContactMatrix objects can be constructed by specifying numeric vectors as \code{anchor1} and \code{anchor2} in the \code{ContactMatrix} function.
These vectors will define the regions corresponding to the rows and columns of the matrix.
Specifically, each value of the vector acts as an index to specify the relevant coordinates from \code{regions}.
This means that the range of entries must lie within \code{[1, length(regions)]}.

Alternatively, ContactMatrix objects can be constructed by directly supplying the GRanges of the interacting loci in \code{ContactMatrix}.
If \code{regions} is not specified, this will be constructed automatically from the two sets of supplied GRanges.
If \code{regions} is supplied, exact matching will be performed to identify the indices in \code{regions} corresponding to the regions in the supplied GRanges.
Missing values are not tolerated and will cause an error to be raised.

Both methods will return an ContactMatrix object containing all of the specified information.
Sorting of \code{regions} is also performed automatically, with re-indexing of all anchor indices to preserve the correct pairings between regions.
}

\section{Choosing between matrix classes}{
The ContactMatrix class provides support for any matrix-like object that implements \code{dim}, \code{rbind}, \code{cbind}, \code{[} and \code{t} methods.
The choice of class depends on the type of data and the intended application.
Some of the common choices are described in more detail here:
\itemize{
\item Base matrices are simple to generate and are most efficient for dense data. 
This is often sufficient for most use cases where small regions of the interaction space are being examined.
\item Sparse matrices from the \pkg{Matrix} package are useful for large areas of the interaction space where most entries are zero.
This reduces memory usage compared to a dense representation (though conversely, is less efficient for dense data).
Note that all numeric values are coerced to double-precision types, which may take up more memory than a direct integer representation.
Another issue is how missing values should be interpreted in the sparseMatrix -- see \code{?\link{inflate}} for more details.
\item Packed symmetric matrices from the \pkg{Matrix} package provide some memory savings for symmetric regions of the interaction space.
\item Delayed or HDF5-backed matrices from the \pkg{DelayedArray} and \pkg{HDF5Array} packages allow very large matrices to be represented without loading into memory.
}
}

\examples{
set.seed(1000)
N <- 30
all.starts <- sample(1000, N)
all.ends <- all.starts + round(runif(N, 5, 20))
all.regions <- GRanges(rep(c("chrA", "chrB"), c(N-10, 10)),
    IRanges(all.starts, all.ends))

Nr <- 10
Nc <- 20
all.anchor1 <- sample(N, Nr)
all.anchor2 <- sample(N, Nc)
counts <- matrix(rpois(Nr*Nc, lambda=10), Nr, Nc)
x <- ContactMatrix(counts, all.anchor1, all.anchor2, all.regions)

# Equivalent construction:
ContactMatrix(counts, all.regions[all.anchor1], 
    all.regions[all.anchor2])
ContactMatrix(counts, all.regions[all.anchor1], 
    all.regions[all.anchor2], all.regions)

# Also works directly with Matrix objects.
counts2 <- Matrix::Matrix(counts)
x2 <- ContactMatrix(counts2, all.anchor1, all.anchor2, all.regions)
counts2 <- as(counts2, "dgCMatrix")
x2 <- ContactMatrix(counts2, all.anchor1, all.anchor2, all.regions)
}

\author{
Aaron Lun
}

\seealso{
\code{?"\link{ContactMatrix-access}"},
\code{?"\link{ContactMatrix-subset}"},
\code{?"\link{ContactMatrix-sort}"}
}

\name{Interaction overlaps}
\alias{Interaction-overlaps}

\alias{findOverlaps}
\alias{findOverlaps,GInteractions,Vector-method}
\alias{findOverlaps,Vector,GInteractions-method}
\alias{findOverlaps,GInteractions,GInteractions-method}
\alias{findOverlaps,GInteractions,missing-method}

\alias{countOverlaps}
\alias{countOverlaps,GInteractions,Vector-method}
\alias{countOverlaps,Vector,GInteractions-method}
\alias{countOverlaps,GInteractions,GInteractions-method}
\alias{countOverlaps,GInteractions,missing-method}

\alias{overlapsAny}
\alias{overlapsAny,GInteractions,Vector-method}
\alias{overlapsAny,Vector,GInteractions-method}
\alias{overlapsAny,GInteractions,GInteractions-method}
\alias{overlapsAny,GInteractions,missing-method}

\alias{subsetByOverlaps}
\alias{subsetByOverlaps,GInteractions,Vector-method}
\alias{subsetByOverlaps,Vector,GInteractions-method}
\alias{subsetByOverlaps,GInteractions,GInteractions-method}

\alias{findOverlaps,InteractionSet,Vector-method}
\alias{findOverlaps,Vector,InteractionSet-method}
\alias{findOverlaps,InteractionSet,InteractionSet-method}
\alias{findOverlaps,InteractionSet,missing-method}

\alias{countOverlaps,InteractionSet,Vector-method}
\alias{countOverlaps,Vector,InteractionSet-method}
\alias{countOverlaps,InteractionSet,InteractionSet-method}
\alias{countOverlaps,InteractionSet,missing-method}

\alias{overlapsAny,InteractionSet,Vector-method}
\alias{overlapsAny,Vector,InteractionSet-method}
\alias{overlapsAny,InteractionSet,InteractionSet-method}
\alias{overlapsAny,InteractionSet,missing-method}

\alias{subsetByOverlaps,InteractionSet,Vector-method}
\alias{subsetByOverlaps,Vector,InteractionSet-method}
\alias{subsetByOverlaps,InteractionSet,InteractionSet-method}

\alias{findOverlaps,InteractionSet,GInteractions-method}
\alias{findOverlaps,GInteractions,InteractionSet-method}

\alias{countOverlaps,InteractionSet,GInteractions-method}
\alias{countOverlaps,GInteractions,InteractionSet-method}

\alias{overlapsAny,InteractionSet,GInteractions-method}
\alias{overlapsAny,GInteractions,InteractionSet-method}

\alias{subsetByOverlaps,InteractionSet,GInteractions-method}
\alias{subsetByOverlaps,GInteractions,InteractionSet-method}

\title{Find overlaps between interactions in one or two dimensions}

\description{
Find overlaps between interactions and linear intervals, between interactions and pairs of intervals, 
    and between interactions and other interactions in a GInteractions or InteractionSet object.
}

\usage{
\S4method{findOverlaps}{GInteractions,GInteractions}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    select=c("all", "first", "last", "arbitrary"),
    ignore.strand=TRUE, ..., use.region="both")

\S4method{overlapsAny}{GInteractions,GInteractions}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    ignore.strand=TRUE, ..., use.region="both")

\S4method{countOverlaps}{GInteractions,GInteractions}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    ignore.strand=TRUE, ..., use.region="both")

\S4method{subsetByOverlaps}{GInteractions,GInteractions}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    ignore.strand=TRUE, ..., use.region="both")
}

\arguments{
\item{query, subject}{
    A \link[S4Vectors]{Vector}, \link{GInteractions} or \link{InteractionSet} object, depending on the specified method.
    At least one of these must be a GInteractions or InteractionSet object.
    Also, \code{subject} can be missing if \code{query} is a GInteractions or InteractionSet object.
}

\item{maxgap, minoverlap, type, select}{
    See \code{?\link[GenomicRanges]{findOverlaps}} in the \pkg{GenomicRanges} package.
}

\item{ignore.strand}{
A logical scalar indicating whether strand information in \code{query} or \code{subject} should be ignored.
Note that the default setting here is different to that in \code{\link[GenomicRanges]{findOverlaps}} as genomic interactions are usually unstranded.
}
\item{...}{
Further arguments to pass to \code{\link[GenomicRanges]{findOverlaps}}.
This includes \code{drop.self} and \code{drop.redundant} when \code{subject} is missing.
}
\item{use.region}{
A string specifying the regions to be used to identify overlaps.
}
}

\value{
For \code{findOverlaps}, a Hits object is returned if \code{select="all"}, and an integer vector of subject indices otherwise.

For \code{countOverlaps} and \code{overlapsAny}, an integer or logical vector is returned, respectively.

For \code{subsetByOverlaps}, a subsetted object of the same class as \code{query} is returned.
}

\section{Overview of overlaps for GInteractions}{
All methods can be applied using a GInteractions as either the \code{query} or \code{subject}, and a Vector as the other argument.
In such cases, the Vector is assumed to represent some region on the linear genome (e.g., GRanges) or set of such regions (GRangesList).
An overlap will be defined between the interval and an GInteractions interaction if either anchor region of the latter overlaps the former.
This is considered to be a one-dimensional overlap, i.e., on the linear genome.

The same methods can be applied using two GInteractions objects as the \code{query} and \code{subject}.
In such cases, a two-dimensional overlap will be computed between the anchor regions of the two objects.
An overlap is defined if each anchor region of the first object overlaps at least one anchor region of the second object,
    and each anchor region of the second object overlaps at least one anchor region of the first object, i.e., there are overlapping areas in the two-dimensional interaction space. 
If \code{subject} is missing, overlaps will be computed between interactions in \code{query}.
}

\section{Description of overlap methods}{
When \code{select="all"}, \code{findOverlaps} returns a Hits object containing overlapping pairs of queries and subjects (or more specifically, their indices in the supplied objects 
    - see \code{?\link[GenomicRanges]{findOverlaps}} for more details).
For other values of \code{select}, an integer vector is returned with one entry for each element of \code{query},
    which specifies the index of the chosen (first, last or arbitrary) overlapping feature in \code{subject} for that query. 
Queries with no overlaps at all are assigned \code{NA} values.

For the other methods, \code{countOverlaps} returns an integer vector indicating the number of elements in \code{subject} that were overlapped by each element in \code{query}.
\code{overlapsAny} returns a logical vector indicating which elements in \code{query} were overlapped by at least one element in \code{subject}. 
\code{subsetByOverlaps} returns a subsetted \code{query} containing only those elements overlapped by at least one element in \code{subject}.
}

\section{Choice of regions to define overlaps}{
For one-dimensional overlaps, \code{use.region="both"} by default such that overlaps with either anchor region are considered.
If \code{use.region="first"}, overlaps are only considered between the interval and the first anchor region.
Similarly, if \code{use.region="second"}, only the second anchor region is used.

Equivalent choices are available for two-dimensional overlaps:
\itemize{
\item By default, \code{use.region="both"} such that the order of first/second anchor regions in the query and subject is ignored.
This means that the first anchor region in the query can overlap both the first \emph{or} second anchor regions in the subject.
Similarly, the second anchor region in the query can overlap both the first \emph{or} ssecond anchor regions in the subject. 
\item If \code{use.region="same"}, overlaps are only considered between the first anchor regions for the query and the subject, 
or between the second anchor regions for the query and subject.
Overlaps between the first query region and the second subject region, or the second query region and the first subject region, are ignored.
\item If \code{use.region="reverse"}, overlaps are only considered between the first anchor regions for the query and the second anchor regions for the subject, and vice versa.
Overlaps between the first query/subject regions or between the second query/subject regions are ignored.
}
The latter two options tend only to be useful if the order of first/second regions is informative.
}

\section{Details for InteractionSet}{
Each method can also be applied with InteractionSet objects, and the behaviour is largely the same as that described for GInteractions objects.
For a given InteractionSet object \code{x}, the corresponding method is called on the GInteractions object in the \code{interactions} slot of \code{x}.
The return value is identical to that from calling the method on \code{interactions(x)}, except for \code{subsetByOverlaps} for InteractionSet queries
    (which returns a subsetted InteractionSet object, containing only those rows/interactions overlapping the \code{subject}).
}

\examples{
example(GInteractions, echo=FALSE)

# Making a larger object, for more overlaps.
Np <- 100
N <- length(regions(gi))
all.anchor1 <- sample(N, Np, replace=TRUE)
all.anchor2 <- sample(N, Np, replace=TRUE)
gi <- GInteractions(all.anchor1, all.anchor2, regions(gi))

# GRanges overlaps:
of.interest <- resize(sample(regions(gi), 2), width=1, fix="center")
findOverlaps(of.interest, gi)
findOverlaps(gi, of.interest)
findOverlaps(gi, of.interest, select="first")
overlapsAny(gi, of.interest)
overlapsAny(of.interest, gi)
countOverlaps(gi, of.interest)
countOverlaps(of.interest, gi)
subsetByOverlaps(gi, of.interest)
subsetByOverlaps(of.interest, gi)

# GRangesList overlaps:
pairing <- GRangesList(first=regions(gi)[1:3], second=regions(gi)[4:6], 
    third=regions(gi)[7:10], fourth=regions(gi)[15:17])
findOverlaps(pairing, gi)
findOverlaps(gi, pairing)
findOverlaps(gi, pairing, select="last")
overlapsAny(gi, pairing)
overlapsAny(pairing, gi)
countOverlaps(gi, pairing)
countOverlaps(pairing, gi)
subsetByOverlaps(gi, pairing)
subsetByOverlaps(pairing, gi)

# GInteractions overlaps (split into two):
first.half <- gi[1:(Np/2)]
second.half <- gi[Np/2+1:(Np/2)]
findOverlaps(first.half, second.half)
findOverlaps(first.half, second.half, select="arbitrary")
overlapsAny(first.half, second.half)
countOverlaps(first.half, second.half)
subsetByOverlaps(first.half, second.half)

findOverlaps(gi)
countOverlaps(gi)
overlapsAny(gi) # trivial result

#################
# Same can be done for an InteractionSet object:

Nlibs <- 4
counts <- matrix(rpois(Nlibs*Np, lambda=10), ncol=Nlibs)
colnames(counts) <- seq_len(Nlibs)
iset <- InteractionSet(counts, gi)

findOverlaps(of.interest, iset)
findOverlaps(iset, pairing)
findOverlaps(iset[1:(Np/2),], iset[Np/2+1:(Np/2),])

# Obviously returns InteractionSet objects instead
subsetByOverlaps(of.interest, iset)
subsetByOverlaps(iset, pairing)
subsetByOverlaps(iset[1:(Np/2),], iset[Np/2+1:(Np/2),])

# Self-overlaps
findOverlaps(iset)
countOverlaps(iset)
overlapsAny(iset) # trivial result
}

\author{
Aaron Lun
}

\seealso{
\code{\link[GenomicRanges]{findOverlaps}},
\code{\link{linkOverlaps}}
}

\name{ContactMatrix overlaps}

\alias{overlapsAny,ContactMatrix,Vector-method}
\alias{overlapsAny,ContactMatrix,GInteractions-method}
\alias{overlapsAny,ContactMatrix,InteractionSet-method}

\title{Find overlaps between GRanges and a ContactMatrix}

\description{
Find overlaps for the set of regions representing the rows or columns of a ContactMatrix.
}

\usage{
\S4method{overlapsAny}{ContactMatrix,Vector}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    ignore.strand=TRUE, ...)

\S4method{overlapsAny}{ContactMatrix,GInteractions}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    ignore.strand=TRUE, ..., use.region="both")

\S4method{overlapsAny}{ContactMatrix,InteractionSet}(query, subject, maxgap=-1L, minoverlap=0L,
    type=c("any", "start", "end", "within", "equal"),
    ignore.strand=TRUE, ..., use.region="both")
}

\arguments{
\item{query}{A \link{ContactMatrix} object.}

\item{subject}{A \link[GenomicRanges]{GRanges}, \link[GenomicRanges]{GRangesList}, \link{GInteractions} or \link{InteractionSet} object.}

\item{maxgap, minoverlap, type, ..., ignore.strand}{
    See \code{?\link[GenomicRanges]{findOverlaps}} in the \pkg{GenomicRanges} package.
}

\item{use.region}{
    A string specifying how the interacting regions are to be matched to row/column regions, see Details.
}
}

\value{
For \code{overlapsAny,ContactMatrix,GRanges-method}, a named list of two logical vectors is returned specifying the rows of columns of \code{query} overlapped by \code{subject}.

For the other \code{overlapsAny} methods, a logical matrix is returned indicating which entries in \code{query} are overlapped by \code{subject}.
}

\details{
When \code{subject} is a GRanges, overlaps are identified between the row regions of \code{query} and the regions in \code{subject}.
This is repeated for the column regions of \code{query}.
A list of two logical vectors is returned, specifying the rows and columns in \code{query} that are overlapped by any region in \code{subject}.
These vectors can be directly used to subset \code{query}.
Alternatively, they can be used in \code{outer} to generate a logical matrix for masking -- see Examples.

For all other classes of \code{subject}, two-dimensional overlaps are identified.
A logical matrix is returned indicating which entries in the ContactMatrix have overlaps with the specified interactions.
For any given entry, an overlap is only considered if the regions for the corresponding row and column both overlap anchor regions in the \code{subject}.
See \code{?"\link{findOverlaps,GInteractions,GInteractions-method}"} for more details.

If \code{use.region="both"}, overlaps between any row/column region and the first/second interacting region of \code{subject} are considered.
If \code{use.region="same"}, only overlaps between row regions and the first interacting regions, or between column regions and the second interacting regions are considered.
If \code{use.region="reverse"}, only overlaps between row regions and the second interacting regions, or between row regions and the first interacting regions are considered.
}

\examples{
example(ContactMatrix, echo=FALSE)

of.interest <- resize(sample(regions(x), 2), width=1, fix="center")
olap <- overlapsAny(x, of.interest)
olap
x[olap$row,]
x[,olap$column]
x[olap$row,olap$column]

keep <- outer(olap$row, olap$column, "|") # OR mask
temp <- as.matrix(x)
temp[!keep] <- NA

keep <- outer(olap$row, olap$column, "&") # AND mask
temp <- as.matrix(x)
temp[!keep] <- NA

# Two dimensional overlaps.
pairing <- GRangesList(first=regions(x), second=regions(x))
olap <- overlapsAny(x, pairing)
olap
olap <- overlapsAny(sort(x), pairing) # A bit prettier
olap
}

\author{
Aaron Lun
}

\name{GRanges methods}

\alias{trim,GInteractions-method}
\alias{trim,InteractionSet-method}
\alias{trim,ContactMatrix-method}

\alias{shift,GInteractions-method}
\alias{shift,InteractionSet-method}
\alias{shift,ContactMatrix-method}

\alias{flank,GInteractions-method}
\alias{flank,InteractionSet-method}
\alias{flank,ContactMatrix-method}

\alias{narrow,GInteractions-method}
\alias{narrow,InteractionSet-method}
\alias{narrow,ContactMatrix-method}

\alias{resize,GInteractions-method}
\alias{resize,InteractionSet-method}
\alias{resize,ContactMatrix-method}

\alias{width,GInteractions-method}
\alias{width,InteractionSet-method}
\alias{width,ContactMatrix-method}

\title{Equivalents of GRanges methods}

\description{Methods for GInteractions, InteractionSet and ContactMatrix that operate on the internal GenomicRanges.}

\usage{
\S4method{shift}{GInteractions}(x, shift=0L, use.names=TRUE) 
\S4method{shift}{InteractionSet}(x, shift=0L, use.names=TRUE) 
\S4method{shift}{ContactMatrix}(x, shift=0L, use.names=TRUE) 

\S4method{narrow}{GInteractions}(x, start=NA, end=NA, width=NA, use.names=TRUE)
\S4method{narrow}{InteractionSet}(x, start=NA, end=NA, width=NA, use.names=TRUE)
\S4method{narrow}{ContactMatrix}(x, start=NA, end=NA, width=NA, use.names=TRUE)

\S4method{resize}{GInteractions}(x, width, fix="start", use.names=TRUE, ...)
\S4method{resize}{InteractionSet}(x, width, fix="start", use.names=TRUE, ...)
\S4method{resize}{ContactMatrix}(x, width, fix="start", use.names=TRUE, ...)

\S4method{flank}{GInteractions}(x, width, start=TRUE, both=FALSE, use.names=TRUE, ignore.strand=FALSE)
\S4method{flank}{InteractionSet}(x, width, start=TRUE, both=FALSE, use.names=TRUE, ignore.strand=FALSE)
\S4method{flank}{ContactMatrix}(x, width, start=TRUE, both=FALSE, use.names=TRUE, ignore.strand=FALSE)

\S4method{trim}{GInteractions}(x, use.names=FALSE)
\S4method{trim}{InteractionSet}(x, use.names=FALSE)
\S4method{trim}{ContactMatrix}(x, use.names=FALSE)

\S4method{width}{GInteractions}(x)
\S4method{width}{InteractionSet}(x)
\S4method{width}{ContactMatrix}(x)
}

\arguments{
\item{x}{A \link{GInteractions}, \link{InteractionSet} or \link{ContactMatrix} object.}
\item{shift, start, end, width, fix, both}{Further interaction-specific arguments to pass to the relevant GenomicRanges methods, see details.}
\item{use.names, ignore.strand, ...}{Further overall arguments to pass to the relevant GenomicRanges methods.}
}

\details{
The \code{trim}, \code{resize}, \code{narrow} and \code{shift} methods will apply the GenomicRanges methods with the same name to the anchor regions of \code{x}.
For example, \code{resize(x, width)} will produce an object that is equivalent to running \code{resize} on the first and second anchor regions directly.
This is \emph{not} the same as running the methods on the \code{regions} slot of \code{x}, which is an important distinction when the arguments are vectors.

The interaction-specific arguments can be scalars, vectors or a list of two scalars/vectors.
Scalars and vectors will be recycled across the length of the first and second anchor regions.
For lists, the first element will be applied to the first anchor regions, and the second element will be applied to the second anchor regions.

After any modifications are applied, resorting may be performed to ensure that the entries in the \code{regions} slot of the output object are ordered.
This means that the order of the ranges in the \code{regions} slot may change between the input and output object.
However, the number and order of the \emph{interactions} will not change.

For GInteractions and InteractionSet objects, the \code{width} method will return a list with two integer vectors \code{first} and \code{second} with the same length as \code{x}.
These contain the widths of the first or second anchor regions corresponding to each interaction.
For ContactMatrix objects, the method will return a list with the vectors \code{row} and \code{column}, with lengths equal to the number of rows and column respectively.
}

\value{
Depending on the method, an object of the same class as \code{x}, or information regarding the genomic regions -- see Details.
}

\examples{
example(GInteractions, echo=FALSE)

trim(gi)
width(gi)

# Functions are applied along the length of 'gi':
new.sizes <- round(runif(length(gi), 10, 50))
gi2 <- resize(gi, width=new.sizes)
gi2
resize(first(gi), width=new.sizes)
resize(second(gi), width=new.sizes)

# ... not along the length of 'regions(gi)': note the difference!
mod.regions <- suppressWarnings(resize(regions(gi), width=new.sizes))
mod.regions[anchors(gi, type="first", id=TRUE)] # not the same as first(gi2)!
mod.regions[anchors(gi, type="second", id=TRUE)] # not the same second(gi2)!

example(ContactMatrix, echo=FALSE)
trim(x)
width(x)
}

\author{
Aaron Lun, based on suggestions from Liz Ing-Simmons.
}

\seealso{
\code{?"\link[GenomicRanges]{trim,GenomicRanges-method}"},
\code{?"\link[GenomicRanges]{resize,GenomicRanges-method}"},
\code{?"\link[GenomicRanges]{narrow,GenomicRanges-method}"},
\code{?"\link[GenomicRanges]{shift,GenomicRanges-method}"},
\code{?"\link[GenomicRanges]{flank,GenomicRanges-method}"},
\code{?"\link[GenomicRanges]{width,GenomicRanges-method}"}
}
